/**
 *******************************************************************************
 * @file    tmpm440_adc.h
 * @brief   This file provides all the functions prototypes for ADC driver.
 * @version V2.0.2.4
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __TMPM440_ADC_H
#define __TMPM440_ADC_H

#ifdef __cplusplus
extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "TMPM440.h"
#include "tx04_common.h"
#ifdef __IAR_SYSTEMS_ICC__
#if 1 != __LITTLE_ENDIAN__
#pragma bitfields=disjoint_types
#endif
#endif

/* #define LITTLE_ENDIAN */

#if defined(__TMPM440_ADC_H)

/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @addtogroup ADC
  * @{
  */

/** @defgroup ADC_Exported_Types
  * @{
  */

#define ADC_CONVERSION_CLK_10           ((uint8_t)0x00)
#define ADC_CONVERSION_CLK_20           ((uint8_t)0x10)
#define ADC_CONVERSION_CLK_30           ((uint8_t)0x20)
#define ADC_CONVERSION_CLK_40           ((uint8_t)0x30)
#define ADC_CONVERSION_CLK_80           ((uint8_t)0x40)
#define ADC_CONVERSION_CLK_160          ((uint8_t)0x50)
#define ADC_CONVERSION_CLK_320          ((uint8_t)0x60)
#define IS_ADC_HOLD_TIME(param)         (((param) == ADC_CONVERSION_CLK_10) || \
                                         ((param) == ADC_CONVERSION_CLK_20) || \
                                         ((param) == ADC_CONVERSION_CLK_30) || \
                                         ((param) == ADC_CONVERSION_CLK_40) || \
                                         ((param) == ADC_CONVERSION_CLK_80) || \
                                         ((param) == ADC_CONVERSION_CLK_160)|| \
                                         ((param) == ADC_CONVERSION_CLK_320))

#define ADC_FC_DIVIDE_LEVEL_2           ((uint8_t)0x01)
#define ADC_FC_DIVIDE_LEVEL_4           ((uint8_t)0x02)
#define ADC_FC_DIVIDE_LEVEL_8           ((uint8_t)0x03)
#define ADC_FC_DIVIDE_LEVEL_16          ((uint8_t)0x04)
#define IS_ADC_PRESCALER(param)         (((param) == ADC_FC_DIVIDE_LEVEL_2) || \
                                         ((param) == ADC_FC_DIVIDE_LEVEL_4) || \
                                         ((param) == ADC_FC_DIVIDE_LEVEL_8) || \
                                         ((param) == ADC_FC_DIVIDE_LEVEL_16))

#define ADC_INT_SINGLE                  ((uint8_t)0x00)
#define ADC_INT_CONVERSION_2            ((uint8_t)0x10)
#define ADC_INT_CONVERSION_3            ((uint8_t)0x20)
#define ADC_INT_CONVERSION_4            ((uint8_t)0x30)
#define ADC_INT_CONVERSION_5            ((uint8_t)0x40)
#define ADC_INT_CONVERSION_6            ((uint8_t)0x50)
#define ADC_INT_CONVERSION_7            ((uint8_t)0x60)
#define ADC_INT_CONVERSION_8            ((uint8_t)0x70)
#define IS_ADC_INT_MODE(param)          (((param) == ADC_INT_SINGLE) || \
                                         ((param) == ADC_INT_CONVERSION_2) || \
                                         ((param) == ADC_INT_CONVERSION_3) || \
                                         ((param) == ADC_INT_CONVERSION_4) || \
                                         ((param) == ADC_INT_CONVERSION_5) || \
                                         ((param) == ADC_INT_CONVERSION_6) || \
                                         ((param) == ADC_INT_CONVERSION_7) || \
                                         ((param) == ADC_INT_CONVERSION_8))

#define IS_ADC_UNIT(param)              (((param) == TSB_ADA) || ((param) == TSB_ADB) || \
                                         ((param) == TSB_ADC))

#define ADC_AN_00                       ((uint8_t)0x00)
#define ADC_AN_01                       ((uint8_t)0x01)
#define ADC_AN_02                       ((uint8_t)0x02)
#define ADC_AN_03                       ((uint8_t)0x03)
#define ADC_AN_04                       ((uint8_t)0x04)
#define ADC_AN_05                       ((uint8_t)0x05)
#define ADC_AN_06                       ((uint8_t)0x06)
#define ADC_AN_07                       ((uint8_t)0x07)

#define IS_UNIT_AB_INPUT_CHANNEL(param)  ((param) <= ADC_AN_07)
#define IS_UNIT_C_INPUT_CHANNEL(param)   ((param) <= ADC_AN_03)

#define IS_UNIT_AB_SCAN_CHANNEL(start, range)       (((start) <= ADC_AN_07) && \
                                                     ((range) >= 1U) && \
                                                     (((start) + (range)) <= 8U))

#define IS_UNIT_C_SCAN_CHANNEL(start, range)       (((start) <= ADC_AN_03) && \
                                                    ((range) >= 1U) && \
                                                    (((start) + (range)) <= 4U))

#define ADC_REG_00                      ((uint8_t)0x00)
#define ADC_REG_01                      ((uint8_t)0x01)
#define ADC_REG_02                      ((uint8_t)0x02)
#define ADC_REG_03                      ((uint8_t)0x03)
#define ADC_REG_04                      ((uint8_t)0x04)
#define ADC_REG_05                      ((uint8_t)0x05)
#define ADC_REG_06                      ((uint8_t)0x06)
#define ADC_REG_07                      ((uint8_t)0x07)
#define ADC_REG_SP                      ((uint8_t)0x08)
#define IS_ADC_REG(param)               ((param) <= ADC_REG_SP)

#define ADC_EXT_TRG                     ((uint8_t)0x00)
#define ADC_MATCH_TMRB_NORMAL                  ((uint8_t)0x02)
#define IS_ADC_HW_TRG_NORMAL(param)     (((param) == ADC_EXT_TRG) || \
                                         ((param) == ADC_MATCH_TMRB_NORMAL))

#define ADC_MATCH_TMRB_TOP                  ((uint8_t)0x08)
#define IS_ADC_HW_TRG_TOP(param)        (((param) == ADC_EXT_TRG) || \
                                         ((param) == ADC_MATCH_TMRB_TOP))

#define ADC_APPLY_VREF_IN_CONVERSION    ((uint8_t)0x00)
#define ADC_APPLY_VREF_AT_ANY_TIME      ((uint8_t)0x01)
#define IS_ADC_VREF_CTRL(param)         (((param) == ADC_APPLY_VREF_IN_CONVERSION) || \
                                         ((param) == ADC_APPLY_VREF_AT_ANY_TIME))

    typedef enum {
        ADC_CMPCR_0 = 0U,
        ADC_CMPCR_1 = 1U
    } ADC_CMPCRx;
#define IS_ADC_CMPCRx(param)            ((param) <= ADC_CMPCR_1)

    typedef enum {
        ADC_LARGER_THAN_CMP_REG = 0U,
        ADC_SMALLER_THAN_CMP_REG = 1U
    } ADC_CmpCondition;
#define IS_ADC_CMPCONDITION(param)      ((param) <= ADC_SMALLER_THAN_CMP_REG)

    typedef enum {
        ADC_SEQUENCE_CMP_MODE = 0U,
        ADC_CUMULATION_CMP_MODE = 1U
    } ADC_CmpCntMode;
#define IS_ADC_CMPMODE(param)           ((param) <= ADC_CUMULATION_CMP_MODE)

/**
  * @brief  ADC Monitor Configuration Structure definition
  */
    typedef struct {
        uint8_t CmpChannel;     /*!< Select which ADC channel among ADC_AN_00 to ADC_AN_07 will be used */
        uint32_t CmpCnt;        /*!< How many times will valid comparisons be counted, range from 1 to 16 */
        ADC_CmpCondition Condition;     /*!< Conditon to compare AINx with ADCMPy. (x = 00 to 07, y = 0 to 1) */
        ADC_CmpCntMode CntMode; /*!< Mode to compare AINx with ADCMPy. (x = 00 to 07, y = 0 to 1) */
        uint32_t CmpValue;      /*!< Comparison value to be set in ADCMP0 or ADCMP1, max value is 4095 */
    } ADC_MonitorTypeDef;

#define IS_ADC_CMPCNT(param)            (((param) >= 1U) && ((param) <= 16U))
#define IS_ADC_CMPVALUE_12BIT(param)    ((param) <= 4095U)

/**
  * @brief Union to store ADC state and store ADC result
  */
#if defined ( __CC_ARM )
#ifdef __BIG_ENDIAN
    typedef union {             /* big endian */
        uint32_t All;
        struct {
            uint32_t Reserved2:28;      /*!< bit4 to bit 31, reserved */
            uint32_t TopComplete:1;     /*!< bit3, top-priority AD conversion complete flag */
            uint32_t Reserved1:1;       /*!< bit2, reserved */
            uint32_t NormalComplete:1;  /*!< bit1, normal AD conversion complete flag */
            uint32_t Reserved0:1;       /*!< bit0, reserved */
        } Bit;
    } ADC_State;

    typedef union {             /* big endian */
        uint32_t All;
        struct {
            uint32_t ADResult_MR:12;    /*!< bit20 to bit31, store AD result */
            uint32_t Reserved1:2;       /*!< bit18 to bit19, reserved */
            uint32_t OverRun_MR:1;      /*!< bit17, new AD result is stored before the old one is read */
            uint32_t Stored_MR:1;       /*!< bit16, AD result has been stored */

            uint32_t Reserved0:2;       /*!< bit14 to bit15, reserved */
            uint32_t OverRun:1; /*!< bit13, new AD result is stored before the old one is read */
            uint32_t Stored:1;  /*!< bit12, AD result has been stored */
            uint32_t ADResult:12;       /*!< bit0 to bit11, store AD result */

        } Bit;
    } ADC_Result;
#else
    typedef union {             /* little endian */
        uint32_t All;
        struct {
            uint32_t Reserved0:1;       /*!< bit0, reserved */
            uint32_t NormalComplete:1;  /*!< bit1, normal AD conversion complete flag */
            uint32_t Reserved1:1;       /*!< bit2, reserved */
            uint32_t TopComplete:1;     /*!< bit3, top-priority AD conversion complete flag */
            uint32_t Reserved2:28;      /*!< bit4 to bit 31, reserved */
        } Bit;
    } ADC_State;

    typedef union {             /* little endian */
        uint32_t All;
        struct {
            uint32_t ADResult:12;       /*!< bit0 to bit11, store AD result */
            uint32_t Stored:1;  /*!< bit12, AD result has been stored */
            uint32_t OverRun:1; /*!< bit13, new AD result is stored before the old one is read */
            uint32_t Reserved0:2;       /*!< bit14 to bit15, reserved */

            uint32_t Stored_MR:1;       /*!< bit16, AD result has been stored */
            uint32_t OverRun_MR:1;      /*!< bit17, new AD result is stored before the old one is read */
            uint32_t Reserved1:2;       /*!< bit18 to bit19, reserved */
            uint32_t ADResult_MR:12;    /*!< bit20 to bit31, store AD result */

        } Bit;
    } ADC_Result;

#endif
#elif  defined(__IAR_SYSTEMS_ICC__)
    typedef union {             /* little endian */
        uint32_t All;
        struct {
            uint32_t Reserved0:1;       /*!< bit0, reserved */
            uint32_t NormalComplete:1;  /*!< bit1, normal AD conversion complete flag */
            uint32_t Reserved1:1;       /*!< bit2, reserved */
            uint32_t TopComplete:1;     /*!< bit3, top-priority AD conversion complete flag */
            uint32_t Reserved2:28;      /*!< bit4 to bit 31, reserved */
        } Bit;
    } ADC_State;

    typedef union {             /* little endian */
        uint32_t All;
        struct {
            uint32_t ADResult:12;       /*!< bit0 to bit11, store AD result */
            uint32_t Stored:1;  /*!< bit12, AD result has been stored */
            uint32_t OverRun:1; /*!< bit13, new AD result is stored before the old one is read */
            uint32_t Reserved0:2;       /*!< bit14 to bit15, reserved */

            uint32_t Stored_MR:1;       /*!< bit16, AD result has been stored */
            uint32_t OverRun_MR:1;      /*!< bit17, new AD result is stored before the old one is read */
            uint32_t Reserved1:2;       /*!< bit18 to bit19, reserved */
            uint32_t ADResult_MR:12;    /*!< bit20 to bit31, store AD result */

        } Bit;
    } ADC_Result;
#endif

#define ADC_DMA_REQ_NORMAL              ((uint8_t)0x00)
#define ADC_DMA_REQ_TOP                 ((uint8_t)0x01)
#define IS_ADC_DMA_REQ(param)           ((param) <= ADC_DMA_REQ_TOP)

/** @} */
/* End of group ADC_Exported_Types */

/** @defgroup ADC_Exported_FunctionPrototypes
  * @{
  */
    void ADC_SWReset(TSB_AD_TypeDef * ADx);
    void ADC_SetClk(TSB_AD_TypeDef * ADx, uint32_t Sample_HoldTime, uint32_t Prescaler_Output);
    void ADC_Start(TSB_AD_TypeDef * ADx);
    void ADC_SetScanMode(TSB_AD_TypeDef * ADx, FunctionalState NewState);
    void ADC_SetRepeatMode(TSB_AD_TypeDef * ADx, FunctionalState NewState);
    void ADC_SetINTMode(TSB_AD_TypeDef * ADx, uint8_t INTMode);
    void ADC_SetInputChannel(TSB_AD_TypeDef * ADx, uint8_t InputChannel);
    void ADC_SetScanChannel(TSB_AD_TypeDef * ADx, uint8_t StartChannel, uint8_t Range);
    void ADC_SetVrefCut(TSB_AD_TypeDef * ADx, uint8_t VrefCtrl);
    void ADC_SetIdleMode(TSB_AD_TypeDef * ADx, FunctionalState NewState);
    void ADC_SetVref(TSB_AD_TypeDef * ADx, FunctionalState NewState);
    void ADC_SetInputChannelTop(TSB_AD_TypeDef * ADx, uint8_t TopInputChannel);
    void ADC_StartTopConvert(TSB_AD_TypeDef * ADx);
    void ADC_SetMonitor(TSB_AD_TypeDef * ADx, ADC_CMPCRx ADCMPx, FunctionalState NewState);
    void ADC_ConfigMonitor(TSB_AD_TypeDef * ADx, ADC_CMPCRx ADCMPx, ADC_MonitorTypeDef * Monitor);
    void ADC_SetHWTrg(TSB_AD_TypeDef * ADx, uint8_t HwSource, FunctionalState NewState);
    void ADC_SetHWTrgTop(TSB_AD_TypeDef * ADx, uint8_t HwSource, FunctionalState NewState);
    ADC_State ADC_GetConvertState(TSB_AD_TypeDef * ADx);
    ADC_Result ADC_GetConvertResult(TSB_AD_TypeDef * ADx, uint8_t ADREGx);
    void ADC_SetDMAReq(TSB_AD_TypeDef * ADx, uint8_t DMAReq, FunctionalState NewState);


/** @} */
/* End of group ADC_Exported_FunctionPrototypes */

/** @} */
/* End of group ADC */

/** @} */
/* End of group TX04_Periph_Driver */

#endif                          /* defined(__TMPM440_ADC_H) */

#ifdef __cplusplus
}
#endif
#endif                          /* __TMPM440_ADC_H */
