/**
 *******************************************************************************
 * @file    tmpm440_dmac.h
 * @brief   This file provides all the functions prototypes for DMAC driver.
 * @version V2.0.2.4
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __TMPM440_DMAC_H
#define __TMPM440_DMAC_H


#ifdef __cplusplus
extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "TMPM440.h"
#include "tx04_common.h"

#ifdef __IAR_SYSTEMS_ICC__
#if 1 != __LITTLE_ENDIAN__
#pragma bitfields=disjoint_types
#endif
#endif

#if defined(__TMPM440_DMAC_H)
/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @addtogroup DMAC
  * @{
  */

/** @defgroup DMAC_Exported_Constants
  * @{
  */

#define IS_DMAC_TX_SIZE(param)   ((param) <= 0x0FFFU)
#define IS_DMAC_LINK_ADDR_MAX_VALUE(param)   ((param) <= 0xFFFFFFF0U)

#define DMAC_MEMORY_TO_MEMORY         ((uint32_t)0x00000000)
#define DMAC_MEMORY_TO_PERIPH         ((uint32_t)0x00000800)
#define DMAC_PERIPH_TO_MEMORY         ((uint32_t)0x00001000)
#define DMAC_PERIPH_TO_PERIPH         ((uint32_t)0x00001800)
#define IS_DMAC_TX_DIRECTION(param) (((param) == DMAC_MEMORY_TO_MEMORY) || \
                                     ((param) == DMAC_MEMORY_TO_PERIPH) || \
                                     ((param) == DMAC_PERIPH_TO_PERIPH) || \
                                     ((param) == DMAC_PERIPH_TO_MEMORY))

/** @} */
/* End of group DMAC_Exported_Constants */

/** @defgroup DMAC_Exported_Types
  * @{
  */
    typedef enum {
        DMAC_CHANNEL_0 = 0U,
        DMAC_CHANNEL_1 = 1U
    } DMAC_Channel;
#define IS_DMAC_CHANNEL(param) (((param) == DMAC_CHANNEL_0) || \
                                ((param) == DMAC_CHANNEL_1))

#define DMAC_UNIT_A  TSB_DMACA
#define DMAC_UNIT_B  TSB_DMACB
#define DMAC_UNIT_C  TSB_DMACC

#define IS_DMAC_UNIT(param) (((param) == DMAC_UNIT_A)|| \
                             ((param) == DMAC_UNIT_B)|| \
                             ((param) == DMAC_UNIT_C))
#define IS_DMAC_SINGLE_REQ_UNIT(param) (((param) == DMAC_UNIT_B)|| \
                             ((param) == DMAC_UNIT_C))

    typedef enum {
        DMAC_LITTLE_ENDIAN = 0U,
        DMAC_BIG_ENDIAN = 1U
    } DMAC_Endian_Mode;
#define IS_DMAC_ENDIAN(param)   (((param) == DMAC_BIG_ENDIAN) || \
                                 ((param) == DMAC_LITTLE_ENDIAN))

    typedef enum {
        DMAC_BYTE = 0U,
        DMAC_HALF_WORD = 1U,
        DMAC_WORD = 2U
    } DMAC_BitWidth;
#define IS_DMAC_BIT_WIDTH(param) (((param) == DMAC_BYTE) || \
                                  ((param) == DMAC_HALF_WORD) || \
                                  ((param) == DMAC_WORD))

    typedef enum {
        DMAC_TX_NO_REQ = 0U,
        DMAC_TX_END_REQ = 1U,
        DMAC_TX_ERR_REQ = 2U,
        DMAC_TX_REQS = 3U
    } DMAC_TxINTReq;

#if defined ( __CC_ARM )
#ifdef __BIG_ENDIAN
    typedef union {
        uint32_t All;
        struct {
            uint32_t Reserved:30;
            uint32_t CH1_INTReq:1;
            uint32_t CH0_INTReq:1;
        } Bit;
    } DMAC_INTReq;
#else
    typedef union {
        uint32_t All;
        struct {
            uint32_t CH0_INTReq:1;
            uint32_t CH1_INTReq:1;
            uint32_t Reserved:30;
        } Bit;
    } DMAC_INTReq;;
#endif
#elif  defined(__IAR_SYSTEMS_ICC__)
    typedef union {
        uint32_t All;
        struct {
            uint32_t CH0_INTReq:1;
            uint32_t CH1_INTReq:1;
            uint32_t Reserved:30;
        } Bit;
    } DMAC_INTReq;
#endif

    typedef enum {
        DMAC_INT_TX_END = 0U,
        DMAC_INT_TX_ERR = 1U
    } DMAC_INTSrc;
#define IS_DMAC_INT_SRC(param)  (((param) == DMAC_INT_TX_END) || \
                                 ((param) == DMAC_INT_TX_ERR))

    typedef enum {
        DMAC_1_BEAT = 0U,
        DMAC_4_BEATS = 1U,
        DMAC_8_BEATS = 2U,
        DMAC_16_BEATS = 3U,
        DMAC_32_BEATS = 4U,
        DMAC_64_BEATS = 5U,
        DMAC_128_BEATS = 6U,
        DMAC_256_BEATS = 7U
    } DMAC_BurstSize;
#define IS_DMAC_BURST_SIZE(param)  (((param) == DMAC_1_BEAT) || \
                                    ((param) == DMAC_4_BEATS) || \
                                    ((param) == DMAC_8_BEATS) || \
                                    ((param) == DMAC_16_BEATS) || \
                                    ((param) == DMAC_32_BEATS) || \
                                    ((param) == DMAC_64_BEATS) || \
                                    ((param) == DMAC_128_BEATS) || \
                                    ((param) == DMAC_256_BEATS))

    typedef enum {
        DMACA_ESIO0_RX = 0U,
        DMACA_ESIO0_TX = 1U,
        DMACA_NORMAL_UNITA_ADC = 2U,
        DMACA_SIO3_UART3_RX = 3U,
        DMACA_SIO3_UART3_TX = 4U,
        DMACA_SIO0_UART0_RX = 5U,
        DMACA_SIO0_UART0_TX = 6U,
        DMACA_TMRB00_CMP_MATCH = 7U,
        DMACA_TMRB04_CMP_MATCH = 8U,
        DMACA_TMRB10_CMP_MATCH = 9U,
        DMACA_TMRB14_CMP_MATCH = 10U,
        DMACA_TMRC_CMP0_MATCH = 11U,
        DMACA_TMRC_CMP1_MATCH = 12U,
        DMACA_HIGHEST_UNITA_ADC = 13U,
        DMACA_PHCNT0_CMP0_MATCH = 14U,
        DMACA_PIN = 15U
    } DMACA_ReqNum;

    typedef enum {
        DMACB_ESIO1_RX = 0U,
        DMACB_ESIO1_TX = 1U,
        DMACB_NORMAL_UNITB_ADC = 2U,
        DMACB_SIO4_UART4_RX = 3U,
        DMACB_SIO4_UART4_TX = 4U,
        DMACB_SIO1_UART1_RX = 5U,
        DMACB_SIO1_UART1_TX = 6U,
        DMACB_UART0_RX = 7U,
        DMACB_UART0_TX = 8U,
        DMACB_TMRB08_CAPTURE0 = 9U,
        DMACB_TMRC0_CAPTURE0 = 10U,
        DMACB_TMRC0_CAPTURE1 = 11U,
        DMACB_HIGHEST_UNITB_ADC = 12U,
        DMACB_PCNT1_CMP0_MATCH = 13U,
        DMACB_TMRD0_CMP_MATCH = 14U,
        DMACB_PIN = 15U
    } DMACB_ReqNum;

    typedef enum {
        DMACC_ESIO2_RX = 0U,
        DMACC_ESIO2_TX = 1U,
        DMACC_NORMAL_UNITC_ADC = 2U,
        DMACC_SIO5_UART5_RX = 3U,
        DMACC_SIO5_UART5_TX = 4U,
        DMACC_SIO2_UART2_RX = 5U,
        DMACC_SIO2_UART2_TX = 6U,
        DMACC_UART1_RX = 7U,
        DMACC_UART1_TX = 8U,
        DMACC_TMRB19_CAPTURE0 = 9U,
        DMACC_TMRC0_CAPTURE2 = 10U,
        DMACC_TMRC0_CAPTURE3 = 11U,
        DMACC_HIGHEST_UNITC_ADC = 12U,
        DMACC_PHCP_CYCLE0 = 13U,
        DMACC_TMRD10_CMP_MATCH = 14U,
        DMACC_PIN = 15U
    } DMACC_ReqNum;

#define IS_DMACA_REQ_NUM(param)  (((param) == DMACA_ESIO0_RX) || \
                                   ((param) == DMACA_ESIO0_TX) || \
                                   ((param) == DMACA_NORMAL_UNITA_ADC) || \
                                   ((param) == DMACA_SIO3_UART3_RX) || \
                                   ((param) == DMACA_SIO3_UART3_TX) || \
                                   ((param) == DMACA_SIO0_UART0_RX) || \
                                   ((param) == DMACA_SIO0_UART0_TX) || \
                                   ((param) == DMACA_TMRB00_CMP_MATCH) || \
                                   ((param) == DMACA_TMRB04_CMP_MATCH) || \
                                   ((param) == DMACA_TMRB10_CMP_MATCH) || \
                                   ((param) == DMACA_TMRB14_CMP_MATCH) || \
                                   ((param) == DMACA_TMRC_CMP0_MATCH) || \
                                   ((param) == DMACA_TMRC_CMP1_MATCH) || \
                                   ((param) == DMACA_HIGHEST_UNITA_ADC) || \
                                   ((param) == DMACA_PHCNT0_CMP0_MATCH ) || \
                                   ((param) == DMACA_PIN))

#define IS_DMACB_REQ_NUM(param)  (((param) == DMACB_ESIO1_RX) || \
                                   ((param) == DMACB_ESIO1_TX) || \
                                   ((param) == DMACB_NORMAL_UNITB_ADC) || \
                                   ((param) == DMACB_SIO4_UART4_RX) || \
                                   ((param) == DMACB_SIO4_UART4_TX) || \
                                   ((param) == DMACB_SIO1_UART1_RX) || \
                                   ((param) == DMACB_SIO1_UART1_TX) || \
                                   ((param) == DMACB_UART0_RX) || \
                                   ((param) == DMACB_UART0_TX) || \
                                   ((param) == DMACB_TMRB08_CAPTURE0) || \
                                   ((param) == DMACB_TMRC0_CAPTURE0) || \
                                   ((param) == DMACB_TMRC0_CAPTURE1) || \
                                   ((param) == DMACB_HIGHEST_UNITB_ADC) || \
                                   ((param) == DMACB_PCNT1_CMP0_MATCH) || \
                                   ((param) == DMACB_TMRD0_CMP_MATCH ) || \
                                   ((param) == DMACB_PIN))

#define IS_DMACC_REQ_NUM(param)  (((param) == DMACC_ESIO2_RX) || \
                                   ((param) == DMACC_ESIO2_TX) || \
                                   ((param) == DMACC_NORMAL_UNITC_ADC) || \
                                   ((param) == DMACC_SIO5_UART5_RX) || \
                                   ((param) == DMACC_SIO5_UART5_TX) || \
                                   ((param) == DMACC_SIO2_UART2_RX) || \
                                   ((param) == DMACC_SIO2_UART2_TX) || \
                                   ((param) == DMACC_UART1_RX) || \
                                   ((param) == DMACC_UART1_TX) || \
                                   ((param) == DMACC_TMRB19_CAPTURE0) || \
                                   ((param) == DMACC_TMRC0_CAPTURE2) || \
                                   ((param) == DMACC_TMRC0_CAPTURE3) || \
                                   ((param) == DMACC_HIGHEST_UNITC_ADC) || \
                                   ((param) == DMACC_PHCP_CYCLE0) || \
                                   ((param) == DMACC_TMRD10_CMP_MATCH ) || \
                                   ((param) == DMACC_PIN))

#define IS_DMACA_BURST_REQ_NUM(param)  (((param) == DMACA_ESIO0_RX) || \
                                   ((param) == DMACA_ESIO0_TX) || \
                                   ((param) == DMACA_NORMAL_UNITA_ADC) || \
                                   ((param) == DMACA_SIO3_UART3_RX) || \
                                   ((param) == DMACA_SIO3_UART3_TX) || \
                                   ((param) == DMACA_SIO0_UART0_RX) || \
                                   ((param) == DMACA_SIO0_UART0_TX) || \
                                   ((param) == DMACA_TMRB00_CMP_MATCH) || \
                                   ((param) == DMACA_TMRB04_CMP_MATCH) || \
                                   ((param) == DMACA_TMRB10_CMP_MATCH) || \
                                   ((param) == DMACA_TMRB14_CMP_MATCH) || \
                                   ((param) == DMACA_TMRC_CMP0_MATCH) || \
                                   ((param) == DMACA_TMRC_CMP1_MATCH) || \
                                   ((param) == DMACA_HIGHEST_UNITA_ADC) || \
                                   ((param) == DMACA_PHCNT0_CMP0_MATCH ) || \
                                   ((param) == DMACA_PIN))

#define IS_DMACB_BURST_REQ_NUM(param)  (((param) == DMACB_ESIO1_RX) || \
                                   ((param) == DMACB_ESIO1_TX) || \
                                   ((param) == DMACB_NORMAL_UNITB_ADC) || \
                                   ((param) == DMACB_SIO4_UART4_RX) || \
                                   ((param) == DMACB_SIO4_UART4_TX) || \
                                   ((param) == DMACB_SIO1_UART1_RX) || \
                                   ((param) == DMACB_SIO1_UART1_TX) || \
                                   ((param) == DMACB_UART0_RX) || \
                                   ((param) == DMACB_UART0_TX) || \
                                   ((param) == DMACB_TMRB08_CAPTURE0) || \
                                   ((param) == DMACB_TMRC0_CAPTURE0) || \
                                   ((param) == DMACB_TMRC0_CAPTURE1) || \
                                   ((param) == DMACB_HIGHEST_UNITB_ADC) || \
                                   ((param) == DMACB_PCNT1_CMP0_MATCH) || \
                                   ((param) == DMACB_TMRD0_CMP_MATCH ) || \
                                   ((param) == DMACB_PIN))

#define IS_DMACC_BURST_REQ_NUM(param)  (((param) == DMACC_ESIO2_RX) || \
                                   ((param) == DMACC_ESIO2_TX) || \
                                   ((param) == DMACC_NORMAL_UNITC_ADC) || \
                                   ((param) == DMACC_SIO5_UART5_RX) || \
                                   ((param) == DMACC_SIO5_UART5_TX) || \
                                   ((param) == DMACC_SIO2_UART2_RX) || \
                                   ((param) == DMACC_SIO2_UART2_TX) || \
                                   ((param) == DMACC_UART1_RX) || \
                                   ((param) == DMACC_UART1_TX) || \
                                   ((param) == DMACC_TMRB19_CAPTURE0) || \
                                   ((param) == DMACC_TMRC0_CAPTURE2) || \
                                   ((param) == DMACC_TMRC0_CAPTURE3) || \
                                   ((param) == DMACC_HIGHEST_UNITC_ADC) || \
                                   ((param) == DMACC_PHCP_CYCLE0) || \
                                   ((param) == DMACC_TMRD10_CMP_MATCH ) || \
                                   ((param) == DMACC_PIN))

#define IS_DMACB_SINGLE_REQ_NUM(param)  (((param) == DMACB_UART0_RX) || \
                                         ((param) == DMACB_UART0_TX))

#define IS_DMACC_SINGLE_REQ_NUM(param)  (((param) == DMACC_UART1_RX) || \
                                         ((param) == DMACC_UART1_TX))

#if defined ( __CC_ARM )
#ifdef __BIG_ENDIAN
    typedef union {
        uint32_t All;
        struct {
            uint32_t Reserved:16;
            uint32_t PIN:1;
            uint32_t PHCNT0_CMP0_MATCH:1;
            uint32_t HIGHEST_UNITA_ADC:1;
            uint32_t TMRC_CMP1_MATCH:1;
            uint32_t TMRC_CMP0_MATCH:1;
            uint32_t TMRB14_CMP_MATCH:1;
            uint32_t TMRB10_CMP_MATCH:1;
            uint32_t TMRB04_CMP_MATCH:1;
            uint32_t TMRB00_CMP_MATCH:1;
            uint32_t SIO0_UART0_TX:1;
            uint32_t SIO0_UART0_RX:1;
            uint32_t SIO3_UART3_TX:1;
            uint32_t SIO3_UART3_RX:1;
            uint32_t NORMAL_UNITA_ADC:1;
            uint32_t ESIO0_TX:1;
            uint32_t ESIO0_RX:1;
        } Bit_A;
        struct {
            uint32_t Reserved:16;
            uint32_t PIN:1;
            uint32_t TMRD0_CMP_MATCH:1;
            uint32_t PCNT1_CMP0_MATCH:1;
            uint32_t HIGHEST_UNITB_ADC:1;
            uint32_t TMRC0_CAPTURE1:1;
            uint32_t TMRC0_CAPTURE0:1;
            uint32_t TMRB08_CAPTURE0:1;
            uint32_t UART0_TX:1;
            uint32_t UART0_RX:1;
            uint32_t SIO1_UART1_TX:1;
            uint32_t SIO1_UART1_RX:1;
            uint32_t SIO4_UART4_TX:1;
            uint32_t SIO4_UART4_RX:1;
            uint32_t NORMAL_UNITB_ADC:1;
            uint32_t ESIO1_TX:1;
            uint32_t ESIO1_RX:1;
        } Bit_B;
        struct {
            uint32_t Reserved:16;
            uint32_t PIN:1;
            uint32_t TMRD10_CMP_MATCH:1;
            uint32_t PHCP_CYCLE0:1;
            uint32_t HIGHEST_UNITC_ADC:1;
            uint32_t TMRC0_CAPTURE3:1;
            uint32_t TMRC0_CAPTURE2:1;
            uint32_t TMRB19_CAPTURE0:1;
            uint32_t UART1_TX:1;
            uint32_t UART1_RX:1;
            uint32_t SIO1_UART2_TX:1;
            uint32_t SIO1_UART2_RX:1;
            uint32_t SIO5_UART5_TX:1;
            uint32_t SIO5_UART5_RX:1;
            uint32_t NORMAL_UNITC_ADC:1;
            uint32_t ESIO2_TX:1;
            uint32_t ESIO2_RX:1;
        } Bit_C;
    } DMAC_Burst_ReqState;
#else
    typedef union {
        uint32_t All;
        struct {
            uint32_t ESIO0_RX:1;
            uint32_t ESIO0_TX:1;
            uint32_t NORMAL_UNITA_ADC:1;
            uint32_t SIO3_UART3_RX:1;
            uint32_t SIO3_UART3_TX:1;
            uint32_t SIO0_UART0_RX:1;
            uint32_t SIO0_UART0_TX:1;
            uint32_t TMRB00_CMP_MATCH:1;
            uint32_t TMRB04_CMP_MATCH:1;
            uint32_t TMRB10_CMP_MATCH:1;
            uint32_t TMRB14_CMP_MATCH:1;
            uint32_t TMRC_CMP0_MATCH:1;
            uint32_t TMRC_CMP1_MATCH:1;
            uint32_t HIGHEST_UNITA_ADC:1;
            uint32_t PHCNT0_CMP0_MATCH:1;
            uint32_t PIN:1;
            uint32_t Reserved:16;
        } Bit_A;
        struct {
            uint32_t ESIO1_RX:1;
            uint32_t ESIO1_TX:1;
            uint32_t NORMAL_UNITB_ADC:1;
            uint32_t SIO4_UART4_RX:1;
            uint32_t SIO4_UART4_TX:1;
            uint32_t SIO1_UART1_RX:1;
            uint32_t SIO1_UART1_TX:1;
            uint32_t UART0_RX:1;
            uint32_t UART0_TX:1;
            uint32_t TMRB08_CAPTURE0:1;
            uint32_t TMRC0_CAPTURE0:1;
            uint32_t TMRC0_CAPTURE1:1;
            uint32_t HIGHEST_UNITB_ADC:1;
            uint32_t PCNT1_CMP0_MATCH:1;
            uint32_t TMRD0_CMP_MATCH:1;
            uint32_t PIN:1;
            uint32_t Reserved:16;
        } Bit_B;
        struct {
            uint32_t ESIO2_RX:1;
            uint32_t ESIO2_TX:1;
            uint32_t NORMAL_UNITC_ADC:1;
            uint32_t SIO5_UART5_RX:1;
            uint32_t SIO5_UART5_TX:1;
            uint32_t SIO1_UART2_RX:1;
            uint32_t SIO1_UART2_TX:1;
            uint32_t UART1_RX:1;
            uint32_t UART1_TX:1;
            uint32_t TMRB19_CAPTURE0:1;
            uint32_t TMRC0_CAPTURE2:1;
            uint32_t TMRC0_CAPTURE3:1;
            uint32_t HIGHEST_UNITC_ADC:1;
            uint32_t PHCP_CYCLE0:1;
            uint32_t TMRD10_CMP_MATCH:1;
            uint32_t PIN:1;
            uint32_t Reserved:16;
        } Bit_C;
    } DMAC_Burst_ReqState;
#endif
#elif  defined(__IAR_SYSTEMS_ICC__)
    typedef union {
        uint32_t All;
        struct {
            uint32_t ESIO0_RX:1;
            uint32_t ESIO0_TX:1;
            uint32_t NORMAL_UNITA_ADC:1;
            uint32_t SIO3_UART3_RX:1;
            uint32_t SIO3_UART3_TX:1;
            uint32_t SIO0_UART0_RX:1;
            uint32_t SIO0_UART0_TX:1;
            uint32_t TMRB00_CMP_MATCH:1;
            uint32_t TMRB04_CMP_MATCH:1;
            uint32_t TMRB10_CMP_MATCH:1;
            uint32_t TMRB14_CMP_MATCH:1;
            uint32_t TMRC_CMP0_MATCH:1;
            uint32_t TMRC_CMP1_MATCH:1;
            uint32_t HIGHEST_UNITA_ADC:1;
            uint32_t PHCNT0_CMP0_MATCH:1;
            uint32_t PIN:1;
            uint32_t Reserved:16;
        } Bit_A;
        struct {
            uint32_t ESIO1_RX:1;
            uint32_t ESIO1_TX:1;
            uint32_t NORMAL_UNITB_ADC:1;
            uint32_t SIO4_UART4_RX:1;
            uint32_t SIO4_UART4_TX:1;
            uint32_t SIO1_UART1_RX:1;
            uint32_t SIO1_UART1_TX:1;
            uint32_t UART0_RX:1;
            uint32_t UART0_TX:1;
            uint32_t TMRB08_CAPTURE0:1;
            uint32_t TMRC0_CAPTURE0:1;
            uint32_t TMRC0_CAPTURE1:1;
            uint32_t HIGHEST_UNITB_ADC:1;
            uint32_t PCNT1_CMP0_MATCH:1;
            uint32_t TMRD0_CMP_MATCH:1;
            uint32_t PIN:1;
            uint32_t Reserved:16;
        } Bit_B;
        struct {
            uint32_t ESIO2_RX:1;
            uint32_t ESIO2_TX:1;
            uint32_t NORMAL_UNITC_ADC:1;
            uint32_t SIO5_UART5_RX:1;
            uint32_t SIO5_UART5_TX:1;
            uint32_t SIO2_UART2_RX:1;
            uint32_t SIO2_UART2_TX:1;
            uint32_t UART1_RX:1;
            uint32_t UART1_TX:1;
            uint32_t TMRB19_CAPTURE0:1;
            uint32_t TMRC0_CAPTURE2:1;
            uint32_t TMRC0_CAPTURE3:1;
            uint32_t HIGHEST_UNITC_ADC:1;
            uint32_t PHCP_CYCLE0:1;
            uint32_t TMRD10_CMP_MATCH:1;
            uint32_t PIN:1;
            uint32_t Reserved:16;
        } Bit_C;
    } DMAC_Burst_ReqState;
#endif

#if defined ( __CC_ARM )
#ifdef __BIG_ENDIAN
    typedef union {
        uint32_t All;
        struct {
            uint32_t Reserved2:23;
            uint32_t UART0_TX:1;
            uint32_t UART0_RX:1;
            uint32_t Reserved1:7;
        } Bit_B;
        struct {
            uint32_t Reserved2:23;
            uint32_t UART1_TX:1;
            uint32_t UART1_RX:1;
            uint32_t Reserved1:7;
        } Bit_C;
    } DMAC_Single_ReqState;
#else
    typedef union {
        uint32_t All;
        struct {
            uint32_t Reserved1:7;
            uint32_t UART0_RX:1;
            uint32_t UART0_TX:1;
            uint32_t Reserved2:23;
        } Bit_B;
        struct {
            uint32_t Reserved1:7;
            uint32_t UART1_RX:1;
            uint32_t UART1_TX:1;
            uint32_t Reserved2:23;
        } Bit_C;
    } DMAC_Single_ReqState;
#endif
#elif  defined(__IAR_SYSTEMS_ICC__)
    typedef union {
        uint32_t All;
        struct {
            uint32_t Reserved1:7;
            uint32_t UART0_RX:1;
            uint32_t UART0_TX:1;
            uint32_t Reserved2:23;
        } Bit_B;
        struct {
            uint32_t Reserved1:7;
            uint32_t UART1_RX:1;
            uint32_t UART1_TX:1;
            uint32_t Reserved2:23;
        } Bit_C;
    } DMAC_Single_ReqState;
#endif

/** @} */
/* End of group DMAC_Exported_Types */

/** @defgroup DMAC_Exported_Types
  * @{
  */

/** 
  * @brief  DMA Structure definition  
  */

    typedef struct {
        uint32_t TxDirection;   /*!< Set transfer direction  */
        uint32_t SrcAddr;       /*!< Set source address  */
        uint32_t DstAddr;       /*!< Set destination address  */
        FunctionalState SrcIncrementState;      /*!< Specifies whether the source
                                                   address is incremented or not  */
        FunctionalState DstIncrementState;      /*!< Specifies whether the destination
                                                   address is incremented or not  */
        DMAC_BitWidth SrcBitWidth;      /*!< Set transfer source bit width  */
        DMAC_BitWidth DstBitWidth;      /*!< Set transfer destination bit width  */
        DMAC_BurstSize SrcBurstSize;    /* Set transfer source burst size */
        DMAC_BurstSize DstBurstSize;    /* Set transfer destination burst size */
        uint32_t TxSize;        /*!< Set the total number of transfer  */
        DMACA_ReqNum A_TxDstPeriph;     /*!< Set transfer destination  */
        DMACA_ReqNum A_TxSrcPeriph;     /*!< Set transfer source peripheral  */
        DMACB_ReqNum B_TxDstPeriph;     /*!< Set transfer destination peripheral  */
        DMACB_ReqNum B_TxSrcPeriph;     /*!< Set transfer source peripheral  */
        DMACC_ReqNum C_TxDstPeriph;     /*!< Set transfer destination peripheral  */
        DMACC_ReqNum C_TxSrcPeriph;     /*!< Set transfer source peripheral  */
        FunctionalState TxINT;  /*!< Set transfer interrupt */
    } DMAC_InitTypeDef;

/** @} */
/* End of group DMAC_Exported_Types */

/** @defgroup DMAC_Exported_FunctionPrototypes
  * @{
  */
    void DMAC_Enable(TSB_DMAC_TypeDef * DMACx);
    void DMAC_Disable(TSB_DMAC_TypeDef * DMACx);
    DMAC_INTReq DMAC_GetINTReq(TSB_DMAC_TypeDef * DMACx);
    DMAC_TxINTReq DMAC_GetTxINTReq(TSB_DMAC_TypeDef * DMACx, DMAC_Channel Chx);
    void DMAC_ClearTxINTReq(TSB_DMAC_TypeDef * DMACx, DMAC_Channel Chx, DMAC_INTSrc INTSource);
    DMAC_TxINTReq DMAC_GetRawTxINTReq(TSB_DMAC_TypeDef * DMACx, DMAC_Channel Chx);
    WorkState DMAC_GetChannelTxState(TSB_DMAC_TypeDef * DMACx, DMAC_Channel Chx);
    void DMACA_SetSWBurstReq(DMACA_ReqNum BurstReq);
    void DMACB_SetSWBurstReq(DMACB_ReqNum BurstReq);
    void DMACC_SetSWBurstReq(DMACC_ReqNum BurstReq);
    DMAC_Burst_ReqState DMAC_GetSWBurstReqState(TSB_DMAC_TypeDef * DMACx);
    void DMACB_SetSWSingleReq(DMACB_ReqNum SingleReq);
    void DMACC_SetSWSingleReq(DMACC_ReqNum SingleReq);
    DMAC_Single_ReqState DMAC_GetSWSingleReqState(TSB_DMAC_TypeDef * DMACx);
    void DMAC_SetLinkedList(TSB_DMAC_TypeDef * DMACx, DMAC_Channel Chx, uint32_t LinkedAddr);
    WorkState DMAC_GetFIFOState(TSB_DMAC_TypeDef * DMACx, DMAC_Channel Chx);
    void DMAC_SetDMAHalt(TSB_DMAC_TypeDef * DMACx, DMAC_Channel Chx, FunctionalState NewState);
    void DMAC_SetLockedTx(TSB_DMAC_TypeDef * DMACx, DMAC_Channel Chx, FunctionalState NewState);
    void DMAC_SetTxINTConfig(TSB_DMAC_TypeDef * DMACx, DMAC_Channel Chx, DMAC_INTSrc INTSource,
                             FunctionalState NewState);
    void DMAC_SetDMAChannel(TSB_DMAC_TypeDef * DMACx, DMAC_Channel Chx, FunctionalState NewState);
    void DMAC_Init(TSB_DMAC_TypeDef * DMACx, DMAC_Channel Chx, DMAC_InitTypeDef * InitStruct);
    void DMAC_SetTransferEndian(TSB_DMAC_TypeDef * DMACx, DMAC_Endian_Mode Endian);
/** @} */
/* End of group DMAC_Exported_FunctionPrototype */

/** @} */
/* End of group DMAC */

/** @} */
/* End of group TX04_Periph_Driver */

#endif                          /* defined(__TMPM440_DMAC_H) */

#ifdef __cplusplus
}
#endif
#endif                          /* __TMPM440_DMAC_H */
