/**
 *******************************************************************************
 * @file    tmpm440_fc.h
 * @brief   This file provides all the functions prototypes for FC driver.
 * @version V2.0.2.3
 * @date    2014/09/28
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __TMPM440_FC_H
#define __TMPM440_FC_H


/* #define SINGLE_BOOT_MODE */

#ifdef __cplusplus
extern "C" {
#endif                          /* __cplusplus */

/*Select MCU type*/
#if !defined(TMPM440F10XBG) && !defined(TMPM440FEXBG)
#define TMPM440FEXBG
#endif                          /* !defined(TMPM440F10XBG) && !defined(TMPM440FEXBG) */

/* Includes ------------------------------------------------------------------*/
#include "TMPM440.h"
#include "tx04_common.h"

/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @addtogroup FC
  * @{
  */

/** @defgroup FC_Exported_Types
  * @{
  */
    typedef enum {
        FC_SUCCESS = 0U,
        FC_ERROR_PROTECTED = 1U,
        FC_ERROR_OVER_TIME = 2U
    } FC_Result;

/** @} */
/* End of group FC_Exported_Types */

/** @defgroup FC_Exported_Constants
  * @{
  */

#define FLASH_PAGE_SIZE                 ((uint32_t)0x00000100)  /* Page Size 256 Byte/64 words */

#define FC_BLOCK_0                      ((uint8_t)0x00)
#define FC_BLOCK_1                      FC_BLOCK_0 + 1U
#define FC_BLOCK_2                      FC_BLOCK_1 + 1U
#define FC_BLOCK_3                      FC_BLOCK_2 + 1U
#define FC_BLOCK_4                      FC_BLOCK_3 + 1U
#define FC_BLOCK_5                      FC_BLOCK_4 + 1U
#define FC_BLOCK_6                      FC_BLOCK_5 + 1U
#define FC_BLOCK_7                      FC_BLOCK_6 + 1U
#define FC_BLOCK_8                      FC_BLOCK_7 + 1U
#define FC_BLOCK_9                      FC_BLOCK_8 + 1U
#define FC_BLOCK_10                     FC_BLOCK_9 + 1U
#define FC_BLOCK_11                     FC_BLOCK_10 + 1U
#define FC_BLOCK_12                     FC_BLOCK_11 + 1U
#define FC_BLOCK_13                     FC_BLOCK_12 + 1U
#define FC_BLOCK_14                     FC_BLOCK_13 + 1U
#define FC_BLOCK_15                     FC_BLOCK_14 + 1U
#define FC_BLOCK_16                     FC_BLOCK_15 + 1U
#define FC_BLOCK_17                     FC_BLOCK_16 + 1U
#define FC_BLOCK_18                     FC_BLOCK_17 + 1U
#define FC_BLOCK_19                     FC_BLOCK_18 + 1U
#define FC_BLOCK_20                     FC_BLOCK_19 + 1U
#define FC_BLOCK_21                     FC_BLOCK_20 + 1U
#define FC_BLOCK_22                     FC_BLOCK_21 + 1U
#define FC_BLOCK_23                     FC_BLOCK_22 + 1U
#define FC_BLOCK_24                     FC_BLOCK_23 + 1U
#define FC_BLOCK_25                     FC_BLOCK_24 + 1U
#define FC_BLOCK_26                     FC_BLOCK_25 + 1U
#define FC_BLOCK_27                     FC_BLOCK_26 + 1U

#define FC_BLOCK_MAX    FC_BLOCK_27

#ifdef TMPM440F10XBG
#define FLASH_CHIP_SIZE                 ((uint32_t)0x00100000)  /* Flash chip size is 1024KByte */

#elif defined(TMPM440FEXBG)
#define FLASH_CHIP_SIZE                 ((uint32_t)0x000C0000)  /* Flash chip size is 768KByte */
#endif

#define IS_FC_BLOCK_NUM(param)          ((param) <= FC_BLOCK_MAX)

#define FC_BLOCK_GROUP_0                 ((uint8_t)0x00)
#define FC_BLOCK_GROUP_1                 FC_BLOCK_GROUP_0 + 1U
#define FC_BLOCK_GROUP_2                 FC_BLOCK_GROUP_1 + 1U
#define FC_BLOCK_GROUP_3                 FC_BLOCK_GROUP_2 + 1U

#define IS_FC_BLOCK_GROUP(param)        ((param) <= FC_BLOCK_GROUP_3)

#ifdef SINGLE_BOOT_MODE
#define FLASH_START_ADDR                 ((uint32_t)0x3F800000) /* SINGLE_BOOT_MODE */
#else
#define FLASH_START_ADDR                 ((uint32_t)0x00000000) /* User Boot Mode As Default */
#endif

#define FLASH_END_ADDR                       (FLASH_START_ADDR + FLASH_CHIP_SIZE - 1U)

#ifdef SINGLE_BOOT_MODE
#define IS_FC_ADDR(param)                (((param) >= FLASH_START_ADDR) && \
                                          ((param) <= FLASH_END_ADDR))
#else
#define IS_FC_ADDR(param)                ((param) <= FLASH_END_ADDR)
#endif

#define IS_FC_PAGE_ADDR(param)          ((((param) - FLASH_START_ADDR) % FLASH_PAGE_SIZE) == 0U)

/** @} */
/* End of group FC_Exported_Constants */


/** @defgroup FC_Exported_FunctionPrototypes
  * @{
  */

    uint8_t FC_AddrToBlockNum(uint32_t Addr);
    void FC_SetSecurityBit(FunctionalState NewState);
    FunctionalState FC_GetSecurityBit(void);
    WorkState FC_GetBusyState(void);
    FunctionalState FC_GetBlockProtectState(uint8_t BlockNum);
    FC_Result FC_ProgramBlockProtectState(uint8_t BlockNum);
    FC_Result FC_EraseBlockProtectState(uint8_t BlockGroup);
    FC_Result FC_WritePage(uint32_t PageAddr, uint32_t * Data);
    FC_Result FC_EraseBlock(uint32_t BlockAddr);
    FC_Result FC_EraseChip(void);
    void FC_SetCtrlReg(FunctionalState NewState);

/** @} */
/* End of group FC_Exported_FunctionPrototypes */

/** @} */
/* End of group FC */

/** @} */
/* End of group TX04_Periph_Driver */

#ifdef __cplusplus
}
#endif                          /* __cplusplus */
#endif                          /* __TMPM440_FC_H */
