/**
 *******************************************************************************
 * @file    tmpm440_gpio.h
 * @brief   This file provides all the functions prototypes for GPIO driver. 
 * @version V2.0.2.3
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __TMPM440_GPIO_H
#define __TMPM440_GPIO_H

#ifdef __cplusplus
extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "TMPM440.h"
#include "tx04_common.h"

/** @addtogroup TX04_Periph_Driver
  * @{
  */
/** @addtogroup GPIO
  * @{
  */
/** @addtogroup  GPIO_Parameter_Definition
  * @{
  */
    typedef struct {
        uint8_t IOMode;         /* Set the port input or output mode       */
        uint8_t PullUp;         /* Enable or disable Pullup function       */
        uint8_t OpenDrain;      /* Enable or disable open drain function */
        uint8_t OpenDrain2;     /* Enable or disable open drain function */
        uint8_t PullDown;       /* Enable or disable Pulldown function */
    } GPIO_InitTypeDef;

#define GPIO_INPUT_MODE                 ((uint8_t)0x00)
#define GPIO_OUTPUT_MODE                ((uint8_t)0x01)
#define GPIO_IO_MODE_NONE               ((uint8_t)0x02)
#define IS_GPIO_IO_MODE_STATE(param)    (((param) == GPIO_INPUT_MODE) || \
                                         ((param) == GPIO_OUTPUT_MODE) || \
                                         ((param) == GPIO_IO_MODE_NONE))

#define GPIO_PULLUP_DISABLE             ((uint8_t)0x00)
#define GPIO_PULLUP_ENABLE              ((uint8_t)0x01)
#define GPIO_PULLUP_NONE                ((uint8_t)0x02)
#define IS_GPIO_PULLUP_STATE(param)     (((param) == GPIO_PULLUP_ENABLE) || \
                                         ((param) == GPIO_PULLUP_DISABLE) || \
                                         ((param) == GPIO_PULLUP_NONE))

#define GPIO_PULLDOWN_DISABLE           ((uint8_t)0x00)
#define GPIO_PULLDOWN_ENABLE            ((uint8_t)0x01)
#define GPIO_PULLDOWN_NONE              ((uint8_t)0x02)
#define IS_GPIO_PULLDOWN_STATE(param)   (((param) == GPIO_PULLDOWN_ENABLE) || \
                                         ((param) == GPIO_PULLDOWN_DISABLE) || \
                                         ((param) == GPIO_PULLDOWN_NONE))

#define GPIO_OPEN_DRAIN_DISABLE          ((uint8_t)0x00)
#define GPIO_OPEN_DRAIN_ENABLE           ((uint8_t)0x01)
#define GPIO_OPEN_DRAIN_NONE             ((uint8_t)0x02)
#define IS_GPIO_OPEN_DRAIN_STATE(param)  (((param) == GPIO_OPEN_DRAIN_ENABLE) || \
                                          ((param) == GPIO_OPEN_DRAIN_DISABLE) || \
                                          ((param) == GPIO_OPEN_DRAIN_NONE))

#define GPIO_FUNC_REG_1                 ((uint8_t)0x00)
#define GPIO_FUNC_REG_2                 ((uint8_t)0x01)
#define GPIO_FUNC_REG_3                 ((uint8_t)0x02)

    typedef enum {
        GPIO_PA = 0U,
        GPIO_PB = 1U,
        GPIO_PC = 2U,
        GPIO_PD = 3U,
        GPIO_PE = 4U,
        GPIO_PF = 5U,
        GPIO_PG = 6U,
        GPIO_PH = 7U,
        GPIO_PJ = 9U,
        GPIO_PK = 10U,
        GPIO_PL = 11U,
        GPIO_PM = 12U,
        GPIO_PN = 13U,
        GPIO_PP = 15U,
        GPIO_PR = 17U,
        GPIO_PT = 19U,
        GPIO_PU = 20U,
        GPIO_PV = 21U,
        GPIO_PW = 22U,
        GPIO_PY = 24U,
        GPIO_PAA = 26U,
        GPIO_PAB = 27U,
        GPIO_PAC = 28U,
        GPIO_PAD = 29U,
        GPIO_PAE = 30U,
        GPIO_PAF = 31U,
        GPIO_PAG = 32U,
        GPIO_PAH = 33U,
        GPIO_PAJ = 35U
    } GPIO_Port;

#define IS_GPIO_PORT(param)             (((param) == GPIO_PA)|| \
                                         ((param) == GPIO_PB)|| \
                                         ((param) == GPIO_PC)|| \
                                         ((param) == GPIO_PD)|| \
                                         ((param) == GPIO_PE)|| \
                                         ((param) == GPIO_PF)|| \
                                         ((param) == GPIO_PG)|| \
                                         ((param) == GPIO_PH)|| \
                                         ((param) == GPIO_PJ)|| \
                                         ((param) == GPIO_PK)|| \
                                         ((param) == GPIO_PL)|| \
                                         ((param) == GPIO_PM)|| \
                                         ((param) == GPIO_PN)|| \
                                         ((param) == GPIO_PP)|| \
                                         ((param) == GPIO_PR)|| \
                                         ((param) == GPIO_PT)|| \
                                         ((param) == GPIO_PU)|| \
                                         ((param) == GPIO_PV)|| \
                                         ((param) == GPIO_PW)|| \
                                         ((param) == GPIO_PY)|| \
                                         ((param) == GPIO_PAA)|| \
                                         ((param) == GPIO_PAB)|| \
                                         ((param) == GPIO_PAC)|| \
                                         ((param) == GPIO_PAD)|| \
                                         ((param) == GPIO_PAE)|| \
                                         ((param) == GPIO_PAF)|| \
                                         ((param) == GPIO_PAG)|| \
                                         ((param) == GPIO_PAH)|| \
                                         ((param) == GPIO_PAJ))

#define IS_GPIO_WRITE(param)            (((param) == GPIO_PA)|| \
                                         ((param) == GPIO_PB)|| \
                                         ((param) == GPIO_PC)|| \
                                         ((param) == GPIO_PD)|| \
                                         ((param) == GPIO_PE)|| \
                                         ((param) == GPIO_PF)|| \
                                         ((param) == GPIO_PG)|| \
                                         ((param) == GPIO_PH)|| \
                                         ((param) == GPIO_PJ)|| \
                                         ((param) == GPIO_PK)|| \
                                         ((param) == GPIO_PL)|| \
                                         ((param) == GPIO_PM)|| \
                                         ((param) == GPIO_PN)|| \
                                         ((param) == GPIO_PP)|| \
                                         ((param) == GPIO_PR)|| \
                                         ((param) == GPIO_PT)|| \
                                         ((param) == GPIO_PU)|| \
                                         ((param) == GPIO_PV)|| \
                                         ((param) == GPIO_PW)|| \
                                         ((param) == GPIO_PY)|| \
                                         ((param) == GPIO_PAD)|| \
                                         ((param) == GPIO_PAE)|| \
                                         ((param) == GPIO_PAF)|| \
                                         ((param) == GPIO_PAG)|| \
                                         ((param) == GPIO_PAH)|| \
                                         ((param) == GPIO_PAJ))

#define IS_GPIO_INPUT(param)            (((param) == GPIO_PA)|| \
                                         ((param) == GPIO_PB)|| \
                                         ((param) == GPIO_PC)|| \
                                         ((param) == GPIO_PD)|| \
                                         ((param) == GPIO_PE)|| \
                                         ((param) == GPIO_PF)|| \
                                         ((param) == GPIO_PG)|| \
                                         ((param) == GPIO_PH)|| \
                                         ((param) == GPIO_PJ)|| \
                                         ((param) == GPIO_PK)|| \
                                         ((param) == GPIO_PL)|| \
                                         ((param) == GPIO_PM)|| \
                                         ((param) == GPIO_PN)|| \
                                         ((param) == GPIO_PP)|| \
                                         ((param) == GPIO_PR)|| \
                                         ((param) == GPIO_PT)|| \
                                         ((param) == GPIO_PU)|| \
                                         ((param) == GPIO_PV)|| \
                                         ((param) == GPIO_PW)|| \
                                         ((param) == GPIO_PY)|| \
                                         ((param) == GPIO_PAA)|| \
                                         ((param) == GPIO_PAB)|| \
                                         ((param) == GPIO_PAC)|| \
                                         ((param) == GPIO_PAD)|| \
                                         ((param) == GPIO_PAE)|| \
                                         ((param) == GPIO_PAF)|| \
                                         ((param) == GPIO_PAG)|| \
                                         ((param) == GPIO_PAH)|| \
                                         ((param) == GPIO_PAJ))

#define IS_GPIO_OUTPUT(param)           (((param) == GPIO_PA)|| \
                                         ((param) == GPIO_PB)|| \
                                         ((param) == GPIO_PC)|| \
                                         ((param) == GPIO_PD)|| \
                                         ((param) == GPIO_PE)|| \
                                         ((param) == GPIO_PF)|| \
                                         ((param) == GPIO_PG)|| \
                                         ((param) == GPIO_PH)|| \
                                         ((param) == GPIO_PJ)|| \
                                         ((param) == GPIO_PK)|| \
                                         ((param) == GPIO_PL)|| \
                                         ((param) == GPIO_PM)|| \
                                         ((param) == GPIO_PN)|| \
                                         ((param) == GPIO_PP)|| \
                                         ((param) == GPIO_PR)|| \
                                         ((param) == GPIO_PT)|| \
                                         ((param) == GPIO_PU)|| \
                                         ((param) == GPIO_PV)|| \
                                         ((param) == GPIO_PW)|| \
                                         ((param) == GPIO_PY)|| \
                                         ((param) == GPIO_PAD)|| \
                                         ((param) == GPIO_PAE)|| \
                                         ((param) == GPIO_PAF)|| \
                                         ((param) == GPIO_PAG)|| \
                                         ((param) == GPIO_PAH)|| \
                                         ((param) == GPIO_PAJ))

#define IS_GPIO_PULLUP(param)           (((param) == GPIO_PA)|| \
                                         ((param) == GPIO_PB)|| \
                                         ((param) == GPIO_PC)|| \
                                         ((param) == GPIO_PD)|| \
                                         ((param) == GPIO_PE)|| \
                                         ((param) == GPIO_PF)|| \
                                         ((param) == GPIO_PG)|| \
                                         ((param) == GPIO_PH)|| \
                                         ((param) == GPIO_PJ)|| \
                                         ((param) == GPIO_PK)|| \
                                         ((param) == GPIO_PL)|| \
                                         ((param) == GPIO_PM)|| \
                                         ((param) == GPIO_PN)|| \
                                         ((param) == GPIO_PP)|| \
                                         ((param) == GPIO_PR)|| \
                                         ((param) == GPIO_PT)|| \
                                         ((param) == GPIO_PU)|| \
                                         ((param) == GPIO_PV)|| \
                                         ((param) == GPIO_PW)|| \
                                         ((param) == GPIO_PY)|| \
                                         ((param) == GPIO_PAA)|| \
                                         ((param) == GPIO_PAB)|| \
                                         ((param) == GPIO_PAC)|| \
                                         ((param) == GPIO_PAD)|| \
                                         ((param) == GPIO_PAE)|| \
                                         ((param) == GPIO_PAF)|| \
                                         ((param) == GPIO_PAG)|| \
                                         ((param) == GPIO_PAH)|| \
                                         ((param) == GPIO_PAJ))

#define IS_GPIO_PULLDOWN(param)         (((param) == GPIO_PG))

#define IS_GPIO_OPEN_DRAIN(param)       (((param) == GPIO_PE)|| \
                                         ((param) == GPIO_PH)|| \
                                         ((param) == GPIO_PK)|| \
                                         ((param) == GPIO_PM)|| \
                                         ((param) == GPIO_PR)|| \
                                         ((param) == GPIO_PW)|| \
                                         ((param) == GPIO_PAJ))

#define IS_GPIO_FUNCTION(param)         (((param) == GPIO_PA)|| \
                                         ((param) == GPIO_PB)|| \
                                         ((param) == GPIO_PC)|| \
                                         ((param) == GPIO_PD)|| \
                                         ((param) == GPIO_PE)|| \
                                         ((param) == GPIO_PF)|| \
                                         ((param) == GPIO_PG)|| \
                                         ((param) == GPIO_PH)|| \
                                         ((param) == GPIO_PJ)|| \
                                         ((param) == GPIO_PK)|| \
                                         ((param) == GPIO_PL)|| \
                                         ((param) == GPIO_PM)|| \
                                         ((param) == GPIO_PN)|| \
                                         ((param) == GPIO_PP)|| \
                                         ((param) == GPIO_PR)|| \
                                         ((param) == GPIO_PT)|| \
                                         ((param) == GPIO_PU)|| \
                                         ((param) == GPIO_PV)|| \
                                         ((param) == GPIO_PW)|| \
                                         ((param) == GPIO_PY)|| \
                                         ((param) == GPIO_PAD)|| \
                                         ((param) == GPIO_PAE)|| \
                                         ((param) == GPIO_PAF)|| \
                                         ((param) == GPIO_PAG)|| \
                                         ((param) == GPIO_PAJ))

#define IS_GPIO_PA_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1)|| \
                                         ((param) == GPIO_FUNC_REG_3))

#define IS_GPIO_PB_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1)|| \
                                         ((param) == GPIO_FUNC_REG_3))

#define IS_GPIO_PC_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1)|| \
                                         ((param) == GPIO_FUNC_REG_2)|| \
                                         ((param) == GPIO_FUNC_REG_3))

#define IS_GPIO_PD_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1)|| \
                                         ((param) == GPIO_FUNC_REG_2))

#define IS_GPIO_PE_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1)|| \
                                         ((param) == GPIO_FUNC_REG_2)|| \
                                         ((param) == GPIO_FUNC_REG_3))

#define IS_GPIO_PF_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1)|| \
                                         ((param) == GPIO_FUNC_REG_2)|| \
                                         ((param) == GPIO_FUNC_REG_3))

#define IS_GPIO_PG_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1))

#define IS_GPIO_PH_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1)|| \
                                         ((param) == GPIO_FUNC_REG_2))

#define IS_GPIO_PJ_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1))

#define IS_GPIO_PK_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1)|| \
                                         ((param) == GPIO_FUNC_REG_2)|| \
                                         ((param) == GPIO_FUNC_REG_3))

#define IS_GPIO_PL_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1))

#define IS_GPIO_PM_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1)|| \
                                         ((param) == GPIO_FUNC_REG_2)|| \
                                         ((param) == GPIO_FUNC_REG_3))

#define IS_GPIO_PN_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1)|| \
                                         ((param) == GPIO_FUNC_REG_3))

#define IS_GPIO_PP_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1)|| \
                                         ((param) == GPIO_FUNC_REG_2)|| \
                                         ((param) == GPIO_FUNC_REG_3))

#define IS_GPIO_PR_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1)|| \
                                         ((param) == GPIO_FUNC_REG_2)|| \
                                         ((param) == GPIO_FUNC_REG_3))

#define IS_GPIO_PT_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1))

#define IS_GPIO_PU_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1))

#define IS_GPIO_PV_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1))

#define IS_GPIO_PW_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1))

#define IS_GPIO_PY_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1))

#define IS_GPIO_PAD_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1))

#define IS_GPIO_PAE_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1))

#define IS_GPIO_PAF_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1))

#define IS_GPIO_PAG_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1)|| \
                                         ((param) == GPIO_FUNC_REG_2))

#define IS_GPIO_PAJ_FUNCTION_REG(param)  (((param) == GPIO_FUNC_REG_1))
/** @} */
/* End of group GPIO_Parameter_Definition */

/** @defgroup GPIO_Bit_Define
  * @{
  */
#define GPIO_BIT_VALUE_1                ((uint8_t)0x01)
#define GPIO_BIT_VALUE_0                ((uint8_t)0x00)

#define IS_GPIO_BIT_VALUE(BitValue)     (((BitValue) == GPIO_BIT_VALUE_1)|| \
                                         ((BitValue) == GPIO_BIT_VALUE_0))

#define GPIO_BIT_0                      ((uint8_t)0x01)
#define GPIO_BIT_1                      ((uint8_t)0x02)
#define GPIO_BIT_2                      ((uint8_t)0x04)
#define GPIO_BIT_3                      ((uint8_t)0x08)
#define GPIO_BIT_4                      ((uint8_t)0x10)
#define GPIO_BIT_5                      ((uint8_t)0x20)
#define GPIO_BIT_6                      ((uint8_t)0x40)
#define GPIO_BIT_7                      ((uint8_t)0x80)
#define GPIO_BIT_ALL                    ((uint8_t)0xFF)

#define GPIO_PA_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PA_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PA_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PA_FR1_BIT_ALL                      ((uint8_t)0xFF)
#define GPIO_PA_FR3_BIT_ALL                      ((uint8_t)0xFF)

#define GPIO_PB_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PB_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PB_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PB_FR1_BIT_ALL                      ((uint8_t)0xFF)
#define GPIO_PB_FR3_BIT_ALL                      ((uint8_t)0xFF)

#define GPIO_PC_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PC_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PC_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PC_FR1_BIT_ALL                      ((uint8_t)0xFF)
#define GPIO_PC_FR2_BIT_ALL                      ((uint8_t)0xFF)
#define GPIO_PC_FR3_BIT_ALL                      ((uint8_t)0xFF)

#define GPIO_PD_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PD_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PD_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PD_FR1_BIT_ALL                      ((uint8_t)0xFF)
#define GPIO_PD_FR2_BIT_ALL                      ((uint8_t)0xFF)

#define GPIO_PE_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PE_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PE_OPEN_DRAIN_BIT_ALL               ((uint8_t)0xFF)
#define GPIO_PE_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PE_FR1_BIT_ALL                      ((uint8_t)0xFF)
#define GPIO_PE_FR2_BIT_ALL                      ((uint8_t)0xFF)
#define GPIO_PE_FR3_BIT_ALL                      ((uint8_t)0x44)

#define GPIO_PF_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PF_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PF_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PF_FR1_BIT_ALL                      ((uint8_t)0x7F)
#define GPIO_PF_FR2_BIT_ALL                      ((uint8_t)0x40)
#define GPIO_PF_FR3_BIT_ALL                      ((uint8_t)0x40)

#define GPIO_PG_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PG_PULLUP_BIT_ALL                   ((uint8_t)0xFB)
#define GPIO_PG_PULLDOWN_BIT_ALL                 ((uint8_t)0x04)
#define GPIO_PG_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PG_FR1_BIT_ALL                      ((uint8_t)0xFF)

#define GPIO_PH_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PH_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PH_OPEN_DRAIN_BIT_ALL               ((uint8_t)0xFF)
#define GPIO_PH_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PH_FR1_BIT_ALL                      ((uint8_t)0xFF)
#define GPIO_PH_FR2_BIT_ALL                      ((uint8_t)0x48)

#define GPIO_PJ_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PJ_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PJ_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PJ_FR1_BIT_ALL                      ((uint8_t)0xFF)

#define GPIO_PK_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PK_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PK_OPEN_DRAIN_BIT_ALL               ((uint8_t)0x0F)
#define GPIO_PK_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PK_FR1_BIT_ALL                      ((uint8_t)0x77)
#define GPIO_PK_FR2_BIT_ALL                      ((uint8_t)0xCC)
#define GPIO_PK_FR3_BIT_ALL                      ((uint8_t)0xC8)

#define GPIO_PL_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PL_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PL_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PL_FR1_BIT_ALL                      ((uint8_t)0xFF)

#define GPIO_PM_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PM_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PM_OPEN_DRAIN_BIT_ALL               ((uint8_t)0x0F)
#define GPIO_PM_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PM_FR1_BIT_ALL                      ((uint8_t)0x7F)
#define GPIO_PM_FR2_BIT_ALL                      ((uint8_t)0xCC)
#define GPIO_PM_FR3_BIT_ALL                      ((uint8_t)0xC8)

#define GPIO_PN_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PN_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PN_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PN_FR1_BIT_ALL                      ((uint8_t)0xFF)
#define GPIO_PN_FR3_BIT_ALL                      ((uint8_t)0x0C)

#define GPIO_PP_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PP_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PP_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PP_FR1_BIT_ALL                      ((uint8_t)0x77)
#define GPIO_PP_FR2_BIT_ALL                      ((uint8_t)0xCC)
#define GPIO_PP_FR3_BIT_ALL                      ((uint8_t)0xCC)

#define GPIO_PR_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PR_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PR_OPEN_DRAIN_BIT_ALL               ((uint8_t)0x7F)
#define GPIO_PR_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PR_FR1_BIT_ALL                      ((uint8_t)0x3F)
#define GPIO_PR_FR2_BIT_ALL                      ((uint8_t)0x0C)
#define GPIO_PR_FR3_BIT_ALL                      ((uint8_t)0xCC)

#define GPIO_PT_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PT_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PT_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PT_FR1_BIT_ALL                      ((uint8_t)0xFF)

#define GPIO_PU_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PU_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PU_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PU_FR1_BIT_ALL                      ((uint8_t)0xFF)

#define GPIO_PV_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PV_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PV_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PV_FR1_BIT_ALL                      ((uint8_t)0xFF)

#define GPIO_PW_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PW_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PW_OPEN_DRAIN_BIT_ALL               ((uint8_t)0xFF)
#define GPIO_PW_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PW_FR1_BIT_ALL                      ((uint8_t)0xFF)

#define GPIO_PY_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PY_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PY_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PY_FR1_BIT_ALL                      ((uint8_t)0xFF)

#define GPIO_PAA_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PAA_INPUT_BIT_ALL                    ((uint8_t)0xFF)

#define GPIO_PAB_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PAB_INPUT_BIT_ALL                    ((uint8_t)0xFF)

#define GPIO_PAC_PULLUP_BIT_ALL                   ((uint8_t)0x0F)
#define GPIO_PAC_INPUT_BIT_ALL                    ((uint8_t)0x0F)

#define GPIO_PAD_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PAD_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PAD_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PAD_FR1_BIT_ALL                      ((uint8_t)0xFF)

#define GPIO_PAE_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PAE_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PAE_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PAE_FR1_BIT_ALL                      ((uint8_t)0xFF)

#define GPIO_PAF_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PAF_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PAF_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PAF_FR1_BIT_ALL                      ((uint8_t)0xFF)

#define GPIO_PAG_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PAG_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PAG_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PAG_FR1_BIT_ALL                      ((uint8_t)0xFF)
#define GPIO_PAG_FR2_BIT_ALL                      ((uint8_t)0x80)

#define GPIO_PAH_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PAH_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PAH_INPUT_BIT_ALL                    ((uint8_t)0xFF)

#define GPIO_PAJ_OUTPUT_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PAJ_PULLUP_BIT_ALL                   ((uint8_t)0xFF)
#define GPIO_PAJ_OPEN_DRAIN_BIT_ALL               ((uint8_t)0xFF)
#define GPIO_PAJ_INPUT_BIT_ALL                    ((uint8_t)0xFF)
#define GPIO_PAJ_FR1_BIT_ALL                      ((uint8_t)0xFF)

#define IS_GPIO_PA_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PB_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PC_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PD_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PE_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PF_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PG_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PH_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PJ_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PK_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PL_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PM_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PN_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PP_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PR_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PT_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PU_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PV_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PW_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PY_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PAA_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PAB_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PAC_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3))

#define IS_GPIO_PAD_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PAE_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PAF_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PAG_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PAH_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PAJ_DATA_BIT(param)      (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7))

#define IS_GPIO_PA_OUTPUT_BIT(param)     (((param) & GPIO_PA_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PA_OUTPUT_BIT_ALL))))

#define IS_GPIO_PA_FR1_BIT(param)        (((param) & GPIO_PA_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PA_FR1_BIT_ALL))))

#define IS_GPIO_PA_FR3_BIT(param)        (((param) & GPIO_PA_FR3_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PA_FR3_BIT_ALL))))

#define IS_GPIO_PA_PULLUP_BIT(param)     (((param) & GPIO_PA_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PA_PULLUP_BIT_ALL))))

#define IS_GPIO_PA_INPUT_BIT(param)      (((param) & GPIO_PA_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PA_INPUT_BIT_ALL))))

#define IS_GPIO_PB_OUTPUT_BIT(param)     (((param) & GPIO_PB_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PB_OUTPUT_BIT_ALL))))

#define IS_GPIO_PB_FR1_BIT(param)        (((param) & GPIO_PB_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PB_FR1_BIT_ALL))))

#define IS_GPIO_PB_FR3_BIT(param)        (((param) & GPIO_PB_FR3_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PB_FR3_BIT_ALL))))

#define IS_GPIO_PB_PULLUP_BIT(param)     (((param) & GPIO_PB_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PB_PULLUP_BIT_ALL))))

#define IS_GPIO_PB_INPUT_BIT(param)      (((param) & GPIO_PB_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PB_INPUT_BIT_ALL))))

#define IS_GPIO_PC_OUTPUT_BIT(param)     (((param) & GPIO_PC_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PC_OUTPUT_BIT_ALL))))

#define IS_GPIO_PC_FR1_BIT(param)        (((param) & GPIO_PC_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PC_FR1_BIT_ALL))))

#define IS_GPIO_PC_FR2_BIT(param)        (((param) & GPIO_PC_FR2_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PC_FR2_BIT_ALL))))

#define IS_GPIO_PC_FR3_BIT(param)        (((param) & GPIO_PC_FR3_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PC_FR3_BIT_ALL))))

#define IS_GPIO_PC_PULLUP_BIT(param)     (((param) & GPIO_PC_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PC_PULLUP_BIT_ALL))))

#define IS_GPIO_PC_INPUT_BIT(param)      (((param) & GPIO_PC_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PC_INPUT_BIT_ALL))))

#define IS_GPIO_PD_OUTPUT_BIT(param)     (((param) & GPIO_PD_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PD_OUTPUT_BIT_ALL))))

#define IS_GPIO_PD_FR1_BIT(param)        (((param) & GPIO_PD_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PD_FR1_BIT_ALL))))

#define IS_GPIO_PD_FR2_BIT(param)        (((param) & GPIO_PD_FR2_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PD_FR2_BIT_ALL))))

#define IS_GPIO_PD_PULLUP_BIT(param)     (((param) & GPIO_PD_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PD_PULLUP_BIT_ALL))))

#define IS_GPIO_PD_INPUT_BIT(param)      (((param) & GPIO_PD_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PD_INPUT_BIT_ALL))))

#define IS_GPIO_PE_OUTPUT_BIT(param)     (((param) & GPIO_PE_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PE_OUTPUT_BIT_ALL))))

#define IS_GPIO_PE_FR1_BIT(param)        (((param) & GPIO_PE_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PE_FR1_BIT_ALL))))

#define IS_GPIO_PE_FR2_BIT(param)        (((param) & GPIO_PE_FR2_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PE_FR2_BIT_ALL))))

#define IS_GPIO_PE_FR3_BIT(param)        (((param) & GPIO_PE_FR3_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PE_FR3_BIT_ALL))))

#define IS_GPIO_PE_OPEN_DRAIN_BIT(param) (((param) & GPIO_PE_OPEN_DRAIN_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PE_OPEN_DRAIN_BIT_ALL))))

#define IS_GPIO_PE_PULLUP_BIT(param)     (((param) & GPIO_PE_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PE_PULLUP_BIT_ALL))))

#define IS_GPIO_PE_INPUT_BIT(param)      (((param) & GPIO_PE_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PE_INPUT_BIT_ALL))))

#define IS_GPIO_PF_OUTPUT_BIT(param)     (((param) & GPIO_PF_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PF_OUTPUT_BIT_ALL))))

#define IS_GPIO_PF_FR1_BIT(param)        (((param) & GPIO_PF_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PF_FR1_BIT_ALL))))

#define IS_GPIO_PF_FR2_BIT(param)        (((param) & GPIO_PF_FR2_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PF_FR2_BIT_ALL))))

#define IS_GPIO_PF_FR3_BIT(param)        (((param) & GPIO_PF_FR3_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PF_FR3_BIT_ALL))))

#define IS_GPIO_PF_PULLUP_BIT(param)     (((param) & GPIO_PF_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PF_PULLUP_BIT_ALL))))

#define IS_GPIO_PF_INPUT_BIT(param)      (((param) & GPIO_PF_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PF_INPUT_BIT_ALL))))

#define IS_GPIO_PG_OUTPUT_BIT(param)     (((param) & GPIO_PG_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PG_OUTPUT_BIT_ALL))))

#define IS_GPIO_PG_FR1_BIT(param)        (((param) & GPIO_PG_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PG_FR1_BIT_ALL))))

#define IS_GPIO_PG_PULLUP_BIT(param)     (((param) & GPIO_PG_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PG_PULLUP_BIT_ALL))))

#define IS_GPIO_PG_INPUT_BIT(param)      (((param) & GPIO_PG_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PG_INPUT_BIT_ALL))))

#define IS_GPIO_PG_PULLDOWN_BIT(param)   (((param) & GPIO_PG_PULLDOWN_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PG_PULLDOWN_BIT_ALL))))

#define IS_GPIO_PH_OUTPUT_BIT(param)     (((param) & GPIO_PH_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PH_OUTPUT_BIT_ALL))))

#define IS_GPIO_PH_FR1_BIT(param)        (((param) & GPIO_PH_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PH_FR1_BIT_ALL))))

#define IS_GPIO_PH_FR2_BIT(param)        (((param) & GPIO_PH_FR2_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PH_FR2_BIT_ALL))))

#define IS_GPIO_PH_OPEN_DRAIN_BIT(param) (((param) & GPIO_PH_OPEN_DRAIN_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PH_OPEN_DRAIN_BIT_ALL))))

#define IS_GPIO_PH_PULLUP_BIT(param)     (((param) & GPIO_PH_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PH_PULLUP_BIT_ALL))))

#define IS_GPIO_PH_INPUT_BIT(param)      (((param) & GPIO_PH_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PH_INPUT_BIT_ALL))))

#define IS_GPIO_PJ_OUTPUT_BIT(param)     (((param) & GPIO_PJ_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PJ_OUTPUT_BIT_ALL))))

#define IS_GPIO_PJ_FR1_BIT(param)        (((param) & GPIO_PJ_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PJ_FR1_BIT_ALL))))

#define IS_GPIO_PJ_PULLUP_BIT(param)     (((param) & GPIO_PJ_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PJ_PULLUP_BIT_ALL))))

#define IS_GPIO_PJ_INPUT_BIT(param)      (((param) & GPIO_PJ_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PJ_INPUT_BIT_ALL))))

#define IS_GPIO_PK_OUTPUT_BIT(param)     (((param) & GPIO_PK_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PK_OUTPUT_BIT_ALL))))

#define IS_GPIO_PK_FR1_BIT(param)        (((param) & GPIO_PK_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PK_FR1_BIT_ALL))))

#define IS_GPIO_PK_FR2_BIT(param)        (((param) & GPIO_PK_FR2_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PK_FR2_BIT_ALL))))

#define IS_GPIO_PK_FR3_BIT(param)        (((param) & GPIO_PK_FR3_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PK_FR3_BIT_ALL))))

#define IS_GPIO_PK_OPEN_DRAIN_BIT(param) (((param) & GPIO_PK_OPEN_DRAIN_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PK_OPEN_DRAIN_BIT_ALL))))

#define IS_GPIO_PK_PULLUP_BIT(param)     (((param) & GPIO_PK_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PK_PULLUP_BIT_ALL))))

#define IS_GPIO_PK_INPUT_BIT(param)      (((param) & GPIO_PK_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PK_INPUT_BIT_ALL))))

#define IS_GPIO_PL_OUTPUT_BIT(param)     (((param) & GPIO_PL_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PL_OUTPUT_BIT_ALL))))

#define IS_GPIO_PL_FR1_BIT(param)        (((param) & GPIO_PL_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PL_FR1_BIT_ALL))))

#define IS_GPIO_PL_PULLUP_BIT(param)     (((param) & GPIO_PL_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PL_PULLUP_BIT_ALL))))

#define IS_GPIO_PL_INPUT_BIT(param)      (((param) & GPIO_PL_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PL_INPUT_BIT_ALL))))

#define IS_GPIO_PM_OUTPUT_BIT(param)     (((param) & GPIO_PM_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PM_OUTPUT_BIT_ALL))))

#define IS_GPIO_PM_FR1_BIT(param)        (((param) & GPIO_PM_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PM_FR1_BIT_ALL))))

#define IS_GPIO_PM_FR2_BIT(param)        (((param) & GPIO_PM_FR2_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PM_FR2_BIT_ALL))))

#define IS_GPIO_PM_FR3_BIT(param)        (((param) & GPIO_PM_FR3_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PM_FR3_BIT_ALL))))

#define IS_GPIO_PM_OPEN_DRAIN_BIT(param) (((param) & GPIO_PM_OPEN_DRAIN_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PM_OPEN_DRAIN_BIT_ALL))))

#define IS_GPIO_PM_PULLUP_BIT(param)     (((param) & GPIO_PM_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PM_PULLUP_BIT_ALL))))

#define IS_GPIO_PM_INPUT_BIT(param)      (((param) & GPIO_PM_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PM_INPUT_BIT_ALL))))

#define IS_GPIO_PN_OUTPUT_BIT(param)     (((param) & GPIO_PN_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PN_OUTPUT_BIT_ALL))))

#define IS_GPIO_PN_FR1_BIT(param)        (((param) & GPIO_PN_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PN_FR1_BIT_ALL))))

#define IS_GPIO_PN_FR3_BIT(param)        (((param) & GPIO_PN_FR3_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PN_FR3_BIT_ALL))))

#define IS_GPIO_PN_PULLUP_BIT(param)     (((param) & GPIO_PN_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PN_PULLUP_BIT_ALL))))

#define IS_GPIO_PN_INPUT_BIT(param)      (((param) & GPIO_PN_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PN_INPUT_BIT_ALL))))

#define IS_GPIO_PP_OUTPUT_BIT(param)     (((param) & GPIO_PP_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PP_OUTPUT_BIT_ALL))))

#define IS_GPIO_PP_FR1_BIT(param)        (((param) & GPIO_PP_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PP_FR1_BIT_ALL))))

#define IS_GPIO_PP_FR2_BIT(param)        (((param) & GPIO_PP_FR2_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PP_FR2_BIT_ALL))))

#define IS_GPIO_PP_FR3_BIT(param)        (((param) & GPIO_PP_FR3_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PP_FR3_BIT_ALL))))

#define IS_GPIO_PP_PULLUP_BIT(param)     (((param) & GPIO_PP_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PP_PULLUP_BIT_ALL))))

#define IS_GPIO_PP_INPUT_BIT(param)      (((param) & GPIO_PP_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PP_INPUT_BIT_ALL))))

#define IS_GPIO_PR_OUTPUT_BIT(param)     (((param) & GPIO_PR_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PR_OUTPUT_BIT_ALL))))

#define IS_GPIO_PR_FR1_BIT(param)        (((param) & GPIO_PR_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PR_FR1_BIT_ALL))))

#define IS_GPIO_PR_FR2_BIT(param)        (((param) & GPIO_PR_FR2_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PR_FR2_BIT_ALL))))

#define IS_GPIO_PR_FR3_BIT(param)        (((param) & GPIO_PR_FR3_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PR_FR3_BIT_ALL))))

#define IS_GPIO_PR_OPEN_DRAIN_BIT(param) (((param) & GPIO_PR_OPEN_DRAIN_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PR_OPEN_DRAIN_BIT_ALL))))

#define IS_GPIO_PR_PULLUP_BIT(param)     (((param) & GPIO_PR_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PR_PULLUP_BIT_ALL))))

#define IS_GPIO_PR_INPUT_BIT(param)      (((param) & GPIO_PR_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PR_INPUT_BIT_ALL))))

#define IS_GPIO_PT_OUTPUT_BIT(param)     (((param) & GPIO_PT_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PT_OUTPUT_BIT_ALL))))

#define IS_GPIO_PT_FR1_BIT(param)        (((param) & GPIO_PT_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PT_FR1_BIT_ALL))))

#define IS_GPIO_PT_PULLUP_BIT(param)     (((param) & GPIO_PT_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PT_PULLUP_BIT_ALL))))

#define IS_GPIO_PT_INPUT_BIT(param)      (((param) & GPIO_PT_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PT_INPUT_BIT_ALL))))

#define IS_GPIO_PU_OUTPUT_BIT(param)     (((param) & GPIO_PU_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PU_OUTPUT_BIT_ALL))))

#define IS_GPIO_PU_FR1_BIT(param)        (((param) & GPIO_PU_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PU_FR1_BIT_ALL))))

#define IS_GPIO_PU_PULLUP_BIT(param)     (((param) & GPIO_PU_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PU_PULLUP_BIT_ALL))))

#define IS_GPIO_PU_INPUT_BIT(param)      (((param) & GPIO_PU_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PU_INPUT_BIT_ALL))))

#define IS_GPIO_PV_OUTPUT_BIT(param)     (((param) & GPIO_PV_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PV_OUTPUT_BIT_ALL))))

#define IS_GPIO_PV_FR1_BIT(param)        (((param) & GPIO_PV_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PV_FR1_BIT_ALL))))

#define IS_GPIO_PV_PULLUP_BIT(param)     (((param) & GPIO_PV_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PV_PULLUP_BIT_ALL))))

#define IS_GPIO_PV_INPUT_BIT(param)      (((param) & GPIO_PV_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PV_INPUT_BIT_ALL))))

#define IS_GPIO_PW_OUTPUT_BIT(param)     (((param) & GPIO_PW_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PW_OUTPUT_BIT_ALL))))

#define IS_GPIO_PW_FR1_BIT(param)        (((param) & GPIO_PW_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PW_FR1_BIT_ALL))))

#define IS_GPIO_PW_OPEN_DRAIN_BIT(param) (((param) & GPIO_PW_OPEN_DRAIN_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PW_OPEN_DRAIN_BIT_ALL))))

#define IS_GPIO_PW_PULLUP_BIT(param)     (((param) & GPIO_PW_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PW_PULLUP_BIT_ALL))))

#define IS_GPIO_PW_INPUT_BIT(param)      (((param) & GPIO_PW_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PW_INPUT_BIT_ALL))))

#define IS_GPIO_PY_OUTPUT_BIT(param)     (((param) & GPIO_PY_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PY_OUTPUT_BIT_ALL))))

#define IS_GPIO_PY_FR1_BIT(param)        (((param) & GPIO_PY_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PY_FR1_BIT_ALL))))

#define IS_GPIO_PY_PULLUP_BIT(param)     (((param) & GPIO_PY_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PY_PULLUP_BIT_ALL))))

#define IS_GPIO_PY_INPUT_BIT(param)      (((param) & GPIO_PY_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PY_INPUT_BIT_ALL))))

#define IS_GPIO_PAA_PULLUP_BIT(param)     (((param) & GPIO_PAA_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAA_PULLUP_BIT_ALL))))

#define IS_GPIO_PAA_INPUT_BIT(param)      (((param) & GPIO_PAA_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAA_INPUT_BIT_ALL))))

#define IS_GPIO_PAB_PULLUP_BIT(param)     (((param) & GPIO_PAB_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAB_PULLUP_BIT_ALL))))

#define IS_GPIO_PAB_INPUT_BIT(param)      (((param) & GPIO_PAB_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAB_INPUT_BIT_ALL))))

#define IS_GPIO_PAC_PULLUP_BIT(param)     (((param) & GPIO_PAC_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAC_PULLUP_BIT_ALL))))

#define IS_GPIO_PAC_INPUT_BIT(param)      (((param) & GPIO_PAC_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAC_INPUT_BIT_ALL))))

#define IS_GPIO_PAD_OUTPUT_BIT(param)     (((param) & GPIO_PAD_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAD_OUTPUT_BIT_ALL))))

#define IS_GPIO_PAD_FR1_BIT(param)        (((param) & GPIO_PAD_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAD_FR1_BIT_ALL))))

#define IS_GPIO_PAD_PULLUP_BIT(param)     (((param) & GPIO_PAD_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAD_PULLUP_BIT_ALL))))

#define IS_GPIO_PAD_INPUT_BIT(param)      (((param) & GPIO_PAD_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAD_INPUT_BIT_ALL))))

#define IS_GPIO_PAE_OUTPUT_BIT(param)     (((param) & GPIO_PAE_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAE_OUTPUT_BIT_ALL))))

#define IS_GPIO_PAE_FR1_BIT(param)        (((param) & GPIO_PAE_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAE_FR1_BIT_ALL))))

#define IS_GPIO_PAE_PULLUP_BIT(param)     (((param) & GPIO_PAE_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAE_PULLUP_BIT_ALL))))

#define IS_GPIO_PAE_INPUT_BIT(param)      (((param) & GPIO_PAE_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAE_INPUT_BIT_ALL))))

#define IS_GPIO_PAF_OUTPUT_BIT(param)     (((param) & GPIO_PAF_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAF_OUTPUT_BIT_ALL))))

#define IS_GPIO_PAF_FR1_BIT(param)        (((param) & GPIO_PAF_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAF_FR1_BIT_ALL))))

#define IS_GPIO_PAF_PULLUP_BIT(param)     (((param) & GPIO_PAF_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAF_PULLUP_BIT_ALL))))

#define IS_GPIO_PAF_INPUT_BIT(param)      (((param) & GPIO_PAF_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAF_INPUT_BIT_ALL))))

#define IS_GPIO_PAG_OUTPUT_BIT(param)     (((param) & GPIO_PAG_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAG_OUTPUT_BIT_ALL))))

#define IS_GPIO_PAG_FR1_BIT(param)        (((param) & GPIO_PAG_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAG_FR1_BIT_ALL))))

#define IS_GPIO_PAG_FR2_BIT(param)        (((param) & GPIO_PAG_FR2_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAG_FR2_BIT_ALL))))

#define IS_GPIO_PAG_PULLUP_BIT(param)     (((param) & GPIO_PAG_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAG_PULLUP_BIT_ALL))))

#define IS_GPIO_PAG_INPUT_BIT(param)      (((param) & GPIO_PAG_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAG_INPUT_BIT_ALL))))

#define IS_GPIO_PAH_OUTPUT_BIT(param)     (((param) & GPIO_PAH_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAH_OUTPUT_BIT_ALL))))

#define IS_GPIO_PAH_PULLUP_BIT(param)     (((param) & GPIO_PAH_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAH_PULLUP_BIT_ALL))))

#define IS_GPIO_PAH_INPUT_BIT(param)      (((param) & GPIO_PAH_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAH_INPUT_BIT_ALL))))

#define IS_GPIO_PAJ_OUTPUT_BIT(param)     (((param) & GPIO_PAJ_OUTPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAJ_OUTPUT_BIT_ALL))))

#define IS_GPIO_PAJ_FR1_BIT(param)        (((param) & GPIO_PAJ_FR1_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAJ_FR1_BIT_ALL))))

#define IS_GPIO_PAJ_OPEN_DRAIN_BIT(param) (((param) & GPIO_PAJ_OPEN_DRAIN_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAJ_OPEN_DRAIN_BIT_ALL))))

#define IS_GPIO_PAJ_PULLUP_BIT(param)     (((param) & GPIO_PAJ_PULLUP_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAJ_PULLUP_BIT_ALL))))

#define IS_GPIO_PAJ_INPUT_BIT(param)      (((param) & GPIO_PAJ_INPUT_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PAJ_INPUT_BIT_ALL))))

/** for open drain register 2 */

#define IS_GPIO_OPEN_DRAIN2(param)        ((param) == GPIO_PH)
#define GPIO_PH_OPEN_DRAIN2_BIT_ALL       ((uint8_t)0x7F)
#define IS_GPIO_PH_OPEN_DRAIN2_BIT(param) (((param) & GPIO_PH_OPEN_DRAIN2_BIT_ALL)&& \
                                          (!((param) &(uint8_t) (~GPIO_PH_OPEN_DRAIN2_BIT_ALL))))
/* End of open drain register 2 */

/** @} */
/* End of group GPIO_Bit_Define */

/** @defgroup GPIO_Exported_FunctionPrototypes
  * @{
  */
    uint8_t GPIO_ReadData(GPIO_Port GPIO_x);
    uint8_t GPIO_ReadDataBit(GPIO_Port GPIO_x, uint8_t Bit_x);
    void GPIO_WriteData(GPIO_Port GPIO_x, uint8_t Data);
    void GPIO_WriteDataBit(GPIO_Port GPIO_x, uint8_t Bit_x, uint8_t BitValue);
    void GPIO_Init(GPIO_Port GPIO_x, uint8_t Bit_x, GPIO_InitTypeDef * GPIO_InitStruct);
    void GPIO_SetOutput(GPIO_Port GPIO_x, uint8_t Bit_x);
    void GPIO_SetInput(GPIO_Port GPIO_x, uint8_t Bit_x);
    void GPIO_SetInputEnableReg(GPIO_Port GPIO_x, uint8_t Bit_x, FunctionalState NewState);
    void GPIO_SetOutputEnableReg(GPIO_Port GPIO_x, uint8_t Bit_x, FunctionalState NewState);
    void GPIO_SetPullUp(GPIO_Port GPIO_x, uint8_t Bit_x, FunctionalState NewState);
    void GPIO_SetPullDown(GPIO_Port GPIO_x, uint8_t Bit_x, FunctionalState NewState);
    void GPIO_SetOpenDrain(GPIO_Port GPIO_x, uint8_t Bit_x, FunctionalState NewState);
    void GPIO_SetOpenDrain2(GPIO_Port GPIO_x, uint8_t Bit_x, FunctionalState NewState);
    void GPIO_EnableFuncReg(GPIO_Port GPIO_x, uint8_t FuncReg_x, uint8_t Bit_x);
    void GPIO_DisableFuncReg(GPIO_Port GPIO_x, uint8_t FuncReg_x, uint8_t Bit_x);
    void GPIO_WriteDataByte(GPIO_Port GPIO_x, uint8_t Bit_x, uint8_t value);
    void GPIO_ToggleDataByte(GPIO_Port GPIO_x, uint8_t Bit_x);
/** @} */
/* End of group GPIO_Exported_FunctionPrototypes */
/** @} */
/* End of group GPIO */
/** @} */
/* End of group TX04_Periph_Driver */
#ifdef __cplusplus
}
#endif                          /* __cplusplus */
#endif                          /* __TMPM440_GPIO_H */
