/**
 *******************************************************************************
 * @file    tmpm440_kwup.h
 * @brief   This file provides all the functions prototypes for KWUP driver.
 * @version V2.0.2.3
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __TMPM440_KWUP_H
#define __TMPM440_KWUP_H

#ifdef __cplusplus
extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "TMPM440.h"
#include "tx04_common.h"
#ifdef __IAR_SYSTEMS_ICC__ /*IAR*/
#if 1 != __LITTLE_ENDIAN__
#pragma bitfields=disjoint_types
#endif
#endif
/** @addtogroup TX04_Periph_Driver
  * @{
  */
/** @addtogroup KWUP
  * @{
  */
/** @defgroup KWUP_Exported_Types
  * @{
  */
#define IS_KWUP_ALL_UNIT(param)          (((param) == TSB_KWUPA) || \
				                         ((param) == TSB_KWUPB))
        typedef enum {
        KWUP_INPUT_0 = 0U,
        KWUP_INPUT_1 = 1U,
        KWUP_INPUT_2 = 2U,
        KWUP_INPUT_3 = 3U,
        KWUP_INPUT_4 = 4U,
        KWUP_INPUT_5 = 5U,
        KWUP_INPUT_6 = 6U,
        KWUP_INPUT_7 = 7U,
        KWUP_INPUT_8 = 8U,
        KWUP_INPUT_9 = 9U,
        KWUP_INPUT_10 = 10U,
        KWUP_INPUT_11 = 11U,
        KWUP_INPUT_12 = 12U,
        KWUP_INPUT_13 = 13U,
        KWUP_INPUT_14 = 14U,
        KWUP_INPUT_15 = 15U,
        KWUP_INPUT_16 = 16U,
        KWUP_INPUT_17 = 17U,
        KWUP_INPUT_18 = 18U,
        KWUP_INPUT_19 = 19U,
        KWUP_INPUT_20 = 20U,
        KWUP_INPUT_21 = 21U,
        KWUP_INPUT_22 = 22U,
        KWUP_INPUT_23 = 23U,
        KWUP_INPUT_24 = 24U,
        KWUP_INPUT_25 = 25U,
        KWUP_INPUT_26 = 26U,
        KWUP_INPUT_27 = 27U,
        KWUP_INPUT_28 = 28U,
        KWUP_INPUT_29 = 29U,
        KWUP_INPUT_30 = 30U,
        KWUP_INPUT_31 = 31U
    } KWUP_Input;
#define IS_KWUPA_INPUT(param)         (((param) <= KWUP_INPUT_31))
#define IS_KWUPB_INPUT(param)         (((param) <= KWUP_INPUT_7))

    typedef enum {
        KWUP_PUP_CTRL_BY_STATIC = 0U,
        KWUP_PUP_CTRL_BY_DYNAMIC = 1U
    } KWUP_PullUpCtrl;
#define IS_KWUP_PULL_UP_CTRL(param)         (((param) == KWUP_PUP_CTRL_BY_STATIC) || \
                                             ((param) == KWUP_PUP_CTRL_BY_DYNAMIC))

    typedef enum {
        KWUP_ACTIVE_BY_L_LEVEL = 0U,
        KWUP_ACTIVE_BY_H_LEVEL = 1U,
        KWUP_ACTIVE_BY_FALLING_EDGE = 2U,
        KWUP_ACTIVE_BY_RISING_EDGE = 3U,
        KWUP_ACTIVE_BY_BOTH_EDGES = 4U
    } KWUP_ActiveState;
#define IS_KWUP_ACTIVE_STATE(param)         (((param) == KWUP_ACTIVE_BY_L_LEVEL) || \
                                             ((param) == KWUP_ACTIVE_BY_H_LEVEL) || \
                                             ((param) == KWUP_ACTIVE_BY_RISING_EDGE) || \
                                             ((param) == KWUP_ACTIVE_BY_FALLING_EDGE)|| \
                                             ((param) == KWUP_ACTIVE_BY_BOTH_EDGES))

    typedef enum {
        KWUP_CYCLES_2_FS = 0U,
        KWUP_CYCLES_4_FS = 1U,
        KWUP_CYCLES_8_FS = 2U,
        KWUP_CYCLES_16_FS = 3U,
        KWUP_CYCLES_256_FS = 4U,
        KWUP_CYCLES_512_FS = 5U,
        KWUP_CYCLES_1024_FS = 6U,
        KWUP_CYCLES_2048_FS = 7U,
    } KWUP_PullUpCycles;
#define IS_KWUP_PULL_UP_PERIOD_T1(param)         (((param) == KWUP_CYCLES_2_FS) || \
                                                  ((param) == KWUP_CYCLES_4_FS) || \
                                                  ((param) == KWUP_CYCLES_8_FS) || \
                                                  ((param) == KWUP_CYCLES_16_FS))
#define IS_KWUP_PULL_UP_CYCLES_T2(param)         (((param) == KWUP_CYCLES_256_FS) || \
                                                  ((param) == KWUP_CYCLES_512_FS) || \
                                                  ((param) == KWUP_CYCLES_1024_FS) || \
                                                  ((param) == KWUP_CYCLES_2048_FS))


    typedef struct {
        KWUP_Input KeyN;
        KWUP_PullUpCtrl PullUpCtrl;
        KWUP_ActiveState ActiveState;
        FunctionalState INTNewState;
    } KWUP_SettingTypeDef;
#if defined ( __CC_ARM )        /*Keil */
#if defined( __BIG_ENDIAN )
    typedef union {
        uint32_t All;
        struct {
            uint32_t Key31:1;
            uint32_t Key30:1;
            uint32_t Key29:1;
            uint32_t Key28:1;
            uint32_t Key27:1;
            uint32_t Key26:1;
            uint32_t Key25:1;
            uint32_t Key24:1;
            uint32_t Key23:1;
            uint32_t Key22:1;
            uint32_t Key21:1;
            uint32_t Key20:1;
            uint32_t Key19:1;
            uint32_t Key18:1;
            uint32_t Key17:1;
            uint32_t Key16:1;
            uint32_t Key15:1;
            uint32_t Key14:1;
            uint32_t Key13:1;
            uint32_t Key12:1;
            uint32_t Key11:1;
            uint32_t Key10:1;
            uint32_t Key9:1;
            uint32_t Key8:1;
            uint32_t Key7:1;
            uint32_t Key6:1;
            uint32_t Key5:1;
            uint32_t Key4:1;
            uint32_t Key3:1;
            uint32_t Key2:1;
            uint32_t Key1:1;
            uint32_t Key0:1;
        } Bit;
    } KWUP_PortStatus;

    typedef union {
        uint32_t All;
        struct {
            uint32_t Key31:1;
            uint32_t Key30:1;
            uint32_t Key29:1;
            uint32_t Key28:1;
            uint32_t Key27:1;
            uint32_t Key26:1;
            uint32_t Key25:1;
            uint32_t Key24:1;
            uint32_t Key23:1;
            uint32_t Key22:1;
            uint32_t Key21:1;
            uint32_t Key20:1;
            uint32_t Key19:1;
            uint32_t Key18:1;
            uint32_t Key17:1;
            uint32_t Key16:1;
            uint32_t Key15:1;
            uint32_t Key14:1;
            uint32_t Key13:1;
            uint32_t Key12:1;
            uint32_t Key11:1;
            uint32_t Key10:1;
            uint32_t Key9:1;
            uint32_t Key8:1;
            uint32_t Key7:1;
            uint32_t Key6:1;
            uint32_t Key5:1;
            uint32_t Key4:1;
            uint32_t Key3:1;
            uint32_t Key2:1;
            uint32_t Key1:1;
            uint32_t Key0:1;
        } Bit;
    } KWUP_INTStatus;
#else
    typedef union {
        uint32_t All;
        struct {
            uint32_t Key0:1;
            uint32_t Key1:1;
            uint32_t Key2:1;
            uint32_t Key3:1;
            uint32_t Key4:1;
            uint32_t Key5:1;
            uint32_t Key6:1;
            uint32_t Key7:1;
            uint32_t Key8:1;
            uint32_t Key9:1;
            uint32_t Key10:1;
            uint32_t Key11:1;
            uint32_t Key12:1;
            uint32_t Key13:1;
            uint32_t Key14:1;
            uint32_t Key15:1;
            uint32_t Key16:1;
            uint32_t Key17:1;
            uint32_t Key18:1;
            uint32_t Key19:1;
            uint32_t Key20:1;
            uint32_t Key21:1;
            uint32_t Key22:1;
            uint32_t Key23:1;
            uint32_t Key24:1;
            uint32_t Key25:1;
            uint32_t Key26:1;
            uint32_t Key27:1;
            uint32_t Key28:1;
            uint32_t Key29:1;
            uint32_t Key30:1;
            uint32_t Key31:1;
        } Bit;
    } KWUP_PortStatus;

    typedef union {
        uint32_t All;
        struct {
            uint32_t Key0:1;
            uint32_t Key1:1;
            uint32_t Key2:1;
            uint32_t Key3:1;
            uint32_t Key4:1;
            uint32_t Key5:1;
            uint32_t Key6:1;
            uint32_t Key7:1;
            uint32_t Key8:1;
            uint32_t Key9:1;
            uint32_t Key10:1;
            uint32_t Key11:1;
            uint32_t Key12:1;
            uint32_t Key13:1;
            uint32_t Key14:1;
            uint32_t Key15:1;
            uint32_t Key16:1;
            uint32_t Key17:1;
            uint32_t Key18:1;
            uint32_t Key19:1;
            uint32_t Key20:1;
            uint32_t Key21:1;
            uint32_t Key22:1;
            uint32_t Key23:1;
            uint32_t Key24:1;
            uint32_t Key25:1;
            uint32_t Key26:1;
            uint32_t Key27:1;
            uint32_t Key28:1;
            uint32_t Key29:1;
            uint32_t Key30:1;
            uint32_t Key31:1;
        } Bit;
    } KWUP_INTStatus;
#endif
#elif  defined(__IAR_SYSTEMS_ICC__)
    typedef union {
        uint32_t All;
        struct {
            uint32_t Key0:1;
            uint32_t Key1:1;
            uint32_t Key2:1;
            uint32_t Key3:1;
            uint32_t Key4:1;
            uint32_t Key5:1;
            uint32_t Key6:1;
            uint32_t Key7:1;
            uint32_t Key8:1;
            uint32_t Key9:1;
            uint32_t Key10:1;
            uint32_t Key11:1;
            uint32_t Key12:1;
            uint32_t Key13:1;
            uint32_t Key14:1;
            uint32_t Key15:1;
            uint32_t Key16:1;
            uint32_t Key17:1;
            uint32_t Key18:1;
            uint32_t Key19:1;
            uint32_t Key20:1;
            uint32_t Key21:1;
            uint32_t Key22:1;
            uint32_t Key23:1;
            uint32_t Key24:1;
            uint32_t Key25:1;
            uint32_t Key26:1;
            uint32_t Key27:1;
            uint32_t Key28:1;
            uint32_t Key29:1;
            uint32_t Key30:1;
            uint32_t Key31:1;
        } Bit;
    } KWUP_PortStatus;

    typedef union {
        uint32_t All;
        struct {
            uint32_t Key0:1;
            uint32_t Key1:1;
            uint32_t Key2:1;
            uint32_t Key3:1;
            uint32_t Key4:1;
            uint32_t Key5:1;
            uint32_t Key6:1;
            uint32_t Key7:1;
            uint32_t Key8:1;
            uint32_t Key9:1;
            uint32_t Key10:1;
            uint32_t Key11:1;
            uint32_t Key12:1;
            uint32_t Key13:1;
            uint32_t Key14:1;
            uint32_t Key15:1;
            uint32_t Key16:1;
            uint32_t Key17:1;
            uint32_t Key18:1;
            uint32_t Key19:1;
            uint32_t Key20:1;
            uint32_t Key21:1;
            uint32_t Key22:1;
            uint32_t Key23:1;
            uint32_t Key24:1;
            uint32_t Key25:1;
            uint32_t Key26:1;
            uint32_t Key27:1;
            uint32_t Key28:1;
            uint32_t Key29:1;
            uint32_t Key30:1;
            uint32_t Key31:1;
        } Bit;
    } KWUP_INTStatus;
#endif

/** @} */
/* End of group KWUP_Exported_Types */

/** @defgroup KWUP_Exported_FunctionPrototypes
  * @{
  */
    void KWUP_SetConfig(TSB_KWUP_TypeDef * TSB_KWUPx, KWUP_SettingTypeDef * Settings);
    KWUP_PortStatus KWUP_GetPortStatus(TSB_KWUP_TypeDef * TSB_KWUPx);
    void KWUP_SetPullUpConfig(TSB_KWUP_TypeDef * TSB_KWUPx, KWUP_PullUpCycles T1,
                              KWUP_PullUpCycles T2);
    KWUP_INTStatus KWUP_GetINTStatus(TSB_KWUP_TypeDef * TSB_KWUPx);

/** @} */
/* End of group KWUP_Exported_FunctionPrototypes */

/** @} */
/* End of group KWUP */

/** @} */
/* End of group TX04_Periph_Driver */

#ifdef __cplusplus
}
#endif
#endif                          /* __TMPM440_KWUP_H */
