/**
 *******************************************************************************
 * @file    tmpm440_psc.h
 * @brief   This file provides all the functions prototypes for PSC driver.
 * @version V2.0.2.4
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __TMPM440_PSC_H
#define __TMPM440_PSC_H

#ifdef __cplusplus
extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "TMPM440.h"
#include "tx04_common.h"

#ifdef __IAR_SYSTEMS_ICC__ /*IAR*/
#if 1 != __LITTLE_ENDIAN__
#pragma bitfields=disjoint_types
#endif
#endif
#if defined(__TMPM440_PSC_H)
/** @addtogroup TX04_Periph_Driver
  * @{
  */
/** @defgroup PSC 
  * @{
  */
/** @defgroup PSC_Exported_Constants
  * @{
  */
#define PSC_TRIGGER_EVENT_0                  (0x00000001U << 0U)
#define PSC_TRIGGER_EVENT_1                  (0x00000001U << 1U)
#define PSC_TRIGGER_EVENT_2                  (0x00000001U << 2U)
#define PSC_TRIGGER_EVENT_3                  (0x00000001U << 3U)
#define PSC_TRIGGER_EVENT_4                  (0x00000001U << 4U)
#define PSC_TRIGGER_EVENT_5                  (0x00000001U << 5U)
#define PSC_TRIGGER_EVENT_6                  (0x00000001U << 6U)
#define PSC_TRIGGER_EVENT_7                  (0x00000001U << 7U)
#define PSC_TRIGGER_EVENT_8                  (0x00000001U << 8U)
#define PSC_TRIGGER_EVENT_9                  (0x00000001U << 9U)
#define PSC_TRIGGER_EVENT_10                 (0x00000001U << 10U)
#define PSC_TRIGGER_EVENT_11                 (0x00000001U << 11U)
#define PSC_TRIGGER_EVENT_12                 (0x00000001U << 12U)
#define PSC_TRIGGER_EVENT_13                 (0x00000001U << 13U)
#define PSC_TRIGGER_EVENT_14                 (0x00000001U << 14U)
#define PSC_TRIGGER_EVENT_15                 (0x00000001U << 15U)
#define PSC_TRIGGER_EVENT_ALL                (0x0000FFFFU)
#define IS_PSC_TRIGGER_EVENT(param)          (((param) > 0U) && ((param) <= PSC_TRIGGER_EVENT_ALL))
#define IS_PSC_TRIGGER_SEL_EVENT(param)      (((param) == PSC_TRIGGER_EVENT_0) || \
                                              ((param) == PSC_TRIGGER_EVENT_1) || \
                                              ((param) == PSC_TRIGGER_EVENT_2) || \
                                              ((param) == PSC_TRIGGER_EVENT_3) || \
                                              ((param) == PSC_TRIGGER_EVENT_4) || \
                                              ((param) == PSC_TRIGGER_EVENT_5) || \
                                              ((param) == PSC_TRIGGER_EVENT_6) || \
                                              ((param) == PSC_TRIGGER_EVENT_7) || \
                                              ((param) == PSC_TRIGGER_EVENT_8) || \
                                              ((param) == PSC_TRIGGER_EVENT_9) || \
                                              ((param) == PSC_TRIGGER_EVENT_10) || \
                                              ((param) == PSC_TRIGGER_EVENT_11) || \
                                              ((param) == PSC_TRIGGER_EVENT_12) || \
                                              ((param) == PSC_TRIGGER_EVENT_13) || \
                                              ((param) == PSC_TRIGGER_EVENT_14) || \
                                              ((param) == PSC_TRIGGER_EVENT_15))
#define PSC_REN_EDGE_MASK                     (0x00000F00U)
#define PSC_REN_EDGE_MASK_BITWISE             (0xFFFFF0FFU)
#define IS_PSC_TRIGGER_EDGE_EVENT(param)      ((((param) & PSC_REN_EDGE_MASK) != 0U) && \
                                               (((param) & PSC_REN_EDGE_MASK_BITWISE) == 0U))
#define IS_PSC_TRIGGER_EDGE_EVENT_SEL(param)  (((param) == PSC_TRIGGER_EVENT_8) || \
                                               ((param) == PSC_TRIGGER_EVENT_9) || \
                                               ((param) == PSC_TRIGGER_EVENT_10) || \
                                               ((param) == PSC_TRIGGER_EVENT_11))
#define PSC_BIT_0                           ((uint8_t)0x01)
#define PSC_BIT_1                           ((uint8_t)0x02)
#define PSC_BIT_2                           ((uint8_t)0x04)
#define PSC_BIT_3                           ((uint8_t)0x08)
#define PSC_BIT_4                           ((uint8_t)0x10)
#define PSC_BIT_5                           ((uint8_t)0x20)
#define PSC_BIT_6                           ((uint8_t)0x40)
#define PSC_BIT_7                           ((uint8_t)0x80)
#define PSC_BIT_ALL                         ((uint8_t)0xFF)
#define IS_PSC_BIT(param)                   (((param) == PSC_BIT_0)|| \
                                             ((param) == PSC_BIT_1)|| \
                                             ((param) == PSC_BIT_2)|| \
                                             ((param) == PSC_BIT_3)|| \
                                             ((param) == PSC_BIT_4)|| \
                                             ((param) == PSC_BIT_5)|| \
                                             ((param) == PSC_BIT_6)|| \
                                             ((param) == PSC_BIT_7))
#define PSC_BIT_VALUE_1                     ((uint8_t)0x01)
#define PSC_BIT_VALUE_0                     ((uint8_t)0x00)
#define IS_PSC_BIT_VALUE(param)             (((param) == PSC_BIT_VALUE_1)|| \
                                             ((param) == PSC_BIT_VALUE_0))
/** @} */
/* End of group PSC_Exported_Constants */
/** @defgroup PSC_Exported_Types
  * @{
  */
/**
  * @brief Arithmetic parameter signed register
  */
#if defined ( __CC_ARM )
#ifdef __BIG_ENDIAN
        typedef union {
        uint32_t All;
        struct {
            uint32_t Reserved1:9;       /* bit 31-23 */
            uint32_t SignUR1:1; /* bit 22 */
            uint32_t SignUR0:1; /* bit 21 */
            uint32_t SignUL1:1; /* bit 20 */
            uint32_t SignUL0:1; /* bit 19 */
            uint32_t SignUM1:1; /* bit 18 */
            uint32_t SignUM0:1; /* bit 17 */
            uint32_t SignUA0:1; /* bit 16 */
            uint32_t Reserved0:16;      /* bit 15-0 */
        } Bit;
    } PSC_ArithmeticPARAM;
#else
        typedef union {
        uint32_t All;
        struct {
            uint32_t Reserved0:16;      /* bit 0-15 */
            uint32_t SignUA0:1; /* bit 16 */
            uint32_t SignUM0:1; /* bit 17 */
            uint32_t SignUM1:1; /* bit 18 */
            uint32_t SignUL0:1; /* bit 19 */
            uint32_t SignUL1:1; /* bit 20 */
            uint32_t SignUR0:1; /* bit 21 */
            uint32_t SignUR1:1; /* bit 22 */
            uint32_t Reserved1:9;       /* bit 23-31 */
        } Bit;
    } PSC_ArithmeticPARAM;
#endif
#elif  defined(__IAR_SYSTEMS_ICC__)
        typedef union {
        uint32_t All;
        struct {
            uint32_t Reserved0:16;      /* bit 0-15 */
            uint32_t SignUA0:1; /* bit 16 */
            uint32_t SignUM0:1; /* bit 17 */
            uint32_t SignUM1:1; /* bit 18 */
            uint32_t SignUL0:1; /* bit 19 */
            uint32_t SignUL1:1; /* bit 20 */
            uint32_t SignUR0:1; /* bit 21 */
            uint32_t SignUR1:1; /* bit 22 */
            uint32_t Reserved1:9;       /* bit 23-31 */
        } Bit;
    } PSC_ArithmeticPARAM;
#endif

/**
  * @brief Arithmetic result register
  */
#if defined ( __CC_ARM )
#ifdef __BIG_ENDIAN
    typedef union {
        uint32_t All;
        struct {
            uint32_t Reserved2:7;       /* bit 31-25 */
            uint32_t Zero:1;    /* bit 24 */
            uint32_t Reserved1:6;       /* bit 23-18 */
            uint32_t UnderFlow:1;       /* bit 17 */
            uint32_t OverFlow:1;        /* bit 16 */
            uint32_t Reserved0:16;      /* bit 15-0 */
        } Bit;
    } PSC_ArithmeticResult;
#else
    typedef union {
        uint32_t All;
        struct {
            uint32_t Reserved0:16;      /* bit 0-15 */
            uint32_t OverFlow:1;        /* bit 16 */
            uint32_t UnderFlow:1;       /* bit 17 */
            uint32_t Reserved1:6;       /* bit 18-23 */
            uint32_t Zero:1;    /* bit 24 */
            uint32_t Reserved2:7;       /* bit 25-31 */
        } Bit;
    } PSC_ArithmeticResult;
#endif
#elif  defined(__IAR_SYSTEMS_ICC__)
    typedef union {
        uint32_t All;
        struct {
            uint32_t Reserved0:16;      /* bit 0-15 */
            uint32_t OverFlow:1;        /* bit 16 */
            uint32_t UnderFlow:1;       /* bit 17 */
            uint32_t Reserved1:6;       /* bit 18-23 */
            uint32_t Zero:1;    /* bit 24 */
            uint32_t Reserved2:7;       /* bit 25-31 */
        } Bit;
    } PSC_ArithmeticResult;
#endif

/**
  * @brief Overrun flag caused by the startup trigger event No.15 to 0.
  */
#if defined ( __CC_ARM )
#ifdef __BIG_ENDIAN
    typedef union {
        uint32_t All;
        struct {
            uint32_t INTOVRF15:1;       /* bit 31 */
            uint32_t INTOVRF14:1;       /* bit 30 */
            uint32_t INTOVRF13:1;       /* bit 29 */
            uint32_t INTOVRF12:1;       /* bit 28 */
            uint32_t INTOVRF11:1;       /* bit 27 */
            uint32_t INTOVRF10:1;       /* bit 26 */
            uint32_t INTOVRF9:1;        /* bit 25 */
            uint32_t INTOVRF8:1;        /* bit 24 */
            uint32_t INTOVRF7:1;        /* bit 23 */
            uint32_t INTOVRF6:1;        /* bit 22 */
            uint32_t INTOVRF5:1;        /* bit 21 */
            uint32_t INTOVRF4:1;        /* bit 20 */
            uint32_t INTOVRF3:1;        /* bit 19 */
            uint32_t INTOVRF2:1;        /* bit 18 */
            uint32_t INTOVRF1:1;        /* bit 17 */
            uint32_t INTOVRF0:1;        /* bit 16 */
            uint32_t Reserved:16;       /* bit 15-0 */
        } Bit;
    } PSC_OverRunFlag;
#else
    typedef union {
        uint32_t All;
        struct {
            uint32_t Reserved:16;       /* bit 0-15 */
            uint32_t INTOVRF0:1;        /* bit 16 */
            uint32_t INTOVRF1:1;        /* bit 17 */
            uint32_t INTOVRF2:1;        /* bit 18 */
            uint32_t INTOVRF3:1;        /* bit 19 */
            uint32_t INTOVRF4:1;        /* bit 20 */
            uint32_t INTOVRF5:1;        /* bit 21 */
            uint32_t INTOVRF6:1;        /* bit 22 */
            uint32_t INTOVRF7:1;        /* bit 23 */
            uint32_t INTOVRF8:1;        /* bit 24 */
            uint32_t INTOVRF9:1;        /* bit 25 */
            uint32_t INTOVRF10:1;       /* bit 26 */
            uint32_t INTOVRF11:1;       /* bit 27 */
            uint32_t INTOVRF12:1;       /* bit 28 */
            uint32_t INTOVRF13:1;       /* bit 29 */
            uint32_t INTOVRF14:1;       /* bit 30 */
            uint32_t INTOVRF15:1;       /* bit 31 */
        } Bit;
    } PSC_OverRunFlag;
#endif
#elif  defined(__IAR_SYSTEMS_ICC__)
    typedef union {
        uint32_t All;
        struct {
            uint32_t Reserved:16;       /* bit 0-15 */
            uint32_t INTOVRF0:1;        /* bit 16 */
            uint32_t INTOVRF1:1;        /* bit 17 */
            uint32_t INTOVRF2:1;        /* bit 18 */
            uint32_t INTOVRF3:1;        /* bit 19 */
            uint32_t INTOVRF4:1;        /* bit 20 */
            uint32_t INTOVRF5:1;        /* bit 21 */
            uint32_t INTOVRF6:1;        /* bit 22 */
            uint32_t INTOVRF7:1;        /* bit 23 */
            uint32_t INTOVRF8:1;        /* bit 24 */
            uint32_t INTOVRF9:1;        /* bit 25 */
            uint32_t INTOVRF10:1;       /* bit 26 */
            uint32_t INTOVRF11:1;       /* bit 27 */
            uint32_t INTOVRF12:1;       /* bit 28 */
            uint32_t INTOVRF13:1;       /* bit 29 */
            uint32_t INTOVRF14:1;       /* bit 30 */
            uint32_t INTOVRF15:1;       /* bit 31 */
        } Bit;
    } PSC_OverRunFlag;
#endif

/**
  * @brief Startup event occurrence flag caused by the startup trigger event No.15 to0.
  */
#if defined ( __CC_ARM )
#ifdef __BIG_ENDIAN
    typedef union {
        uint32_t All;
        struct {
            uint32_t Reserved:16;       /* bit 31-16 */
            uint32_t INTFLG15:1;        /* bit 15 */
            uint32_t INTFLG14:1;        /* bit 14 */
            uint32_t INTFLG13:1;        /* bit 13 */
            uint32_t INTFLG12:1;        /* bit 12 */
            uint32_t INTFLG11:1;        /* bit 11 */
            uint32_t INTFLG10:1;        /* bit 10 */
            uint32_t INTFLG9:1; /* bit 9 */
            uint32_t INTFLG8:1; /* bit 8 */
            uint32_t INTFLG7:1; /* bit 7 */
            uint32_t INTFLG6:1; /* bit 6 */
            uint32_t INTFLG5:1; /* bit 5 */
            uint32_t INTFLG4:1; /* bit 4 */
            uint32_t INTFLG3:1; /* bit 3 */
            uint32_t INTFLG2:1; /* bit 2 */
            uint32_t INTFLG1:1; /* bit 1 */
            uint32_t INTFLG0:1; /* bit 0 */
        } Bit;
    } PSC_StartupFlag;
#else
    typedef union {
        uint32_t All;
        struct {
            uint32_t INTFLG0:1; /* bit 0 */
            uint32_t INTFLG1:1; /* bit 1 */
            uint32_t INTFLG2:1; /* bit 2 */
            uint32_t INTFLG3:1; /* bit 3 */
            uint32_t INTFLG4:1; /* bit 4 */
            uint32_t INTFLG5:1; /* bit 5 */
            uint32_t INTFLG6:1; /* bit 6 */
            uint32_t INTFLG7:1; /* bit 7 */
            uint32_t INTFLG8:1; /* bit 8 */
            uint32_t INTFLG9:1; /* bit 9 */
            uint32_t INTFLG10:1;        /* bit 10 */
            uint32_t INTFLG11:1;        /* bit 11 */
            uint32_t INTFLG12:1;        /* bit 12 */
            uint32_t INTFLG13:1;        /* bit 13 */
            uint32_t INTFLG14:1;        /* bit 14 */
            uint32_t INTFLG15:1;        /* bit 15 */
            uint32_t Reserved:16;       /* bit 16-31 */
        } Bit;
    } PSC_StartupFlag;
#endif
#elif  defined(__IAR_SYSTEMS_ICC__)
    typedef union {
        uint32_t All;
        struct {
            uint32_t INTFLG0:1; /* bit 0 */
            uint32_t INTFLG1:1; /* bit 1 */
            uint32_t INTFLG2:1; /* bit 2 */
            uint32_t INTFLG3:1; /* bit 3 */
            uint32_t INTFLG4:1; /* bit 4 */
            uint32_t INTFLG5:1; /* bit 5 */
            uint32_t INTFLG6:1; /* bit 6 */
            uint32_t INTFLG7:1; /* bit 7 */
            uint32_t INTFLG8:1; /* bit 8 */
            uint32_t INTFLG9:1; /* bit 9 */
            uint32_t INTFLG10:1;        /* bit 10 */
            uint32_t INTFLG11:1;        /* bit 11 */
            uint32_t INTFLG12:1;        /* bit 12 */
            uint32_t INTFLG13:1;        /* bit 13 */
            uint32_t INTFLG14:1;        /* bit 14 */
            uint32_t INTFLG15:1;        /* bit 15 */
            uint32_t Reserved:16;       /* bit 16-31 */
        } Bit;
    } PSC_StartupFlag;
#endif

/**
  * @brief The edge of startup trigger event 8 to 11
  */
    typedef enum {
        PSC_START_TRIGGER_FALLING = 0U,
        PSC_START_TRIGGER_RISING = 1U
    } PSC_TriggerEventEdge;

#define IS_PSC_START_TRIGGER_EDGE(param)     (((param) == PSC_START_TRIGGER_FALLING) || \
                                              ((param) == PSC_START_TRIGGER_RISING))

/**
  * @brief The state of PSC program execution
  */
    typedef enum {
        PSC_PROGRAM_STOP = 0U,
        PSC_PROGRAM_EXECUTE = 1U
    } PSC_ExecutionState;

/** @} */
/* End of group PSC_Exported_Types */

/** @defgroup PSC_Exported_FunctionPrototypes
  * @{
  */
    void PSC_SetAccumulator(uint32_t UA0);
    uint32_t PSC_GetAccumulator(void);
    void PSC_SetMultiplReg(uint32_t UM0);
    uint32_t PSC_GetMultiplReg(void);
    void PSC_SetShiftCountReg(uint32_t UM1);
    uint32_t PSC_GetShiftCountReg(void);
    void PSC_SetUpperLimitReg(uint32_t UL0);
    uint32_t PSC_GetUpperLimitReg(void);
    void PSC_SetLowerLimitReg(uint32_t UL1);
    uint32_t PSC_GetLowerLimitReg(void);
    void PSC_SetAddSubReg0(uint32_t UR0);
    uint32_t PSC_GetAddSubReg0(void);
    void PSC_SetAddSubReg1(uint32_t UR1);
    uint32_t PSC_GetAddSubReg1(void);
    void PSC_SetArithmeticParameter(PSC_ArithmeticPARAM Param);
    PSC_ArithmeticPARAM PSC_GetArithmeticParameter(void);
    void PSC_SetAddrPointer0(uint32_t AP0);
    uint32_t PSC_GetAddrPointer0(void);
    void PSC_SetAddrPointer1(uint32_t AP1);
    uint32_t PSC_GetAddrPointer1(void);
    void PSC_SetAddrPointer2(uint32_t AP2);
    uint32_t PSC_GetAddrPointer2(void);
    void PSC_SetAddrPointer3(uint32_t AP3);
    uint32_t PSC_GetAddrPointer3(void);
    void PSC_SetBreakPointer(uint32_t BR0);
    uint32_t PSC_GetBreakPointer(void);
    void PSC_SetProgramPointer(uint32_t PG0);
    uint32_t PSC_GetProgramPointer(void);
    void PSC_SetRepetProcVectPointer(uint32_t VG0);
    uint32_t PSC_GetRepetProcVectPointer(void);
    void PSC_SetBreakFUNC(FunctionalState NewState);
    void PSC_SetStepFUNC(FunctionalState NewState);
    void PSC_SetExecutionFUNC(FunctionalState NewState);
    void PSC_SetArithmeticResult(PSC_ArithmeticResult ArithmeticResult);
    PSC_ArithmeticResult PSC_GetArithmeticResult(void);
    void PSC_SetTriggerEventFUNC(uint32_t Event, FunctionalState NewState);
    FunctionalState PSC_GetTriggerEventState(uint32_t Event);
    void PSC_SetTriggerEventEdge(uint32_t Event, PSC_TriggerEventEdge Edge);
    PSC_TriggerEventEdge PSC_GetTriggerEventEdge(uint32_t Event);
    PSC_OverRunFlag PSC_GetOverRunFlag(void);
    PSC_StartupFlag PSC_GetStartupFlag(void);
    void PSC_ClearEventOverrunFlag(uint32_t Event);
    void PSC_SetStartup(uint32_t Event);
    void PSC_SetOutputData(uint8_t Data);
    void PSC_SetOutputDataBit(uint8_t Bit_x, uint8_t BitValue);
    void PSC_SetDataOutputFUNC(uint8_t Bit_x, FunctionalState NewState);
    FunctionalState PSC_GetDataOutputState(uint8_t Bit_x);
    uint8_t PSC_GetInputData(void);
    uint8_t PSC_GetInputDataBit(uint8_t Bit_x);
    PSC_ExecutionState PSC_GetExecutionState(void);
/** @} */
/* End of group PSC_Exported_FunctionPrototypes */

/** @} */
/* End of group PSC */

/** @} */
/* End of group TX04_Periph_Driver */

#endif                          /* defined(__TMPM440_PSC_H) */

#ifdef __cplusplus
}
#endif
#endif                          /* __TMPM440_PSC_H */
