/**
 *******************************************************************************
 * @file    tmpm440_gpio.c
 * @brief   This file provides API functions for GPIO driver.
 * @version V2.0.2.3
 * @date    2014/05/20
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 *
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm440_gpio.h"

#if defined(__TMPM440_GPIO_H)
#define ENABLE_BIT 1U
/** @addtogroup TX04_Periph_Driver
  * @{
  */
/** @defgroup GPIO
  * @brief GPIO driver modules
  * @{
  */
/** @defgroup GPIO_Exported_Functions
  * @{
  */
/**
  * @brief  Read GPIO Data register.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PA, GPIO_PB, GPIO_PC, GPIO_PD,
  *   GPIO_PE, GPIO_PF, GPIO_PG, GPIO_PH,
  *   GPIO_PJ, GPIO_PK, GPIO_PL, GPIO_PM,
  *   GPIO_PN, GPIO_PP, GPIO_PR, GPIO_PT,
  *   GPIO_PU, GPIO_PV, GPIO_PW, GPIO_PY,
  *   GPIO_PAA, GPIO_PAB, GPIO_PAC, GPIO_PAD,
  *   GPIO_PAE, GPIO_PAF, GPIO_PAG, GPIO_PAH,
  *   GPIO_PAJ.
  * @retval Data:The value of DATA register.
  */
uint8_t GPIO_ReadData(GPIO_Port GPIO_x)
{
    uint8_t Data = 0U;

    /* Check the parameters */
    assert_param(IS_GPIO_PORT(GPIO_x));

    /* Selects GPIO */
    switch (GPIO_x) {
    case GPIO_PA:
        Data = (uint8_t) (TSB_PA->DATA);
        break;
    case GPIO_PB:
        Data = (uint8_t) (TSB_PB->DATA);
        break;
    case GPIO_PC:
        Data = (uint8_t) (TSB_PC->DATA);
        break;
    case GPIO_PD:
        Data = (uint8_t) (TSB_PD->DATA);
        break;
    case GPIO_PE:
        Data = (uint8_t) (TSB_PE->DATA);
        break;
    case GPIO_PF:
        Data = (uint8_t) (TSB_PF->DATA);
        break;
    case GPIO_PG:
        Data = (uint8_t) (TSB_PG->DATA);
        break;
    case GPIO_PH:
        Data = (uint8_t) (TSB_PH->DATA);
        break;
    case GPIO_PJ:
        Data = (uint8_t) (TSB_PJ->DATA);
        break;
    case GPIO_PK:
        Data = (uint8_t) (TSB_PK->DATA);
        break;
    case GPIO_PL:
        Data = (uint8_t) (TSB_PL->DATA);
        break;
    case GPIO_PM:
        Data = (uint8_t) (TSB_PM->DATA);
        break;
    case GPIO_PN:
        Data = (uint8_t) (TSB_PN->DATA);
        break;
    case GPIO_PP:
        Data = (uint8_t) (TSB_PP->DATA);
        break;
    case GPIO_PR:
        Data = (uint8_t) (TSB_PR->DATA);
        break;
    case GPIO_PT:
        Data = (uint8_t) (TSB_PT->DATA);
        break;
    case GPIO_PU:
        Data = (uint8_t) (TSB_PU->DATA);
        break;
    case GPIO_PV:
        Data = (uint8_t) (TSB_PV->DATA);
        break;
    case GPIO_PW:
        Data = (uint8_t) (TSB_PW->DATA);
        break;
    case GPIO_PY:
        Data = (uint8_t) (TSB_PY->DATA);
        break;
    case GPIO_PAA:
        Data = (uint8_t) (TSB_PAA->DATA);
        break;
    case GPIO_PAB:
        Data = (uint8_t) (TSB_PAB->DATA);
        break;
    case GPIO_PAC:
        Data = (uint8_t) (TSB_PAC->DATA);
        break;
    case GPIO_PAD:
        Data = (uint8_t) (TSB_PAD->DATA);
        break;
    case GPIO_PAE:
        Data = (uint8_t) (TSB_PAE->DATA);
        break;
    case GPIO_PAF:
        Data = (uint8_t) (TSB_PAF->DATA);
        break;
    case GPIO_PAG:
        Data = (uint8_t) (TSB_PAG->DATA);
        break;
    case GPIO_PAH:
        Data = (uint8_t) (TSB_PAH->DATA);
        break;
    case GPIO_PAJ:
        Data = (uint8_t) (TSB_PAJ->DATA);
        break;
    default:
        /* Do nothing */
        break;
    }
    return (Data);
}

/**
  * @brief  Read Bit of GPIO Data register.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PA, GPIO_PB, GPIO_PC, GPIO_PD,
  *   GPIO_PE, GPIO_PF, GPIO_PG, GPIO_PH,
  *   GPIO_PJ, GPIO_PK, GPIO_PL, GPIO_PM,
  *   GPIO_PN, GPIO_PP, GPIO_PR, GPIO_PT,
  *   GPIO_PU, GPIO_PV, GPIO_PW, GPIO_PY,
  *   GPIO_PAA, GPIO_PAB, GPIO_PAC, GPIO_PAD,
  *   GPIO_PAE, GPIO_PAF, GPIO_PAG, GPIO_PAH,
  *   GPIO_PAJ.
  * @param  Bit_x: Select GPIO pin.
  *   This parameter can be one of the following values:
  *   GPIO_BIT_0, GPIO_BIT_1, GPIO_BIT_2, GPIO_BIT_3,
  *   GPIO_BIT_4, GPIO_BIT_5, GPIO_BIT_6, GPIO_BIT_7.
  * @retval BitValue:The value of specified Bit.
  *   This parameter can be one of the following values:
  *   GPIO_BIT_VALUE_0, GPIO_BIT_VALUE_1
  */
uint8_t GPIO_ReadDataBit(GPIO_Port GPIO_x, uint8_t Bit_x)
{
    uint8_t BitValue = GPIO_BIT_VALUE_0;
    uint8_t Data = 0U;
    uint8_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_GPIO_PORT(GPIO_x));

    /* Selects GPIO */
    switch (GPIO_x) {
    case GPIO_PA:
        /* Check the parameters */
        assert_param(IS_GPIO_PA_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PA->DATA);
        break;
    case GPIO_PB:
        /* Check the parameters */
        assert_param(IS_GPIO_PB_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PB->DATA);
        break;
    case GPIO_PC:
        /* Check the parameters */
        assert_param(IS_GPIO_PC_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PC->DATA);
        break;
    case GPIO_PD:
        /* Check the parameters */
        assert_param(IS_GPIO_PD_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PD->DATA);
        break;
    case GPIO_PE:
        /* Check the parameters */
        assert_param(IS_GPIO_PE_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PE->DATA);
        break;
    case GPIO_PF:
        /* Check the parameters */
        assert_param(IS_GPIO_PF_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PF->DATA);
        break;
    case GPIO_PG:
        /* Check the parameters */
        assert_param(IS_GPIO_PG_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PG->DATA);
        break;
    case GPIO_PH:
        /* Check the parameters */
        assert_param(IS_GPIO_PH_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PH->DATA);
        break;
    case GPIO_PJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PJ_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PJ->DATA);
        break;
    case GPIO_PK:
        /* Check the parameters */
        assert_param(IS_GPIO_PK_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PK->DATA);
        break;
    case GPIO_PL:
        /* Check the parameters */
        assert_param(IS_GPIO_PL_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PL->DATA);
        break;
    case GPIO_PM:
        /* Check the parameters */
        assert_param(IS_GPIO_PM_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PM->DATA);
        break;
    case GPIO_PN:
        /* Check the parameters */
        assert_param(IS_GPIO_PN_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PN->DATA);
        break;
    case GPIO_PP:
        /* Check the parameters */
        assert_param(IS_GPIO_PP_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PP->DATA);
        break;
    case GPIO_PR:
        /* Check the parameters */
        assert_param(IS_GPIO_PR_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PR->DATA);
        break;
    case GPIO_PT:
        /* Check the parameters */
        assert_param(IS_GPIO_PT_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PT->DATA);
        break;
    case GPIO_PU:
        /* Check the parameters */
        assert_param(IS_GPIO_PU_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PU->DATA);
        break;
    case GPIO_PV:
        /* Check the parameters */
        assert_param(IS_GPIO_PV_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PV->DATA);
        break;
    case GPIO_PW:
        /* Check the parameters */
        assert_param(IS_GPIO_PW_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PW->DATA);
        break;
    case GPIO_PY:
        /* Check the parameters */
        assert_param(IS_GPIO_PY_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PY->DATA);
        break;
    case GPIO_PAA:
        /* Check the parameters */
        assert_param(IS_GPIO_PAA_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PAA->DATA);
        break;
    case GPIO_PAB:
        /* Check the parameters */
        assert_param(IS_GPIO_PAB_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PAB->DATA);
        break;
    case GPIO_PAC:
        /* Check the parameters */
        assert_param(IS_GPIO_PAC_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PAC->DATA);
        break;
    case GPIO_PAD:
        /* Check the parameters */
        assert_param(IS_GPIO_PAD_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PAD->DATA);
        break;
    case GPIO_PAE:
        /* Check the parameters */
        assert_param(IS_GPIO_PAE_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PAE->DATA);
        break;
    case GPIO_PAF:
        /* Check the parameters */
        assert_param(IS_GPIO_PAF_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PAF->DATA);
        break;
    case GPIO_PAG:
        /* Check the parameters */
        assert_param(IS_GPIO_PAG_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PAG->DATA);
        break;
    case GPIO_PAH:
        /* Check the parameters */
        assert_param(IS_GPIO_PAH_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PAH->DATA);
        break;
    case GPIO_PAJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PAJ_DATA_BIT(Bit_x));
        Data = (uint8_t) (TSB_PAJ->DATA);
        break;
    default:
        /* Do nothing */
        break;
    }
    tmp = (uint8_t) (Data & Bit_x);
    if (tmp) {
        BitValue = GPIO_BIT_VALUE_1;
    } else {
        BitValue = GPIO_BIT_VALUE_0;
    }
    return (BitValue);
}

/**
  * @brief  Write specified value to GPIO DATA register.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PA, GPIO_PB, GPIO_PC, GPIO_PD,
  *   GPIO_PE, GPIO_PF, GPIO_PG, GPIO_PH,
  *   GPIO_PJ, GPIO_PK, GPIO_PL, GPIO_PM,
  *   GPIO_PN, GPIO_PP, GPIO_PR, GPIO_PT,
  *   GPIO_PU, GPIO_PV, GPIO_PW, GPIO_PY,
  *   GPIO_PAD, GPIO_PAE, GPIO_PAF, GPIO_PAG,
  *   GPIO_PAH, GPIO_PAJ.
  * @param  Data: specified value will be written to GPIO DATA register.
  * @retval None
  */
void GPIO_WriteData(GPIO_Port GPIO_x, uint8_t Data)
{
    /* Check the parameters */
    assert_param(IS_GPIO_WRITE(GPIO_x));

    /* Selects GPIO */
    switch (GPIO_x) {
    case GPIO_PA:
        TSB_PA->DATA = (uint32_t) Data;
        break;
    case GPIO_PB:
        TSB_PB->DATA = (uint32_t) Data;
        break;
    case GPIO_PC:
        TSB_PC->DATA = (uint32_t) Data;
        break;
    case GPIO_PD:
        TSB_PD->DATA = (uint32_t) Data;
        break;
    case GPIO_PE:
        TSB_PE->DATA = (uint32_t) Data;
        break;
    case GPIO_PF:
        TSB_PF->DATA = (uint32_t) Data;
        break;
    case GPIO_PG:
        TSB_PG->DATA = (uint32_t) Data;
        break;
    case GPIO_PH:
        TSB_PH->DATA = (uint32_t) Data;
        break;
    case GPIO_PJ:
        TSB_PJ->DATA = (uint32_t) Data;
        break;
    case GPIO_PK:
        TSB_PK->DATA = (uint32_t) Data;
        break;
    case GPIO_PL:
        TSB_PL->DATA = (uint32_t) Data;
        break;
    case GPIO_PM:
        TSB_PM->DATA = (uint32_t) Data;
        break;
    case GPIO_PN:
        TSB_PN->DATA = (uint32_t) Data;
        break;
    case GPIO_PP:
        TSB_PP->DATA = (uint32_t) Data;
        break;
    case GPIO_PR:
        TSB_PR->DATA = (uint32_t) Data;
        break;
    case GPIO_PT:
        TSB_PT->DATA = (uint32_t) Data;
        break;
    case GPIO_PU:
        TSB_PU->DATA = (uint32_t) Data;
        break;
    case GPIO_PV:
        TSB_PV->DATA = (uint32_t) Data;
        break;
    case GPIO_PW:
        TSB_PW->DATA = (uint32_t) Data;
        break;
    case GPIO_PY:
        TSB_PY->DATA = (uint32_t) Data;
        break;
    case GPIO_PAD:
        TSB_PAD->DATA = (uint32_t) Data;
        break;
    case GPIO_PAE:
        TSB_PAE->DATA = (uint32_t) Data;
        break;
    case GPIO_PAF:
        TSB_PAF->DATA = (uint32_t) Data;
        break;
    case GPIO_PAG:
        TSB_PAG->DATA = (uint32_t) Data;
        break;
    case GPIO_PAH:
        TSB_PAH->DATA = (uint32_t) Data;
        break;
    case GPIO_PAJ:
        TSB_PAJ->DATA = (uint32_t) Data;
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Write to specified Bit of GPIO DATA register.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PA, GPIO_PB, GPIO_PC, GPIO_PD,
  *   GPIO_PE, GPIO_PF, GPIO_PG, GPIO_PH,
  *   GPIO_PJ, GPIO_PK, GPIO_PL, GPIO_PM,
  *   GPIO_PN, GPIO_PP, GPIO_PR, GPIO_PT,
  *   GPIO_PU, GPIO_PV, GPIO_PW, GPIO_PY,
  *   GPIO_PAD, GPIO_PAE, GPIO_PAF, GPIO_PAG,
  *   GPIO_PAH, GPIO_PAJ.
  * @param  Bit_x: Select GPIO pin.
  *   This parameter can be one of the following values:
  *   GPIO_BIT_0, GPIO_BIT_1, GPIO_BIT_2, GPIO_BIT_3,
  *   GPIO_BIT_4, GPIO_BIT_5, GPIO_BIT_6, GPIO_BIT_7.
  * @param BitValue:The value of specified Bit.
  *   This parameter can be one of the following values:
  *   GPIO_BIT_VALUE_0, GPIO_BIT_VALUE_1
  * @retval None
  */
void GPIO_WriteDataBit(GPIO_Port GPIO_x, uint8_t Bit_x, uint8_t BitValue)
{
    uint8_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_GPIO_WRITE(GPIO_x));
    assert_param(IS_GPIO_BIT_VALUE(BitValue));

    /* Selects GPIO */
    switch (GPIO_x) {
    case GPIO_PA:
        /* Check the parameters */
        assert_param(IS_GPIO_PA_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PA->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PA->DATA = (uint32_t) tmp;
        break;
    case GPIO_PB:
        /* Check the parameters */
        assert_param(IS_GPIO_PB_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PB->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PB->DATA = (uint32_t) tmp;
        break;
    case GPIO_PC:
        /* Check the parameters */
        assert_param(IS_GPIO_PC_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PC->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PC->DATA = (uint32_t) tmp;
        break;
    case GPIO_PD:
        /* Check the parameters */
        assert_param(IS_GPIO_PD_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PD->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PD->DATA = (uint32_t) tmp;
        break;
    case GPIO_PE:
        /* Check the parameters */
        assert_param(IS_GPIO_PE_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PE->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PE->DATA = (uint32_t) tmp;
        break;
    case GPIO_PF:
        /* Check the parameters */
        assert_param(IS_GPIO_PF_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PF->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PF->DATA = (uint32_t) tmp;
        break;
    case GPIO_PG:
        /* Check the parameters */
        assert_param(IS_GPIO_PG_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PG->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PG->DATA = (uint32_t) tmp;
        break;
    case GPIO_PH:
        /* Check the parameters */
        assert_param(IS_GPIO_PH_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PH->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PH->DATA = (uint32_t) tmp;
        break;
    case GPIO_PJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PJ_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PJ->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PJ->DATA = (uint32_t) tmp;
        break;
    case GPIO_PK:
        /* Check the parameters */
        assert_param(IS_GPIO_PK_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PK->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PK->DATA = (uint32_t) tmp;
        break;
    case GPIO_PL:
        /* Check the parameters */
        assert_param(IS_GPIO_PL_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PL->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PL->DATA = (uint32_t) tmp;
        break;
    case GPIO_PM:
        /* Check the parameters */
        assert_param(IS_GPIO_PM_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PM->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PM->DATA = (uint32_t) tmp;
        break;
    case GPIO_PN:
        /* Check the parameters */
        assert_param(IS_GPIO_PN_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PN->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PN->DATA = (uint32_t) tmp;
        break;
    case GPIO_PP:
        /* Check the parameters */
        assert_param(IS_GPIO_PP_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PP->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PP->DATA = (uint32_t) tmp;
        break;
    case GPIO_PR:
        /* Check the parameters */
        assert_param(IS_GPIO_PR_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PR->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PR->DATA = (uint32_t) tmp;
        break;
    case GPIO_PT:
        /* Check the parameters */
        assert_param(IS_GPIO_PT_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PT->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PT->DATA = (uint32_t) tmp;
        break;
    case GPIO_PU:
        /* Check the parameters */
        assert_param(IS_GPIO_PU_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PU->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PU->DATA = (uint32_t) tmp;
        break;
    case GPIO_PV:
        /* Check the parameters */
        assert_param(IS_GPIO_PV_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PV->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PV->DATA = (uint32_t) tmp;
        break;
    case GPIO_PW:
        /* Check the parameters */
        assert_param(IS_GPIO_PW_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PW->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PW->DATA = (uint32_t) tmp;
        break;
    case GPIO_PY:
        /* Check the parameters */
        assert_param(IS_GPIO_PY_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PY->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PY->DATA = (uint32_t) tmp;
        break;
    case GPIO_PAD:
        /* Check the parameters */
        assert_param(IS_GPIO_PAD_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PAD->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PAD->DATA = (uint32_t) tmp;
        break;
    case GPIO_PAE:
        /* Check the parameters */
        assert_param(IS_GPIO_PAE_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PAE->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PAE->DATA = (uint32_t) tmp;
        break;
    case GPIO_PAF:
        /* Check the parameters */
        assert_param(IS_GPIO_PAF_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PAF->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PAF->DATA = (uint32_t) tmp;
        break;
    case GPIO_PAG:
        /* Check the parameters */
        assert_param(IS_GPIO_PAG_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PAG->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PAG->DATA = (uint32_t) tmp;
        break;
    case GPIO_PAH:
        /* Check the parameters */
        assert_param(IS_GPIO_PAH_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PAH->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PAH->DATA = (uint32_t) tmp;
        break;
    case GPIO_PAJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PAJ_DATA_BIT(Bit_x));
        tmp = (uint8_t) (TSB_PAJ->DATA);
        if (BitValue) {
            tmp |= Bit_x;
        } else {
            Bit_x = (~Bit_x);
            tmp &= Bit_x;
        }
        TSB_PAJ->DATA = (uint32_t) tmp;
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Set a value to a specified bit of GPIO DATA register.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PA, GPIO_PB, GPIO_PC, GPIO_PD,
  *   GPIO_PE, GPIO_PF, GPIO_PG, GPIO_PH,
  *   GPIO_PJ, GPIO_PK, GPIO_PL, GPIO_PM,
  *   GPIO_PN, GPIO_PP, GPIO_PR, GPIO_PT,
  *   GPIO_PU, GPIO_PV, GPIO_PW, GPIO_PY,
  *   GPIO_PAD, GPIO_PAE, GPIO_PAF, GPIO_PAG,
  *   GPIO_PAH, GPIO_PAJ.
  * @param  Bit_x   : target bit (ex. GPIO_BIT_1|GPIO_BIT_2 )
  *   This parameter can be one of the following values:
  *   GPIO_BIT_0, GPIO_BIT_1, GPIO_BIT_2, GPIO_BIT_3,
  *   GPIO_BIT_4, GPIO_BIT_5, GPIO_BIT_6, GPIO_BIT_7, GPIO_BIT_ALL,
  *   or combination of the effective bits.
  * @param  Value  : byte data. 
  *   The bit of this parameter, specified by Bit_x, will be set to data register.
*/

void GPIO_WriteDataByte(GPIO_Port GPIO_x, uint8_t Bit_x, uint8_t Value)
{
    uint8_t bitPosition = 0U;
    uint8_t tmp = 0U;
    uint8_t byteValue = GPIO_ReadData(GPIO_x);
    while (Bit_x != 0U) {
        if (Bit_x & ENABLE_BIT) {
            if ((Value >> bitPosition) & ENABLE_BIT) {
                byteValue |= (ENABLE_BIT << bitPosition);
            } else {
                tmp = ~(ENABLE_BIT << bitPosition);
                byteValue &= tmp;
            }
        } else {
            /*Do nothing */
        }
        bitPosition++;
        Bit_x >>= 1U;
    }
    GPIO_WriteData(GPIO_x, byteValue);
}

/**
  * @brief  Toggle a value to a specified bit of GPIO DATA register.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PA, GPIO_PB, GPIO_PC, GPIO_PD,
  *   GPIO_PE, GPIO_PF, GPIO_PG, GPIO_PH,
  *   GPIO_PJ, GPIO_PK, GPIO_PL, GPIO_PM,
  *   GPIO_PN, GPIO_PP, GPIO_PR, GPIO_PT,
  *   GPIO_PU, GPIO_PV, GPIO_PW, GPIO_PY,
  *   GPIO_PAD, GPIO_PAE, GPIO_PAF, GPIO_PAG,
  *   GPIO_PAH, GPIO_PAJ.
  * @param  Bit_x   : target bit (ex. GPIO_BIT_1|GPIO_BIT_2 )
  *   This parameter can be one of the following values:
  *   GPIO_BIT_0, GPIO_BIT_1, GPIO_BIT_2, GPIO_BIT_3,
  *   GPIO_BIT_4, GPIO_BIT_5, GPIO_BIT_6, GPIO_BIT_7, GPIO_BIT_ALL,
  *   or combination of the effective bits.
*/
void GPIO_ToggleDataByte(GPIO_Port GPIO_x, uint8_t Bit_x)
{
    uint8_t byteValue = GPIO_ReadData(GPIO_x);
    uint8_t bitPosition = 0U;
    uint8_t tmp = 0U;
    while (Bit_x != 0U) {
        if (Bit_x & ENABLE_BIT) {
            if ((byteValue >> bitPosition) & ENABLE_BIT) {
                tmp = ~(ENABLE_BIT << bitPosition);
                byteValue &= tmp;
            } else {
                byteValue |= (ENABLE_BIT << bitPosition);
            }
        } else {
            /*Do nothing */
        }
        bitPosition++;
        Bit_x >>= 1U;
    }
    GPIO_WriteData(GPIO_x, byteValue);
}

/**
  * @brief  Initialize the specified GPIO pin.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PA, GPIO_PB, GPIO_PC, GPIO_PD,
  *   GPIO_PE, GPIO_PF, GPIO_PG, GPIO_PH,
  *   GPIO_PJ, GPIO_PK, GPIO_PL, GPIO_PM,
  *   GPIO_PN, GPIO_PP, GPIO_PR, GPIO_PT,
  *   GPIO_PU, GPIO_PV, GPIO_PW, GPIO_PY,
  *   GPIO_PAA, GPIO_PAB, GPIO_PAC, GPIO_PAD,
  *   GPIO_PAE, GPIO_PAF, GPIO_PAG, GPIO_PAH,
  *   GPIO_PAJ.
  * @param  Bit_x: Select GPIO pin.
  *   This parameter can be one of the following values:
  *   GPIO_BIT_0, GPIO_BIT_1, GPIO_BIT_2, GPIO_BIT_3,
  *   GPIO_BIT_4, GPIO_BIT_5, GPIO_BIT_6, GPIO_BIT_7, GPIO_BIT_ALL,
  *   or combination of the effective bits.
  * @param  GPIO_InitStruct: The structure containing basic GPIO configuration.
  * @retval None
  */
void GPIO_Init(GPIO_Port GPIO_x, uint8_t Bit_x, GPIO_InitTypeDef * GPIO_InitStruct)
{
    uint8_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_GPIO_PORT(GPIO_x));
    assert_param(IS_POINTER_NOT_NULL(GPIO_InitStruct));
    assert_param(IS_GPIO_IO_MODE_STATE(GPIO_InitStruct->IOMode));
    assert_param(IS_GPIO_PULLUP_STATE(GPIO_InitStruct->PullUp));
    assert_param(IS_GPIO_PULLDOWN_STATE(GPIO_InitStruct->PullDown));
    assert_param(IS_GPIO_OPEN_DRAIN_STATE(GPIO_InitStruct->OpenDrain));

    tmp = GPIO_InitStruct->IOMode;
    switch (tmp) {
    case GPIO_INPUT_MODE:
        GPIO_SetInput(GPIO_x, Bit_x);
        break;
    case GPIO_OUTPUT_MODE:
        GPIO_SetOutput(GPIO_x, Bit_x);
        break;
    default:
        /* Do nothing */
        break;
    }
    tmp = GPIO_InitStruct->PullUp;
    switch (tmp) {
    case GPIO_PULLUP_ENABLE:
        GPIO_SetPullUp(GPIO_x, Bit_x, ENABLE);
        break;
    case GPIO_PULLUP_DISABLE:
        GPIO_SetPullUp(GPIO_x, Bit_x, DISABLE);
        break;
    default:
        /* Do nothing */
        break;
    }
    tmp = GPIO_InitStruct->PullDown;
    switch (tmp) {
    case GPIO_PULLDOWN_ENABLE:
        GPIO_SetPullDown(GPIO_x, Bit_x, ENABLE);
        break;
    case GPIO_PULLDOWN_DISABLE:
        GPIO_SetPullDown(GPIO_x, Bit_x, DISABLE);
        break;
    default:
        /* Do nothing */
        break;
    }
    tmp = GPIO_InitStruct->OpenDrain;
    switch (tmp) {
    case GPIO_OPEN_DRAIN_ENABLE:
        GPIO_SetOpenDrain(GPIO_x, Bit_x, ENABLE);
        break;
    case GPIO_OPEN_DRAIN_DISABLE:
        GPIO_SetOpenDrain(GPIO_x, Bit_x, DISABLE);
        break;
    default:
        /* Do nothing */
        break;
    }
    if (GPIO_x == GPIO_PH) {
        /* Check the parameters */
        assert_param(IS_GPIO_OPEN_DRAIN_STATE(GPIO_InitStruct->OpenDrain2));
        tmp = GPIO_InitStruct->OpenDrain2;
        switch (tmp) {
        case GPIO_OPEN_DRAIN_ENABLE:
            GPIO_SetOpenDrain2(GPIO_x, Bit_x, ENABLE);
            break;
        case GPIO_OPEN_DRAIN_DISABLE:
            GPIO_SetOpenDrain2(GPIO_x, Bit_x, DISABLE);
            break;
        default:
            /* Do nothing */
            break;
        }
    } else {
        /* Do nothing */
    }
}

/**
  * @brief  Set specified GPIO Pin as output port.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PA, GPIO_PB, GPIO_PC, GPIO_PD,
  *   GPIO_PE, GPIO_PF, GPIO_PG, GPIO_PH,
  *   GPIO_PJ, GPIO_PK, GPIO_PL, GPIO_PM,
  *   GPIO_PN, GPIO_PP, GPIO_PR, GPIO_PT,
  *   GPIO_PU, GPIO_PV, GPIO_PW, GPIO_PY,
  *   GPIO_PAD, GPIO_PAE, GPIO_PAF, GPIO_PAG,
  *   GPIO_PAH, GPIO_PAJ.
  * @param  Bit_x: Select GPIO pin.
  *   This parameter can be one of the following values:
  *   GPIO_BIT_0, GPIO_BIT_1, GPIO_BIT_2, GPIO_BIT_3,
  *   GPIO_BIT_4, GPIO_BIT_5, GPIO_BIT_6, GPIO_BIT_7, GPIO_BIT_ALL,
  *   or combination of the effective bits.
  * @retval None
  */
void GPIO_SetOutput(GPIO_Port GPIO_x, uint8_t Bit_x)
{
    uint8_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_GPIO_OUTPUT(GPIO_x));

    if (Bit_x == GPIO_BIT_ALL) {
        switch (GPIO_x) {
        case GPIO_PA:
            Bit_x = GPIO_PA_OUTPUT_BIT_ALL;
            break;
        case GPIO_PB:
            Bit_x = GPIO_PB_OUTPUT_BIT_ALL;
            break;
        case GPIO_PC:
            Bit_x = GPIO_PC_OUTPUT_BIT_ALL;
            break;
        case GPIO_PD:
            Bit_x = GPIO_PD_OUTPUT_BIT_ALL;
            break;
        case GPIO_PE:
            Bit_x = GPIO_PE_OUTPUT_BIT_ALL;
            break;
        case GPIO_PF:
            Bit_x = GPIO_PF_OUTPUT_BIT_ALL;
            break;
        case GPIO_PG:
            Bit_x = GPIO_PG_OUTPUT_BIT_ALL;
            break;
        case GPIO_PH:
            Bit_x = GPIO_PH_OUTPUT_BIT_ALL;
            break;
        case GPIO_PJ:
            Bit_x = GPIO_PJ_OUTPUT_BIT_ALL;
            break;
        case GPIO_PK:
            Bit_x = GPIO_PK_OUTPUT_BIT_ALL;
            break;
        case GPIO_PL:
            Bit_x = GPIO_PL_OUTPUT_BIT_ALL;
            break;
        case GPIO_PM:
            Bit_x = GPIO_PM_OUTPUT_BIT_ALL;
            break;
        case GPIO_PN:
            Bit_x = GPIO_PN_OUTPUT_BIT_ALL;
            break;
        case GPIO_PP:
            Bit_x = GPIO_PP_OUTPUT_BIT_ALL;
            break;
        case GPIO_PR:
            Bit_x = GPIO_PR_OUTPUT_BIT_ALL;
            break;
        case GPIO_PT:
            Bit_x = GPIO_PT_OUTPUT_BIT_ALL;
            break;
        case GPIO_PU:
            Bit_x = GPIO_PU_OUTPUT_BIT_ALL;
            break;
        case GPIO_PV:
            Bit_x = GPIO_PV_OUTPUT_BIT_ALL;
            break;
        case GPIO_PW:
            Bit_x = GPIO_PW_OUTPUT_BIT_ALL;
            break;
        case GPIO_PY:
            Bit_x = GPIO_PY_OUTPUT_BIT_ALL;
            break;
        case GPIO_PAD:
            Bit_x = GPIO_PAD_OUTPUT_BIT_ALL;
            break;
        case GPIO_PAE:
            Bit_x = GPIO_PAE_OUTPUT_BIT_ALL;
            break;
        case GPIO_PAF:
            Bit_x = GPIO_PAF_OUTPUT_BIT_ALL;
            break;
        case GPIO_PAG:
            Bit_x = GPIO_PAG_OUTPUT_BIT_ALL;
            break;
        case GPIO_PAH:
            Bit_x = GPIO_PAH_OUTPUT_BIT_ALL;
            break;
        case GPIO_PAJ:
            Bit_x = GPIO_PAJ_OUTPUT_BIT_ALL;
            break;
        default:
            /* Do nothing */
            break;
        }
    } else {
        /* Do nothing */
    }
    /* Selects GPIO */
    switch (GPIO_x) {
    case GPIO_PA:
        /* Check the parameters */
        assert_param(IS_GPIO_PA_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PA->FR1 &= tmp;
        TSB_PA->FR3 &= tmp;
        TSB_PA->IE &= tmp;
        TSB_PA->CR |= Bit_x;
        break;
    case GPIO_PB:
        /* Check the parameters */
        assert_param(IS_GPIO_PB_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PB->FR1 &= tmp;
        TSB_PB->FR3 &= tmp;
        TSB_PB->IE &= tmp;
        TSB_PB->CR |= Bit_x;
        break;
    case GPIO_PC:
        /* Check the parameters */
        assert_param(IS_GPIO_PC_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PC->FR1 &= tmp;
        TSB_PC->FR2 &= tmp;
        TSB_PC->FR3 &= tmp;
        TSB_PC->IE &= tmp;
        TSB_PC->CR |= Bit_x;
        break;
    case GPIO_PD:
        /* Check the parameters */
        assert_param(IS_GPIO_PD_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PD->FR1 &= tmp;
        TSB_PD->FR2 &= tmp;
        TSB_PD->IE &= tmp;
        TSB_PD->CR |= Bit_x;
        break;
    case GPIO_PE:
        /* Check the parameters */
        assert_param(IS_GPIO_PE_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PE->FR1 &= tmp;
        TSB_PE->FR2 &= tmp;
        TSB_PE->FR3 &= tmp;
        TSB_PE->IE &= tmp;
        TSB_PE->CR |= Bit_x;
        break;
    case GPIO_PF:
        /* Check the parameters */
        assert_param(IS_GPIO_PF_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PF->FR1 &= tmp;
        TSB_PF->FR2 &= tmp;
        TSB_PF->FR3 &= tmp;
        TSB_PF->IE &= tmp;
        TSB_PF->CR |= Bit_x;
        break;
    case GPIO_PG:
        /* Check the parameters */
        assert_param(IS_GPIO_PG_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PG->FR1 &= tmp;
        TSB_PG->IE &= tmp;
        TSB_PG->CR |= Bit_x;
        break;
    case GPIO_PH:
        /* Check the parameters */
        assert_param(IS_GPIO_PH_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PH->FR1 &= tmp;
        TSB_PH->FR2 &= tmp;
        TSB_PH->IE &= tmp;
        TSB_PH->CR |= Bit_x;
        break;
    case GPIO_PJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PJ_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PJ->FR1 &= tmp;
        TSB_PJ->IE &= tmp;
        TSB_PJ->CR |= Bit_x;
        break;
    case GPIO_PK:
        /* Check the parameters */
        assert_param(IS_GPIO_PK_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PK->FR1 &= tmp;
        TSB_PK->FR2 &= tmp;
        TSB_PK->FR3 &= tmp;
        TSB_PK->IE &= tmp;
        TSB_PK->CR |= Bit_x;
        break;
    case GPIO_PL:
        /* Check the parameters */
        assert_param(IS_GPIO_PL_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PL->FR1 &= tmp;
        TSB_PL->IE &= tmp;
        TSB_PL->CR |= Bit_x;
        break;
    case GPIO_PM:
        /* Check the parameters */
        assert_param(IS_GPIO_PM_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PM->FR1 &= tmp;
        TSB_PM->FR2 &= tmp;
        TSB_PM->FR3 &= tmp;
        TSB_PM->IE &= tmp;
        TSB_PM->CR |= Bit_x;
        break;
    case GPIO_PN:
        /* Check the parameters */
        assert_param(IS_GPIO_PN_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PN->FR1 &= tmp;
        TSB_PN->FR3 &= tmp;
        TSB_PN->IE &= tmp;
        TSB_PN->CR |= Bit_x;
        break;
    case GPIO_PP:
        /* Check the parameters */
        assert_param(IS_GPIO_PP_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PP->FR1 &= tmp;
        TSB_PP->FR2 &= tmp;
        TSB_PP->FR3 &= tmp;
        TSB_PP->IE &= tmp;
        TSB_PP->CR |= Bit_x;
        break;
    case GPIO_PR:
        /* Check the parameters */
        assert_param(IS_GPIO_PR_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PR->FR1 &= tmp;
        TSB_PR->FR2 &= tmp;
        TSB_PR->FR3 &= tmp;
        TSB_PR->IE &= tmp;
        TSB_PR->CR |= Bit_x;
        break;
    case GPIO_PT:
        /* Check the parameters */
        assert_param(IS_GPIO_PT_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PT->FR1 &= tmp;
        TSB_PT->IE &= tmp;
        TSB_PT->CR |= Bit_x;
        break;
    case GPIO_PU:
        /* Check the parameters */
        assert_param(IS_GPIO_PU_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PU->FR1 &= tmp;
        TSB_PU->IE &= tmp;
        TSB_PU->CR |= Bit_x;
        break;
    case GPIO_PV:
        /* Check the parameters */
        assert_param(IS_GPIO_PV_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PV->FR1 &= tmp;
        TSB_PV->IE &= tmp;
        TSB_PV->CR |= Bit_x;
        break;
    case GPIO_PW:
        /* Check the parameters */
        assert_param(IS_GPIO_PW_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PW->FR1 &= tmp;
        TSB_PW->IE &= tmp;
        TSB_PW->CR |= Bit_x;
        break;
    case GPIO_PY:
        /* Check the parameters */
        assert_param(IS_GPIO_PY_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PY->FR1 &= tmp;
        TSB_PY->IE &= tmp;
        TSB_PY->CR |= Bit_x;
        break;
    case GPIO_PAD:
        /* Check the parameters */
        assert_param(IS_GPIO_PAD_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PAD->FR1 &= tmp;
        TSB_PAD->IE &= tmp;
        TSB_PAD->CR |= Bit_x;
        break;
    case GPIO_PAE:
        /* Check the parameters */
        assert_param(IS_GPIO_PAE_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PAE->FR1 &= tmp;
        TSB_PAE->IE &= tmp;
        TSB_PAE->CR |= Bit_x;
        break;
    case GPIO_PAF:
        /* Check the parameters */
        assert_param(IS_GPIO_PAF_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PAF->FR1 &= tmp;
        TSB_PAF->IE &= tmp;
        TSB_PAF->CR |= Bit_x;
        break;
    case GPIO_PAG:
        /* Check the parameters */
        assert_param(IS_GPIO_PAG_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PAG->FR1 &= tmp;
        TSB_PAG->FR2 &= tmp;
        TSB_PAG->IE &= tmp;
        TSB_PAG->CR |= Bit_x;
        break;
    case GPIO_PAH:
        /* Check the parameters */
        assert_param(IS_GPIO_PAH_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PAH->IE &= tmp;
        TSB_PAH->CR |= Bit_x;
        break;
    case GPIO_PAJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PAJ_OUTPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PAJ->FR1 &= tmp;
        TSB_PAJ->IE &= tmp;
        TSB_PAJ->CR |= Bit_x;
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Set specified GPIO Pin as input port.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PA, GPIO_PB, GPIO_PC, GPIO_PD,
  *   GPIO_PE, GPIO_PF, GPIO_PG, GPIO_PH,
  *   GPIO_PJ, GPIO_PK, GPIO_PL, GPIO_PM,
  *   GPIO_PN, GPIO_PP, GPIO_PR, GPIO_PT,
  *   GPIO_PU, GPIO_PV, GPIO_PW, GPIO_PY,
  *   GPIO_PAA, GPIO_PAB, GPIO_PAC, GPIO_PAD,
  *   GPIO_PAE, GPIO_PAF, GPIO_PAG, GPIO_PAH,
  *   GPIO_PAJ.
  * @param  Bit_x: Select GPIO pin.
  *   This parameter can be one of the following values:
  *   GPIO_BIT_0, GPIO_BIT_1, GPIO_BIT_2, GPIO_BIT_3,
  *   GPIO_BIT_4, GPIO_BIT_5, GPIO_BIT_6, GPIO_BIT_7, GPIO_BIT_ALL,
  *   or combination of the effective bits.
  * @retval None
  */
void GPIO_SetInput(GPIO_Port GPIO_x, uint8_t Bit_x)
{
    uint8_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_GPIO_INPUT(GPIO_x));

    if (Bit_x == GPIO_BIT_ALL) {
        switch (GPIO_x) {
        case GPIO_PA:
            Bit_x = GPIO_PA_INPUT_BIT_ALL;
            break;
        case GPIO_PB:
            Bit_x = GPIO_PB_INPUT_BIT_ALL;
            break;
        case GPIO_PC:
            Bit_x = GPIO_PC_INPUT_BIT_ALL;
            break;
        case GPIO_PD:
            Bit_x = GPIO_PD_INPUT_BIT_ALL;
            break;
        case GPIO_PE:
            Bit_x = GPIO_PE_INPUT_BIT_ALL;
            break;
        case GPIO_PF:
            Bit_x = GPIO_PF_INPUT_BIT_ALL;
            break;
        case GPIO_PG:
            Bit_x = GPIO_PG_INPUT_BIT_ALL;
            break;
        case GPIO_PH:
            Bit_x = GPIO_PH_INPUT_BIT_ALL;
            break;
        case GPIO_PJ:
            Bit_x = GPIO_PJ_INPUT_BIT_ALL;
            break;
        case GPIO_PK:
            Bit_x = GPIO_PK_INPUT_BIT_ALL;
            break;
        case GPIO_PL:
            Bit_x = GPIO_PL_INPUT_BIT_ALL;
            break;
        case GPIO_PM:
            Bit_x = GPIO_PM_INPUT_BIT_ALL;
            break;
        case GPIO_PN:
            Bit_x = GPIO_PN_INPUT_BIT_ALL;
            break;
        case GPIO_PP:
            Bit_x = GPIO_PP_INPUT_BIT_ALL;
            break;
        case GPIO_PR:
            Bit_x = GPIO_PR_INPUT_BIT_ALL;
            break;
        case GPIO_PT:
            Bit_x = GPIO_PT_INPUT_BIT_ALL;
            break;
        case GPIO_PU:
            Bit_x = GPIO_PU_INPUT_BIT_ALL;
            break;
        case GPIO_PV:
            Bit_x = GPIO_PV_INPUT_BIT_ALL;
            break;
        case GPIO_PW:
            Bit_x = GPIO_PW_INPUT_BIT_ALL;
            break;
        case GPIO_PY:
            Bit_x = GPIO_PY_INPUT_BIT_ALL;
            break;
        case GPIO_PAA:
            Bit_x = GPIO_PAA_INPUT_BIT_ALL;
            break;
        case GPIO_PAB:
            Bit_x = GPIO_PAB_INPUT_BIT_ALL;
            break;
        case GPIO_PAC:
            Bit_x = GPIO_PAC_INPUT_BIT_ALL;
            break;
        case GPIO_PAD:
            Bit_x = GPIO_PAD_INPUT_BIT_ALL;
            break;
        case GPIO_PAE:
            Bit_x = GPIO_PAE_INPUT_BIT_ALL;
            break;
        case GPIO_PAF:
            Bit_x = GPIO_PAF_INPUT_BIT_ALL;
            break;
        case GPIO_PAG:
            Bit_x = GPIO_PAG_INPUT_BIT_ALL;
            break;
        case GPIO_PAH:
            Bit_x = GPIO_PAH_INPUT_BIT_ALL;
            break;
        case GPIO_PAJ:
            Bit_x = GPIO_PAJ_INPUT_BIT_ALL;
            break;
        default:
            /* Do nothing */
            break;
        }
    } else {
        /* Do nothing */
    }
    /* Selects GPIO */
    switch (GPIO_x) {
    case GPIO_PA:
        /* Check the parameters */
        assert_param(IS_GPIO_PA_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PA->FR1 &= tmp;
        TSB_PA->FR3 &= tmp;
        TSB_PA->CR &= tmp;
        TSB_PA->IE |= Bit_x;
        break;
    case GPIO_PB:
        /* Check the parameters */
        assert_param(IS_GPIO_PB_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PB->FR1 &= tmp;
        TSB_PB->FR3 &= tmp;
        TSB_PB->CR &= tmp;
        TSB_PB->IE |= Bit_x;
        break;
    case GPIO_PC:
        /* Check the parameters */
        assert_param(IS_GPIO_PC_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PC->FR1 &= tmp;
        TSB_PC->FR2 &= tmp;
        TSB_PC->FR3 &= tmp;
        TSB_PC->CR &= tmp;
        TSB_PC->IE |= Bit_x;
        break;
    case GPIO_PD:
        /* Check the parameters */
        assert_param(IS_GPIO_PD_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PD->FR1 &= tmp;
        TSB_PD->FR2 &= tmp;
        TSB_PD->CR &= tmp;
        TSB_PD->IE |= Bit_x;
        break;
    case GPIO_PE:
        /* Check the parameters */
        assert_param(IS_GPIO_PE_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PE->FR1 &= tmp;
        TSB_PE->FR2 &= tmp;
        TSB_PE->FR3 &= tmp;
        TSB_PE->CR &= tmp;
        TSB_PE->IE |= Bit_x;
        break;
    case GPIO_PF:
        /* Check the parameters */
        assert_param(IS_GPIO_PF_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PF->FR1 &= tmp;
        TSB_PF->FR2 &= tmp;
        TSB_PF->FR3 &= tmp;
        TSB_PF->CR &= tmp;
        TSB_PF->IE |= Bit_x;
        break;
    case GPIO_PG:
        /* Check the parameters */
        assert_param(IS_GPIO_PG_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PG->FR1 &= tmp;
        TSB_PG->CR &= tmp;
        TSB_PG->IE |= Bit_x;
        break;
    case GPIO_PH:
        /* Check the parameters */
        assert_param(IS_GPIO_PH_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PH->FR1 &= tmp;
        TSB_PH->FR2 &= tmp;
        TSB_PH->CR &= tmp;
        TSB_PH->IE |= Bit_x;
        break;
    case GPIO_PJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PJ_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PJ->FR1 &= tmp;
        TSB_PJ->CR &= tmp;
        TSB_PJ->IE |= Bit_x;
        break;
    case GPIO_PK:
        /* Check the parameters */
        assert_param(IS_GPIO_PK_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PK->FR1 &= tmp;
        TSB_PK->FR2 &= tmp;
        TSB_PK->FR3 &= tmp;
        TSB_PK->CR &= tmp;
        TSB_PK->IE |= Bit_x;
        break;
    case GPIO_PL:
        /* Check the parameters */
        assert_param(IS_GPIO_PL_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PL->FR1 &= tmp;
        TSB_PL->CR &= tmp;
        TSB_PL->IE |= Bit_x;
        break;
    case GPIO_PM:
        /* Check the parameters */
        assert_param(IS_GPIO_PM_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PM->FR1 &= tmp;
        TSB_PM->FR2 &= tmp;
        TSB_PM->FR3 &= tmp;
        TSB_PM->CR &= tmp;
        TSB_PM->IE |= Bit_x;
        break;
    case GPIO_PN:
        /* Check the parameters */
        assert_param(IS_GPIO_PN_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PN->FR1 &= tmp;
        TSB_PN->FR3 &= tmp;
        TSB_PN->CR &= tmp;
        TSB_PN->IE |= Bit_x;
        break;
    case GPIO_PP:
        /* Check the parameters */
        assert_param(IS_GPIO_PP_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PP->FR1 &= tmp;
        TSB_PP->FR2 &= tmp;
        TSB_PP->FR3 &= tmp;
        TSB_PP->CR &= tmp;
        TSB_PP->IE |= Bit_x;
        break;
    case GPIO_PR:
        /* Check the parameters */
        assert_param(IS_GPIO_PR_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PR->FR1 &= tmp;
        TSB_PR->FR2 &= tmp;
        TSB_PR->FR3 &= tmp;
        TSB_PR->CR &= tmp;
        TSB_PR->IE |= Bit_x;
        break;
    case GPIO_PT:
        /* Check the parameters */
        assert_param(IS_GPIO_PT_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PT->FR1 &= tmp;
        TSB_PT->CR &= tmp;
        TSB_PT->IE |= Bit_x;
        break;
    case GPIO_PU:
        /* Check the parameters */
        assert_param(IS_GPIO_PU_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PU->FR1 &= tmp;
        TSB_PU->CR &= tmp;
        TSB_PU->IE |= Bit_x;
        break;
    case GPIO_PV:
        /* Check the parameters */
        assert_param(IS_GPIO_PV_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PV->FR1 &= tmp;
        TSB_PV->CR &= tmp;
        TSB_PV->IE |= Bit_x;
        break;
    case GPIO_PW:
        /* Check the parameters */
        assert_param(IS_GPIO_PW_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PW->FR1 &= tmp;
        TSB_PW->CR &= tmp;
        TSB_PW->IE |= Bit_x;
        break;
    case GPIO_PY:
        /* Check the parameters */
        assert_param(IS_GPIO_PY_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PY->FR1 &= tmp;
        TSB_PY->CR &= tmp;
        TSB_PY->IE |= Bit_x;
        break;
    case GPIO_PAA:
        /* Check the parameters */
        assert_param(IS_GPIO_PAA_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PAA->IE |= Bit_x;
        break;
    case GPIO_PAB:
        /* Check the parameters */
        assert_param(IS_GPIO_PAB_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PAB->IE |= Bit_x;
        break;
    case GPIO_PAC:
        /* Check the parameters */
        assert_param(IS_GPIO_PAC_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PAC->IE |= Bit_x;
        break;
    case GPIO_PAD:
        /* Check the parameters */
        assert_param(IS_GPIO_PAD_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PAD->FR1 &= tmp;
        TSB_PAD->CR &= tmp;
        TSB_PAD->IE |= Bit_x;
        break;
    case GPIO_PAE:
        /* Check the parameters */
        assert_param(IS_GPIO_PAE_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PAE->FR1 &= tmp;
        TSB_PAE->CR &= tmp;
        TSB_PAE->IE |= Bit_x;
        break;
    case GPIO_PAF:
        /* Check the parameters */
        assert_param(IS_GPIO_PAF_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PAF->FR1 &= tmp;
        TSB_PAF->CR &= tmp;
        TSB_PAF->IE |= Bit_x;
        break;
    case GPIO_PAG:
        /* Check the parameters */
        assert_param(IS_GPIO_PAG_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PAG->FR1 &= tmp;
        TSB_PAG->FR2 &= tmp;
        TSB_PAG->CR &= tmp;
        TSB_PAG->IE |= Bit_x;
        break;
    case GPIO_PAH:
        /* Check the parameters */
        assert_param(IS_GPIO_PAH_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PAH->CR &= tmp;
        TSB_PAH->IE |= Bit_x;
        break;
    case GPIO_PAJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PAJ_INPUT_BIT(Bit_x));
        tmp = (~Bit_x);
        TSB_PAJ->FR1 &= tmp;
        TSB_PAJ->CR &= tmp;
        TSB_PAJ->IE |= Bit_x;
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Set or clear the bit setting in output control register.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PA, GPIO_PB, GPIO_PC, GPIO_PD,
  *   GPIO_PE, GPIO_PF, GPIO_PG, GPIO_PH,
  *   GPIO_PJ, GPIO_PK, GPIO_PL, GPIO_PM,
  *   GPIO_PN, GPIO_PP, GPIO_PR, GPIO_PT,
  *   GPIO_PU, GPIO_PV, GPIO_PW, GPIO_PY,
  *   GPIO_PAD, GPIO_PAE, GPIO_PAF, GPIO_PAG,
  *   GPIO_PAH, GPIO_PAJ.
  * @param  Bit_x: Select GPIO pin.
  *   This parameter can be one of the following values:
  *   GPIO_BIT_0, GPIO_BIT_1, GPIO_BIT_2, GPIO_BIT_3,
  *   GPIO_BIT_4, GPIO_BIT_5, GPIO_BIT_6, GPIO_BIT_7, GPIO_BIT_ALL,
  *   or combination of the effective bits.
  * @param NewState: The output state of the pin.
  *   This parameter can be one of the following values:
  *   ENABLE , DISABLE.
  * @retval None
  */
void GPIO_SetOutputEnableReg(GPIO_Port GPIO_x, uint8_t Bit_x, FunctionalState NewState)
{
    uint8_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_GPIO_OUTPUT(GPIO_x));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (Bit_x == GPIO_BIT_ALL) {
        switch (GPIO_x) {
        case GPIO_PA:
            Bit_x = GPIO_PA_OUTPUT_BIT_ALL;
            break;
        case GPIO_PB:
            Bit_x = GPIO_PB_OUTPUT_BIT_ALL;
            break;
        case GPIO_PC:
            Bit_x = GPIO_PC_OUTPUT_BIT_ALL;
            break;
        case GPIO_PD:
            Bit_x = GPIO_PD_OUTPUT_BIT_ALL;
            break;
        case GPIO_PE:
            Bit_x = GPIO_PE_OUTPUT_BIT_ALL;
            break;
        case GPIO_PF:
            Bit_x = GPIO_PF_OUTPUT_BIT_ALL;
            break;
        case GPIO_PG:
            Bit_x = GPIO_PG_OUTPUT_BIT_ALL;
            break;
        case GPIO_PH:
            Bit_x = GPIO_PH_OUTPUT_BIT_ALL;
            break;
        case GPIO_PJ:
            Bit_x = GPIO_PJ_OUTPUT_BIT_ALL;
            break;
        case GPIO_PK:
            Bit_x = GPIO_PK_OUTPUT_BIT_ALL;
            break;
        case GPIO_PL:
            Bit_x = GPIO_PL_OUTPUT_BIT_ALL;
            break;
        case GPIO_PM:
            Bit_x = GPIO_PM_OUTPUT_BIT_ALL;
            break;
        case GPIO_PN:
            Bit_x = GPIO_PN_OUTPUT_BIT_ALL;
            break;
        case GPIO_PP:
            Bit_x = GPIO_PP_OUTPUT_BIT_ALL;
            break;
        case GPIO_PR:
            Bit_x = GPIO_PR_OUTPUT_BIT_ALL;
            break;
        case GPIO_PT:
            Bit_x = GPIO_PT_OUTPUT_BIT_ALL;
            break;
        case GPIO_PU:
            Bit_x = GPIO_PU_OUTPUT_BIT_ALL;
            break;
        case GPIO_PV:
            Bit_x = GPIO_PV_OUTPUT_BIT_ALL;
            break;
        case GPIO_PW:
            Bit_x = GPIO_PW_OUTPUT_BIT_ALL;
            break;
        case GPIO_PY:
            Bit_x = GPIO_PY_OUTPUT_BIT_ALL;
            break;
        case GPIO_PAD:
            Bit_x = GPIO_PAD_OUTPUT_BIT_ALL;
            break;
        case GPIO_PAE:
            Bit_x = GPIO_PAE_OUTPUT_BIT_ALL;
            break;
        case GPIO_PAF:
            Bit_x = GPIO_PAF_OUTPUT_BIT_ALL;
            break;
        case GPIO_PAG:
            Bit_x = GPIO_PAG_OUTPUT_BIT_ALL;
            break;
        case GPIO_PAH:
            Bit_x = GPIO_PAH_OUTPUT_BIT_ALL;
            break;
        case GPIO_PAJ:
            Bit_x = GPIO_PAJ_OUTPUT_BIT_ALL;
            break;
        default:
            /* Do nothing */
            break;
        }
    } else {
        /* Do nothing */
    }
    /* Selects GPIO */
    switch (GPIO_x) {
    case GPIO_PA:
        /* Check the parameters */
        assert_param(IS_GPIO_PA_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PA->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PA->CR &= tmp;
        }
        break;
    case GPIO_PB:
        /* Check the parameters */
        assert_param(IS_GPIO_PB_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PB->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PB->CR &= tmp;
        }
        break;
    case GPIO_PC:
        /* Check the parameters */
        assert_param(IS_GPIO_PC_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PC->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PC->CR &= tmp;
        }
        break;
    case GPIO_PD:
        /* Check the parameters */
        assert_param(IS_GPIO_PD_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PD->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PD->CR &= tmp;
        }
        break;
    case GPIO_PE:
        /* Check the parameters */
        assert_param(IS_GPIO_PE_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PE->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PE->CR &= tmp;
        }
        break;
    case GPIO_PF:
        /* Check the parameters */
        assert_param(IS_GPIO_PF_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PF->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PF->CR &= tmp;
        }
        break;
    case GPIO_PG:
        /* Check the parameters */
        assert_param(IS_GPIO_PG_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PG->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PG->CR &= tmp;
        }
        break;
    case GPIO_PH:
        /* Check the parameters */
        assert_param(IS_GPIO_PH_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PH->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PH->CR &= tmp;
        }
        break;
    case GPIO_PJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PJ_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PJ->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PJ->CR &= tmp;
        }
        break;
    case GPIO_PK:
        /* Check the parameters */
        assert_param(IS_GPIO_PK_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PK->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PK->CR &= tmp;
        }
        break;
    case GPIO_PL:
        /* Check the parameters */
        assert_param(IS_GPIO_PL_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PL->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PL->CR &= tmp;
        }
        break;
    case GPIO_PM:
        /* Check the parameters */
        assert_param(IS_GPIO_PM_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PM->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PM->CR &= tmp;
        }
        break;
    case GPIO_PN:
        /* Check the parameters */
        assert_param(IS_GPIO_PN_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PN->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PN->CR &= tmp;
        }
        break;
    case GPIO_PP:
        /* Check the parameters */
        assert_param(IS_GPIO_PP_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PP->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PP->CR &= tmp;
        }
        break;
    case GPIO_PR:
        /* Check the parameters */
        assert_param(IS_GPIO_PR_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PR->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PR->CR &= tmp;
        }
        break;
    case GPIO_PT:
        /* Check the parameters */
        assert_param(IS_GPIO_PT_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PT->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PT->CR &= tmp;
        }
        break;
    case GPIO_PU:
        /* Check the parameters */
        assert_param(IS_GPIO_PU_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PU->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PU->CR &= tmp;
        }
        break;
    case GPIO_PV:
        /* Check the parameters */
        assert_param(IS_GPIO_PV_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PV->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PV->CR &= tmp;
        }
        break;
    case GPIO_PW:
        /* Check the parameters */
        assert_param(IS_GPIO_PW_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PW->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PW->CR &= tmp;
        }
        break;
    case GPIO_PY:
        /* Check the parameters */
        assert_param(IS_GPIO_PY_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PY->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PY->CR &= tmp;
        }
        break;
    case GPIO_PAD:
        /* Check the parameters */
        assert_param(IS_GPIO_PAD_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAD->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAD->CR &= tmp;
        }
        break;
    case GPIO_PAE:
        /* Check the parameters */
        assert_param(IS_GPIO_PAE_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAE->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAE->CR &= tmp;
        }
        break;
    case GPIO_PAF:
        /* Check the parameters */
        assert_param(IS_GPIO_PAF_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAF->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAF->CR &= tmp;
        }
        break;
    case GPIO_PAG:
        /* Check the parameters */
        assert_param(IS_GPIO_PAG_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAG->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAG->CR &= tmp;
        }
        break;
    case GPIO_PAH:
        /* Check the parameters */
        assert_param(IS_GPIO_PAH_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAH->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAH->CR &= tmp;
        }
        break;
    case GPIO_PAJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PAJ_OUTPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAJ->CR |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAJ->CR &= tmp;
        }
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Set or clear the bit setting in input control register.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PA, GPIO_PB, GPIO_PC, GPIO_PD,
  *   GPIO_PE, GPIO_PF, GPIO_PG, GPIO_PH,
  *   GPIO_PJ, GPIO_PK, GPIO_PL, GPIO_PM,
  *   GPIO_PN, GPIO_PP, GPIO_PR, GPIO_PT,
  *   GPIO_PU, GPIO_PV, GPIO_PW, GPIO_PY,
  *   GPIO_PAA, GPIO_PAB, GPIO_PAC, GPIO_PAD,
  *   GPIO_PAE, GPIO_PAF, GPIO_PAG, GPIO_PAH,
  *   GPIO_PAJ.
  * @param  Bit_x: Select GPIO pin.
  *   This parameter can be one of the following values:
  *   GPIO_BIT_0, GPIO_BIT_1, GPIO_BIT_2, GPIO_BIT_3,
  *   GPIO_BIT_4, GPIO_BIT_5, GPIO_BIT_6, GPIO_BIT_7, GPIO_BIT_ALL,
  *   or combination of the effective bits.
  * @param NewState: The input state of the pin.
  *   This parameter can be one of the following values:
  *   ENABLE , DISABLE.
  * @retval None
  */
void GPIO_SetInputEnableReg(GPIO_Port GPIO_x, uint8_t Bit_x, FunctionalState NewState)
{
    uint8_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_GPIO_INPUT(GPIO_x));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (Bit_x == GPIO_BIT_ALL) {
        switch (GPIO_x) {
        case GPIO_PA:
            Bit_x = GPIO_PA_INPUT_BIT_ALL;
            break;
        case GPIO_PB:
            Bit_x = GPIO_PB_INPUT_BIT_ALL;
            break;
        case GPIO_PC:
            Bit_x = GPIO_PC_INPUT_BIT_ALL;
            break;
        case GPIO_PD:
            Bit_x = GPIO_PD_INPUT_BIT_ALL;
            break;
        case GPIO_PE:
            Bit_x = GPIO_PE_INPUT_BIT_ALL;
            break;
        case GPIO_PF:
            Bit_x = GPIO_PF_INPUT_BIT_ALL;
            break;
        case GPIO_PG:
            Bit_x = GPIO_PG_INPUT_BIT_ALL;
            break;
        case GPIO_PH:
            Bit_x = GPIO_PH_INPUT_BIT_ALL;
            break;
        case GPIO_PJ:
            Bit_x = GPIO_PJ_INPUT_BIT_ALL;
            break;
        case GPIO_PK:
            Bit_x = GPIO_PK_INPUT_BIT_ALL;
            break;
        case GPIO_PL:
            Bit_x = GPIO_PL_INPUT_BIT_ALL;
            break;
        case GPIO_PM:
            Bit_x = GPIO_PM_INPUT_BIT_ALL;
            break;
        case GPIO_PN:
            Bit_x = GPIO_PN_INPUT_BIT_ALL;
            break;
        case GPIO_PP:
            Bit_x = GPIO_PP_INPUT_BIT_ALL;
            break;
        case GPIO_PR:
            Bit_x = GPIO_PR_INPUT_BIT_ALL;
            break;
        case GPIO_PT:
            Bit_x = GPIO_PT_INPUT_BIT_ALL;
            break;
        case GPIO_PU:
            Bit_x = GPIO_PU_INPUT_BIT_ALL;
            break;
        case GPIO_PV:
            Bit_x = GPIO_PV_INPUT_BIT_ALL;
            break;
        case GPIO_PW:
            Bit_x = GPIO_PW_INPUT_BIT_ALL;
            break;
        case GPIO_PY:
            Bit_x = GPIO_PY_INPUT_BIT_ALL;
            break;
        case GPIO_PAA:
            Bit_x = GPIO_PAA_INPUT_BIT_ALL;
            break;
        case GPIO_PAB:
            Bit_x = GPIO_PAB_INPUT_BIT_ALL;
            break;
        case GPIO_PAC:
            Bit_x = GPIO_PAC_INPUT_BIT_ALL;
            break;
        case GPIO_PAD:
            Bit_x = GPIO_PAD_INPUT_BIT_ALL;
            break;
        case GPIO_PAE:
            Bit_x = GPIO_PAE_INPUT_BIT_ALL;
            break;
        case GPIO_PAF:
            Bit_x = GPIO_PAF_INPUT_BIT_ALL;
            break;
        case GPIO_PAG:
            Bit_x = GPIO_PAG_INPUT_BIT_ALL;
            break;
        case GPIO_PAH:
            Bit_x = GPIO_PAH_INPUT_BIT_ALL;
            break;
        case GPIO_PAJ:
            Bit_x = GPIO_PAJ_INPUT_BIT_ALL;
            break;
        default:
            /* Do nothing */
            break;
        }
    } else {
        /* Do nothing */
    }
    /* Selects GPIO */
    switch (GPIO_x) {
    case GPIO_PA:
        /* Check the parameters */
        assert_param(IS_GPIO_PA_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PA->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PA->IE &= tmp;
        }
        break;
    case GPIO_PB:
        /* Check the parameters */
        assert_param(IS_GPIO_PB_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PB->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PB->IE &= tmp;
        }
        break;
    case GPIO_PC:
        /* Check the parameters */
        assert_param(IS_GPIO_PC_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PC->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PC->IE &= tmp;
        }
        break;
    case GPIO_PD:
        /* Check the parameters */
        assert_param(IS_GPIO_PD_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PD->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PD->IE &= tmp;
        }
        break;
    case GPIO_PE:
        /* Check the parameters */
        assert_param(IS_GPIO_PE_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PE->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PE->IE &= tmp;
        }
        break;
    case GPIO_PF:
        /* Check the parameters */
        assert_param(IS_GPIO_PF_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PF->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PF->IE &= tmp;
        }
        break;
    case GPIO_PG:
        /* Check the parameters */
        assert_param(IS_GPIO_PG_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PG->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PG->IE &= tmp;
        }
        break;
    case GPIO_PH:
        /* Check the parameters */
        assert_param(IS_GPIO_PH_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PH->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PH->IE &= tmp;
        }
        break;
    case GPIO_PJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PJ_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PJ->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PJ->IE &= tmp;
        }
        break;
    case GPIO_PK:
        /* Check the parameters */
        assert_param(IS_GPIO_PK_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PK->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PK->IE &= tmp;
        }
        break;
    case GPIO_PL:
        /* Check the parameters */
        assert_param(IS_GPIO_PL_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PL->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PL->IE &= tmp;
        }
        break;
    case GPIO_PM:
        /* Check the parameters */
        assert_param(IS_GPIO_PM_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PM->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PM->IE &= tmp;
        }
        break;
    case GPIO_PN:
        /* Check the parameters */
        assert_param(IS_GPIO_PN_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PN->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PN->IE &= tmp;
        }
        break;
    case GPIO_PP:
        /* Check the parameters */
        assert_param(IS_GPIO_PP_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PP->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PP->IE &= tmp;
        }
        break;
    case GPIO_PR:
        /* Check the parameters */
        assert_param(IS_GPIO_PR_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PR->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PR->IE &= tmp;
        }
        break;
    case GPIO_PT:
        /* Check the parameters */
        assert_param(IS_GPIO_PT_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PT->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PT->IE &= tmp;
        }
        break;
    case GPIO_PU:
        /* Check the parameters */
        assert_param(IS_GPIO_PU_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PU->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PU->IE &= tmp;
        }
        break;
    case GPIO_PV:
        /* Check the parameters */
        assert_param(IS_GPIO_PV_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PV->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PV->IE &= tmp;
        }
        break;
    case GPIO_PW:
        /* Check the parameters */
        assert_param(IS_GPIO_PW_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PW->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PW->IE &= tmp;
        }
        break;
    case GPIO_PY:
        /* Check the parameters */
        assert_param(IS_GPIO_PY_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PY->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PY->IE &= tmp;
        }
        break;
    case GPIO_PAA:
        /* Check the parameters */
        assert_param(IS_GPIO_PAA_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAA->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAA->IE &= tmp;
        }
        break;
    case GPIO_PAB:
        /* Check the parameters */
        assert_param(IS_GPIO_PAB_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAB->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAB->IE &= tmp;
        }
        break;
    case GPIO_PAC:
        /* Check the parameters */
        assert_param(IS_GPIO_PAC_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAC->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAC->IE &= tmp;
        }
        break;
    case GPIO_PAD:
        /* Check the parameters */
        assert_param(IS_GPIO_PAD_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAD->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAD->IE &= tmp;
        }
        break;
    case GPIO_PAE:
        /* Check the parameters */
        assert_param(IS_GPIO_PAE_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAE->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAE->IE &= tmp;
        }
        break;
    case GPIO_PAF:
        /* Check the parameters */
        assert_param(IS_GPIO_PAF_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAF->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAF->IE &= tmp;
        }
        break;
    case GPIO_PAG:
        /* Check the parameters */
        assert_param(IS_GPIO_PAG_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAG->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAG->IE &= tmp;
        }
        break;
    case GPIO_PAH:
        /* Check the parameters */
        assert_param(IS_GPIO_PAH_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAH->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAH->IE &= tmp;
        }
        break;
    case GPIO_PAJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PAJ_INPUT_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAJ->IE |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAJ->IE &= tmp;
        }
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Enable or Disable pull-up function of specified GPIO Pin.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PA, GPIO_PB, GPIO_PC, GPIO_PD,
  *   GPIO_PE, GPIO_PF, GPIO_PG, GPIO_PH,
  *   GPIO_PJ, GPIO_PK, GPIO_PL, GPIO_PM,
  *   GPIO_PN, GPIO_PP, GPIO_PR, GPIO_PT,
  *   GPIO_PU, GPIO_PV, GPIO_PW, GPIO_PY,
  *   GPIO_PAA, GPIO_PAB, GPIO_PAC, GPIO_PAD,
  *   GPIO_PAE, GPIO_PAF, GPIO_PAG, GPIO_PAH,
  *   GPIO_PAJ.
  * @param  Bit_x: Select GPIO pin.
  *   This parameter can be one of the following values:
  *   GPIO_BIT_0, GPIO_BIT_1, GPIO_BIT_2, GPIO_BIT_3,
  *   GPIO_BIT_4, GPIO_BIT_5, GPIO_BIT_6, GPIO_BIT_7, GPIO_BIT_ALL,
  *   or combination of the effective bits.
  * @param  NewState: New state of the Pull-Up function.
  *   This parameter can be one of the following values:
  *   ENABLE , DISABLE.
  * @retval None
  */
void GPIO_SetPullUp(GPIO_Port GPIO_x, uint8_t Bit_x, FunctionalState NewState)
{
    uint8_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_GPIO_PULLUP(GPIO_x));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (Bit_x == GPIO_BIT_ALL) {
        switch (GPIO_x) {
        case GPIO_PA:
            Bit_x = GPIO_PA_PULLUP_BIT_ALL;
            break;
        case GPIO_PB:
            Bit_x = GPIO_PB_PULLUP_BIT_ALL;
            break;
        case GPIO_PC:
            Bit_x = GPIO_PC_PULLUP_BIT_ALL;
            break;
        case GPIO_PD:
            Bit_x = GPIO_PD_PULLUP_BIT_ALL;
            break;
        case GPIO_PE:
            Bit_x = GPIO_PE_PULLUP_BIT_ALL;
            break;
        case GPIO_PF:
            Bit_x = GPIO_PF_PULLUP_BIT_ALL;
            break;
        case GPIO_PG:
            Bit_x = GPIO_PG_PULLUP_BIT_ALL;
            break;
        case GPIO_PH:
            Bit_x = GPIO_PH_PULLUP_BIT_ALL;
            break;
        case GPIO_PJ:
            Bit_x = GPIO_PJ_PULLUP_BIT_ALL;
            break;
        case GPIO_PK:
            Bit_x = GPIO_PK_PULLUP_BIT_ALL;
            break;
        case GPIO_PL:
            Bit_x = GPIO_PL_PULLUP_BIT_ALL;
            break;
        case GPIO_PM:
            Bit_x = GPIO_PM_PULLUP_BIT_ALL;
            break;
        case GPIO_PN:
            Bit_x = GPIO_PN_PULLUP_BIT_ALL;
            break;
        case GPIO_PP:
            Bit_x = GPIO_PP_PULLUP_BIT_ALL;
            break;
        case GPIO_PR:
            Bit_x = GPIO_PR_PULLUP_BIT_ALL;
            break;
        case GPIO_PT:
            Bit_x = GPIO_PT_PULLUP_BIT_ALL;
            break;
        case GPIO_PU:
            Bit_x = GPIO_PU_PULLUP_BIT_ALL;
            break;
        case GPIO_PV:
            Bit_x = GPIO_PV_PULLUP_BIT_ALL;
            break;
        case GPIO_PW:
            Bit_x = GPIO_PW_PULLUP_BIT_ALL;
            break;
        case GPIO_PY:
            Bit_x = GPIO_PY_PULLUP_BIT_ALL;
            break;
        case GPIO_PAA:
            Bit_x = GPIO_PAA_PULLUP_BIT_ALL;
            break;
        case GPIO_PAB:
            Bit_x = GPIO_PAB_PULLUP_BIT_ALL;
            break;
        case GPIO_PAC:
            Bit_x = GPIO_PAC_PULLUP_BIT_ALL;
            break;
        case GPIO_PAD:
            Bit_x = GPIO_PAD_PULLUP_BIT_ALL;
            break;
        case GPIO_PAE:
            Bit_x = GPIO_PAE_PULLUP_BIT_ALL;
            break;
        case GPIO_PAF:
            Bit_x = GPIO_PAF_PULLUP_BIT_ALL;
            break;
        case GPIO_PAG:
            Bit_x = GPIO_PAG_PULLUP_BIT_ALL;
            break;
        case GPIO_PAH:
            Bit_x = GPIO_PAH_PULLUP_BIT_ALL;
            break;
        case GPIO_PAJ:
            Bit_x = GPIO_PAJ_PULLUP_BIT_ALL;
            break;
        default:
            /* Do nothing */
            break;
        }
    } else {
        /* Do nothing */
    }
    /* Selects GPIO */
    switch (GPIO_x) {
    case GPIO_PA:
        /* Check the parameters */
        assert_param(IS_GPIO_PA_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PA->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PA->PUP &= tmp;
        }
        break;
    case GPIO_PB:
        /* Check the parameters */
        assert_param(IS_GPIO_PB_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PB->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PB->PUP &= tmp;
        }
        break;
    case GPIO_PC:
        /* Check the parameters */
        assert_param(IS_GPIO_PC_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PC->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PC->PUP &= tmp;
        }
        break;
    case GPIO_PD:
        /* Check the parameters */
        assert_param(IS_GPIO_PD_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PD->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PD->PUP &= tmp;
        }
        break;
    case GPIO_PE:
        /* Check the parameters */
        assert_param(IS_GPIO_PE_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PE->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PE->PUP &= tmp;
        }
        break;
    case GPIO_PF:
        /* Check the parameters */
        assert_param(IS_GPIO_PF_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PF->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PF->PUP &= tmp;
        }
        break;
    case GPIO_PG:
        /* Check the parameters */
        assert_param(IS_GPIO_PG_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PG->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PG->PUP &= tmp;
        }
        break;
    case GPIO_PH:
        /* Check the parameters */
        assert_param(IS_GPIO_PH_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PH->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PH->PUP &= tmp;
        }
        break;
    case GPIO_PJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PJ_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PJ->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PJ->PUP &= tmp;
        }
        break;
    case GPIO_PK:
        /* Check the parameters */
        assert_param(IS_GPIO_PK_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PK->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PK->PUP &= tmp;
        }
        break;
    case GPIO_PL:
        /* Check the parameters */
        assert_param(IS_GPIO_PL_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PL->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PL->PUP &= tmp;
        }
        break;
    case GPIO_PM:
        /* Check the parameters */
        assert_param(IS_GPIO_PM_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PM->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PM->PUP &= tmp;
        }
        break;
    case GPIO_PN:
        /* Check the parameters */
        assert_param(IS_GPIO_PN_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PN->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PN->PUP &= tmp;
        }
        break;
    case GPIO_PP:
        /* Check the parameters */
        assert_param(IS_GPIO_PP_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PP->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PP->PUP &= tmp;
        }
        break;
    case GPIO_PR:
        /* Check the parameters */
        assert_param(IS_GPIO_PR_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PR->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PR->PUP &= tmp;
        }
        break;
    case GPIO_PT:
        /* Check the parameters */
        assert_param(IS_GPIO_PT_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PT->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PT->PUP &= tmp;
        }
        break;
    case GPIO_PU:
        /* Check the parameters */
        assert_param(IS_GPIO_PU_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PU->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PU->PUP &= tmp;
        }
        break;
    case GPIO_PV:
        /* Check the parameters */
        assert_param(IS_GPIO_PV_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PV->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PV->PUP &= tmp;
        }
        break;
    case GPIO_PW:
        /* Check the parameters */
        assert_param(IS_GPIO_PW_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PW->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PW->PUP &= tmp;
        }
        break;
    case GPIO_PY:
        /* Check the parameters */
        assert_param(IS_GPIO_PY_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PY->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PY->PUP &= tmp;
        }
        break;
    case GPIO_PAA:
        /* Check the parameters */
        assert_param(IS_GPIO_PAA_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAA->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAA->PUP &= tmp;
        }
        break;
    case GPIO_PAB:
        /* Check the parameters */
        assert_param(IS_GPIO_PAB_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAB->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAB->PUP &= tmp;
        }
        break;
    case GPIO_PAC:
        /* Check the parameters */
        assert_param(IS_GPIO_PAC_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAC->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAC->PUP &= tmp;
        }
        break;
    case GPIO_PAD:
        /* Check the parameters */
        assert_param(IS_GPIO_PAD_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAD->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAD->PUP &= tmp;
        }
        break;
    case GPIO_PAE:
        /* Check the parameters */
        assert_param(IS_GPIO_PAE_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAE->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAE->PUP &= tmp;
        }
        break;
    case GPIO_PAF:
        /* Check the parameters */
        assert_param(IS_GPIO_PAF_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAF->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAF->PUP &= tmp;
        }
        break;
    case GPIO_PAG:
        /* Check the parameters */
        assert_param(IS_GPIO_PAG_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAG->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAG->PUP &= tmp;
        }
        break;
    case GPIO_PAH:
        /* Check the parameters */
        assert_param(IS_GPIO_PAH_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAH->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAH->PUP &= tmp;
        }
        break;
    case GPIO_PAJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PAJ_PULLUP_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAJ->PUP |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAJ->PUP &= tmp;
        }
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Enable or Disable pull-down function of specified GPIO Pin.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PG.
  * @param  Bit_x: Select GPIO pin.
  *   This parameter can be one of the following values:
  *   GPIO_BIT_0, GPIO_BIT_1, GPIO_BIT_2, GPIO_BIT_3,
  *   GPIO_BIT_4, GPIO_BIT_5, GPIO_BIT_6, GPIO_BIT_7, GPIO_BIT_ALL,
  *   or combination of the effective bits.
  * @param  NewState: New state of the Pull-Down function.
  *   This parameter can be one of the following values:
  *   ENABLE , DISABLE.
  * @retval None
  */
void GPIO_SetPullDown(GPIO_Port GPIO_x, uint8_t Bit_x, FunctionalState NewState)
{
    uint8_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_GPIO_PULLDOWN(GPIO_x));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (Bit_x == GPIO_BIT_ALL) {
        switch (GPIO_x) {
        case GPIO_PG:
            Bit_x = GPIO_PG_PULLDOWN_BIT_ALL;
            break;
        default:
            /* Do nothing */
            break;
        }
    } else {
        /* Do nothing */
    }
    /* Selects GPIO */
    switch (GPIO_x) {
    case GPIO_PG:
        /* Check the parameters */
        assert_param(IS_GPIO_PG_PULLDOWN_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PG->PDN |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PG->PDN &= tmp;
        }
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Set specified GPIO Pin as open drain port or CMOS port.
  *   with Open Drain Register 1.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PE, GPIO_PH, GPIO_PK, GPIO_PM,
  *   GPIO_PR, GPIO_PW, GPIO_PAJ.
  * @param  Bit_x: Select GPIO pin.
  *   This parameter can be one of the following values:
  *   GPIO_BIT_0, GPIO_BIT_1, GPIO_BIT_2, GPIO_BIT_3,
  *   GPIO_BIT_4, GPIO_BIT_5, GPIO_BIT_6, GPIO_BIT_7, GPIO_BIT_ALL,
  *   or combination of the effective bits.
  * @param  NewState: New state of the Open Drian function.
  *   This parameter can be one of the following values:
  *   ENABLE , DISABLE.
  * @retval None
  */
void GPIO_SetOpenDrain(GPIO_Port GPIO_x, uint8_t Bit_x, FunctionalState NewState)
{
    uint8_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_GPIO_OPEN_DRAIN(GPIO_x));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (Bit_x == GPIO_BIT_ALL) {
        switch (GPIO_x) {
        case GPIO_PE:
            Bit_x = GPIO_PE_OPEN_DRAIN_BIT_ALL;
            break;
        case GPIO_PH:
            Bit_x = GPIO_PH_OPEN_DRAIN_BIT_ALL;
            break;
        case GPIO_PK:
            Bit_x = GPIO_PK_OPEN_DRAIN_BIT_ALL;
            break;
        case GPIO_PM:
            Bit_x = GPIO_PM_OPEN_DRAIN_BIT_ALL;
            break;
        case GPIO_PR:
            Bit_x = GPIO_PR_OPEN_DRAIN_BIT_ALL;
            break;
        case GPIO_PW:
            Bit_x = GPIO_PW_OPEN_DRAIN_BIT_ALL;
            break;
        case GPIO_PAJ:
            Bit_x = GPIO_PAJ_OPEN_DRAIN_BIT_ALL;
            break;
        default:
            /* Do nothing */
            break;
        }
    } else {
        /* Do nothing */
    }
    /* Selects GPIO */
    switch (GPIO_x) {
    case GPIO_PE:
        /* Check the parameters */
        assert_param(IS_GPIO_PE_OPEN_DRAIN_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PE->OD |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PE->OD &= tmp;
        }
        break;
    case GPIO_PH:
        /* Check the parameters */
        assert_param(IS_GPIO_PH_OPEN_DRAIN_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PH->OD1 |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PH->OD1 &= tmp;
        }
        break;
    case GPIO_PK:
        /* Check the parameters */
        assert_param(IS_GPIO_PK_OPEN_DRAIN_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PK->OD |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PK->OD &= tmp;
        }
        break;
    case GPIO_PM:
        /* Check the parameters */
        assert_param(IS_GPIO_PM_OPEN_DRAIN_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PM->OD |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PM->OD &= tmp;
        }
        break;
    case GPIO_PR:
        /* Check the parameters */
        assert_param(IS_GPIO_PR_OPEN_DRAIN_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PR->OD |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PR->OD &= tmp;
        }
        break;
    case GPIO_PW:
        /* Check the parameters */
        assert_param(IS_GPIO_PW_OPEN_DRAIN_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PW->OD |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PW->OD &= tmp;
        }
        break;
    case GPIO_PAJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PAJ_OPEN_DRAIN_BIT(Bit_x));
        if (NewState == ENABLE) {
            TSB_PAJ->OD |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PAJ->OD &= tmp;
        }
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Set specified GPIO Pin as open drain port or CMOS port.
  *   with Open Drain Register 2.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PH.
  * @param  Bit_x: Select GPIO pin.
  *   This parameter can be one of the following values:
  *   GPIO_BIT_0, GPIO_BIT_1, GPIO_BIT_2, GPIO_BIT_3,
  *   GPIO_BIT_4, GPIO_BIT_5, GPIO_BIT_6, GPIO_BIT_ALL,
  *   or combination of the effective bits.
  * @param  NewState: New state of the Open Drian function.
  *   This parameter can be one of the following values:
  *   ENABLE , DISABLE.
  * @retval None
  */
void GPIO_SetOpenDrain2(GPIO_Port GPIO_x, uint8_t Bit_x, FunctionalState NewState)
{
    uint8_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_GPIO_OPEN_DRAIN2(GPIO_x));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (GPIO_x == GPIO_PH) {
        if (Bit_x == GPIO_BIT_ALL) {
            Bit_x = GPIO_PH_OPEN_DRAIN2_BIT_ALL;

        } else {
            /* Do nothing */
        }

        /* Check the parameters */
        assert_param(IS_GPIO_PH_OPEN_DRAIN2_BIT(Bit_x));

        if (NewState == ENABLE) {
            TSB_PH->OD2 |= Bit_x;
        } else {
            tmp = (~Bit_x);
            TSB_PH->OD2 &= tmp;
        }
    } else {
        /* Do nothing */
    }

}

/**
  * @brief  Enable specified GPIO Function register.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PA, GPIO_PB, GPIO_PC, GPIO_PD,
  *   GPIO_PE, GPIO_PF, GPIO_PG, GPIO_PH,
  *   GPIO_PJ, GPIO_PK, GPIO_PL, GPIO_PM,
  *   GPIO_PN, GPIO_PP, GPIO_PR, GPIO_PT,
  *   GPIO_PU, GPIO_PV, GPIO_PW, GPIO_PY,
  *   GPIO_PAD, GPIO_PAE, GPIO_PAF, GPIO_PAG,
  *   GPIO_PAJ.
  * @param  FuncReg_x: Select Function register of GPIO.
  *   This parameter can be one of the following values:
  *   GPIO_FUNC_REG_1, GPIO_FUNC_REG_2, GPIO_FUNC_REG_3.
  * @param  Bit_x: Select GPIO pin.
  *   This parameter can be one of the following values:
  *   GPIO_BIT_0, GPIO_BIT_1, GPIO_BIT_2, GPIO_BIT_3,
  *   GPIO_BIT_4, GPIO_BIT_5, GPIO_BIT_6, GPIO_BIT_7,
  *   or combination of the effective bits.
  * @retval None
  */
void GPIO_EnableFuncReg(GPIO_Port GPIO_x, uint8_t FuncReg_x, uint8_t Bit_x)
{
    /* Check the parameters */
    assert_param(IS_GPIO_FUNCTION(GPIO_x));

    /* Selects GPIO */
    switch (GPIO_x) {
    case GPIO_PA:
        /* Check the parameters */
        assert_param(IS_GPIO_PA_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PA_FR1_BIT(Bit_x));
            TSB_PA->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PA_FR3_BIT(Bit_x));
            TSB_PA->FR3 |= Bit_x;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PB:
        /* Check the parameters */
        assert_param(IS_GPIO_PB_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PB_FR1_BIT(Bit_x));
            TSB_PB->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PB_FR3_BIT(Bit_x));
            TSB_PB->FR3 |= Bit_x;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PC:
        /* Check the parameters */
        assert_param(IS_GPIO_PC_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PC_FR1_BIT(Bit_x));
            TSB_PC->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PC_FR2_BIT(Bit_x));
            TSB_PC->FR2 |= Bit_x;
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PC_FR3_BIT(Bit_x));
            TSB_PC->FR3 |= Bit_x;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PD:
        /* Check the parameters */
        assert_param(IS_GPIO_PD_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PD_FR1_BIT(Bit_x));
            TSB_PD->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PD_FR2_BIT(Bit_x));
            TSB_PD->FR2 |= Bit_x;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PE:
        /* Check the parameters */
        assert_param(IS_GPIO_PE_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PE_FR1_BIT(Bit_x));
            TSB_PE->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PE_FR2_BIT(Bit_x));
            TSB_PE->FR2 |= Bit_x;
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PE_FR3_BIT(Bit_x));
            TSB_PE->FR3 |= Bit_x;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PF:
        /* Check the parameters */
        assert_param(IS_GPIO_PF_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PF_FR1_BIT(Bit_x));
            TSB_PF->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PF_FR2_BIT(Bit_x));
            TSB_PF->FR2 |= Bit_x;
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PF_FR3_BIT(Bit_x));
            TSB_PF->FR3 |= Bit_x;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PG:
        /* Check the parameters */
        assert_param(IS_GPIO_PG_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PG_FR1_BIT(Bit_x));
            TSB_PG->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PH:
        /* Check the parameters */
        assert_param(IS_GPIO_PH_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PH_FR1_BIT(Bit_x));
            TSB_PH->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PH_FR2_BIT(Bit_x));
            TSB_PH->FR2 |= Bit_x;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PJ_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PJ_FR1_BIT(Bit_x));
            TSB_PJ->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PK:
        /* Check the parameters */
        assert_param(IS_GPIO_PK_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PK_FR1_BIT(Bit_x));
            TSB_PK->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PK_FR2_BIT(Bit_x));
            TSB_PK->FR2 |= Bit_x;
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PK_FR3_BIT(Bit_x));
            TSB_PK->FR3 |= Bit_x;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PL:
        /* Check the parameters */
        assert_param(IS_GPIO_PL_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PL_FR1_BIT(Bit_x));
            TSB_PL->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PM:
        /* Check the parameters */
        assert_param(IS_GPIO_PM_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PM_FR1_BIT(Bit_x));
            TSB_PM->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PM_FR2_BIT(Bit_x));
            TSB_PM->FR2 |= Bit_x;
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PM_FR3_BIT(Bit_x));
            TSB_PM->FR3 |= Bit_x;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PN:
        /* Check the parameters */
        assert_param(IS_GPIO_PN_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PN_FR1_BIT(Bit_x));
            TSB_PN->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PN_FR3_BIT(Bit_x));
            TSB_PN->FR3 |= Bit_x;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PP:
        /* Check the parameters */
        assert_param(IS_GPIO_PP_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PP_FR1_BIT(Bit_x));
            TSB_PP->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PP_FR2_BIT(Bit_x));
            TSB_PP->FR2 |= Bit_x;
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PP_FR3_BIT(Bit_x));
            TSB_PP->FR3 |= Bit_x;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PR:
        /* Check the parameters */
        assert_param(IS_GPIO_PR_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PR_FR1_BIT(Bit_x));
            TSB_PR->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PR_FR2_BIT(Bit_x));
            TSB_PR->FR2 |= Bit_x;
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PR_FR3_BIT(Bit_x));
            TSB_PR->FR3 |= Bit_x;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PT:
        /* Check the parameters */
        assert_param(IS_GPIO_PT_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PT_FR1_BIT(Bit_x));
            TSB_PT->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PU:
        /* Check the parameters */
        assert_param(IS_GPIO_PU_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PU_FR1_BIT(Bit_x));
            TSB_PU->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PV:
        /* Check the parameters */
        assert_param(IS_GPIO_PV_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PV_FR1_BIT(Bit_x));
            TSB_PV->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PW:
        /* Check the parameters */
        assert_param(IS_GPIO_PW_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PW_FR1_BIT(Bit_x));
            TSB_PW->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PY:
        /* Check the parameters */
        assert_param(IS_GPIO_PY_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PY_FR1_BIT(Bit_x));
            TSB_PY->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PAD:
        /* Check the parameters */
        assert_param(IS_GPIO_PAD_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PAD_FR1_BIT(Bit_x));
            TSB_PAD->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PAE:
        /* Check the parameters */
        assert_param(IS_GPIO_PAE_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PAE_FR1_BIT(Bit_x));
            TSB_PAE->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PAF:
        /* Check the parameters */
        assert_param(IS_GPIO_PAF_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PAF_FR1_BIT(Bit_x));
            TSB_PAF->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PAG:
        /* Check the parameters */
        assert_param(IS_GPIO_PAG_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PAG_FR1_BIT(Bit_x));
            TSB_PAG->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PAG_FR2_BIT(Bit_x));
            TSB_PAG->FR2 |= Bit_x;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PAJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PAJ_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PAJ_FR1_BIT(Bit_x));
            TSB_PAJ->FR1 |= Bit_x;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Disable specified GPIO Function register.
  * @param  GPIO_x: Select GPIO port.
  *   This parameter can be one of the following values:
  *   GPIO_PA, GPIO_PB, GPIO_PC, GPIO_PD,
  *   GPIO_PE, GPIO_PF, GPIO_PG, GPIO_PH,
  *   GPIO_PJ, GPIO_PK, GPIO_PL, GPIO_PM,
  *   GPIO_PN, GPIO_PP, GPIO_PR, GPIO_PT,
  *   GPIO_PU, GPIO_PV, GPIO_PW, GPIO_PY,
  *   GPIO_PAD, GPIO_PAE, GPIO_PAF, GPIO_PAG,
  *   GPIO_PAJ.
  * @param  FuncReg_x: Select Function register of GPIO.
  *   This parameter can be one of the following values:
  *   GPIO_FUNC_REG_1, GPIO_FUNC_REG_2, GPIO_FUNC_REG_3.
  * @param  Bit_x: Select GPIO pin.
  *   This parameter can be one of the following values:
  *   GPIO_BIT_0, GPIO_BIT_1, GPIO_BIT_2, GPIO_BIT_3,
  *   GPIO_BIT_4, GPIO_BIT_5, GPIO_BIT_6, GPIO_BIT_7,
  *   or combination of the effective bits.
  * @retval None
  */
void GPIO_DisableFuncReg(GPIO_Port GPIO_x, uint8_t FuncReg_x, uint8_t Bit_x)
{
    uint8_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_GPIO_FUNCTION(GPIO_x));

    /* Selects GPIO */
    switch (GPIO_x) {
    case GPIO_PA:
        /* Check the parameters */
        assert_param(IS_GPIO_PA_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PA_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PA->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PA_FR3_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PA->FR3 &= tmp;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PB:
        /* Check the parameters */
        assert_param(IS_GPIO_PB_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PB_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PB->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PB_FR3_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PB->FR3 &= tmp;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PC:
        /* Check the parameters */
        assert_param(IS_GPIO_PC_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PC_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PC->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PC_FR2_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PC->FR2 &= tmp;
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PC_FR3_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PC->FR3 &= tmp;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PD:
        /* Check the parameters */
        assert_param(IS_GPIO_PD_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PD_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PD->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PD_FR2_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PD->FR2 &= tmp;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PE:
        /* Check the parameters */
        assert_param(IS_GPIO_PE_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PE_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PE->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PE_FR2_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PE->FR2 &= tmp;
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PE_FR3_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PE->FR3 &= tmp;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PF:
        /* Check the parameters */
        assert_param(IS_GPIO_PF_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PF_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PF->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PF_FR2_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PF->FR2 &= tmp;
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PF_FR3_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PF->FR3 &= tmp;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PG:
        /* Check the parameters */
        assert_param(IS_GPIO_PG_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PG_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PG->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PH:
        /* Check the parameters */
        assert_param(IS_GPIO_PH_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PH_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PH->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PH_FR2_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PH->FR2 &= tmp;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PJ_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PJ_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PJ->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PK:
        /* Check the parameters */
        assert_param(IS_GPIO_PK_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PK_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PK->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PK_FR2_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PK->FR2 &= tmp;
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PK_FR3_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PK->FR3 &= tmp;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PL:
        /* Check the parameters */
        assert_param(IS_GPIO_PL_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PL_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PL->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PM:
        /* Check the parameters */
        assert_param(IS_GPIO_PM_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PM_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PM->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PM_FR2_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PM->FR2 &= tmp;
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PM_FR3_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PM->FR3 &= tmp;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PN:
        /* Check the parameters */
        assert_param(IS_GPIO_PN_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PN_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PN->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PN_FR3_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PN->FR3 &= tmp;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PP:
        /* Check the parameters */
        assert_param(IS_GPIO_PP_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PP_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PP->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PP_FR2_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PP->FR2 &= tmp;
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PP_FR3_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PP->FR3 &= tmp;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PR:
        /* Check the parameters */
        assert_param(IS_GPIO_PR_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PR_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PR->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PR_FR2_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PR->FR2 &= tmp;
            break;
        case GPIO_FUNC_REG_3:
            /* Check the parameters */
            assert_param(IS_GPIO_PR_FR3_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PR->FR3 &= tmp;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PT:
        /* Check the parameters */
        assert_param(IS_GPIO_PT_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PT_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PT->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PU:
        /* Check the parameters */
        assert_param(IS_GPIO_PU_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PU_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PU->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PV:
        /* Check the parameters */
        assert_param(IS_GPIO_PV_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PV_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PV->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PW:
        /* Check the parameters */
        assert_param(IS_GPIO_PW_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PW_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PW->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PY:
        /* Check the parameters */
        assert_param(IS_GPIO_PY_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PY_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PY->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PAD:
        /* Check the parameters */
        assert_param(IS_GPIO_PAD_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PAD_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PAD->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PAE:
        /* Check the parameters */
        assert_param(IS_GPIO_PAE_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PAE_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PAE->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PAF:
        /* Check the parameters */
        assert_param(IS_GPIO_PAF_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PAF_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PAF->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PAG:
        /* Check the parameters */
        assert_param(IS_GPIO_PAG_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PAG_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PAG->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Check the parameters */
            assert_param(IS_GPIO_PAG_FR2_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PAG->FR2 &= tmp;
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    case GPIO_PAJ:
        /* Check the parameters */
        assert_param(IS_GPIO_PAJ_FUNCTION_REG(FuncReg_x));
        switch (FuncReg_x) {
        case GPIO_FUNC_REG_1:
            /* Check the parameters */
            assert_param(IS_GPIO_PAJ_FR1_BIT(Bit_x));
            tmp = (~Bit_x);
            TSB_PAJ->FR1 &= tmp;
            break;
        case GPIO_FUNC_REG_2:
            /* Do nothing */
            break;
        default:
            /* Do nothing */
            break;
        }
        break;
    default:
        /* Do nothing */
        break;
    }
}

/** @} */
/* End of group GPIO_Exported_Functions */
/** @} */
/* End of group GPIO */
/** @} */
/* End of group TX04_Periph_Driver */
#endif                          /*(__TMPM440_GPIO_H) */
