/**
 *******************************************************************************
 * @file	main.c
 * @brief	the application functions of IDLE_KSCAN demo for the TOSHIBA
 *			'TMPM440' Device Series 
 * @version V0.102
 * @date	2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
#include "tmpm440_kscan.h"
#include "tmpm440_gpio.h"
#include "tmpm440_cg.h"
#include "tmpm440_tmrb.h"
#include "sw.h"
#include  "led.h"


/* for TMRB */
#define TMRB19TIME_fPLL_1_1			( 0x7DU&0xFFFFU )       /* TMRB19 for PPG , Ttmrb = 0.08us, 10us/0.08us = 125 = 0x7D */

void configTMRB_TB19OUT(unsigned short time);
void configGPIO_KSCAN(FunctionalState newStateIPUP, FunctionalState newStateOD);
void configKSCAN(void);
void enter_IDLE(void);
void startKSCAN(FunctionalState newState);

/* for MCU */
void __WFI_wait()
{
    __WFI();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
}

/**
  * @brief  The main function of demo
  * @param  None
  * @retval None
  */
int main(void)
{
    SW_Init();
    LED_Init();

    /* Set KSCAN's GPIO */
    configGPIO_KSCAN(ENABLE, DISABLE);

    /* TB19OUT(fPLL:1/1) */
    configTMRB_TB19OUT(TMRB19TIME_fPLL_1_1);

    KSCAN_SetSCLK(KSCAN_KSCL_TBOUT);

    /* KSCAN initial */
    configKSCAN();

    /* KSCAN interrupt */
    CG_ClearINTReq(CG_INT_SRC_INTKSCAN);
    NVIC_ClearPendingIRQ(INTKSCAN_IRQn);

    NVIC_EnableIRQ(INTKSCAN_IRQn);

    KSCAN_SetINTReq(ENABLE);

    startKSCAN(ENABLE);

    LED_On(LED7);
    while (1) {
        if (SW_Get(SW0) == 0) {

            /* LED7 is off before enter IDLE */
            LED_Off(LED7);

            enter_IDLE();

        }
    }
}

/**
  * @brief  Enter the IDLE low power mode
  * @param  None
  * @retval None
  */
void enter_IDLE(void)
{

    CG_SetSTBYReleaseINTSrc(CG_INT_SRC_INTKSCAN, CG_INT_ACTIVE_STATE_RISING, ENABLE);

    /* Set standby mode as IDLE */
    CG_SetSTBYMode(CG_STBY_MODE_IDLE);

    /* to low power mode */
    __DSB();
    __WFI_wait();

    /* INTKSCAN release */
    CG_SetSTBYReleaseINTSrc(CG_INT_SRC_INTKSCAN, CG_INT_ACTIVE_STATE_RISING, DISABLE);
}

/**
  * @brief  ExtINTSrc_IRQHandler
  * @param  None
  * @retval None
  */
void INTKSCAN_IRQHandler(void)
{
    uint8_t ledval;

    CG_ClearINTReq(CG_INT_SRC_INTKSCAN);

    ledval = LED7;              /* NORMAL mode: LED7 is ON */
    ledval |= (~(KSCAN_ReadBufChecked(0U))) & GPIO_BIT_0;       /* KSOUT0xKSIN0 */
    ledval |= (~(KSCAN_ReadBufChecked(1U))) & GPIO_BIT_1;       /* KSOUT1xKSIN1 */
    ledval |= (~(KSCAN_ReadBufChecked(2U))) & GPIO_BIT_2;       /* KSOUT2xKSIN2 */

    LED_On(ledval);
    LED_Off(~ledval);
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1U) {
        __NOP();
    }
}
#endif
