/**
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of EPHC demo for the TOSHIBA
 *          'TMPM440' Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm440_gpio.h"
#include "tmpm440_ephc.h"

/* Private macro -------------------------------------------------------------*/
#define     GPIO_EPHC0           GPIO_PR
#define     EPHC0IN0_BIT         GPIO_BIT_6
#define     EPHC0IN1_BIT         GPIO_BIT_7
#define     GPIO_EPHCOUT         GPIO_PR
#define     EPHC0IN0_OUT_BIT     GPIO_BIT_2
#define     EPHC0IN1_OUT_BIT     GPIO_BIT_3
/* Private variables ---------------------------------------------------------*/
volatile uint16_t count_result[100] = {0U};
volatile uint32_t count_num = 0;

/* Private function prototypes -----------------------------------------------*/

/*******************************************************************************
* Function Name  : App_EPHCNT_Init
* Description    : Initialize phase counter
* Input          : None.
* Return         : None.
*******************************************************************************/
void App_EPHCNT_Init(void)
{
    EPHC_InitTypeDef InitStruct;

    /* PR6/PR7 EPHC0IN0/EPHC0IN1 setting */
    GPIO_SetInput(GPIO_EPHC0, (EPHC0IN0_BIT | EPHC0IN1_BIT));
    GPIO_SetOutput(GPIO_EPHCOUT, (EPHC0IN0_OUT_BIT | EPHC0IN1_OUT_BIT));
    GPIO_EnableFuncReg(GPIO_EPHC0, GPIO_FUNC_REG_3, (EPHC0IN0_BIT | EPHC0IN1_BIT));
    GPIO_DisableFuncReg(GPIO_EPHCOUT, GPIO_FUNC_REG_2, (EPHC0IN0_OUT_BIT | EPHC0IN0_OUT_BIT));
    GPIO_SetInputEnableReg(GPIO_EPHC0, (EPHC0IN0_BIT | EPHC0IN1_BIT), ENABLE);
    GPIO_SetOutputEnableReg(GPIO_EPHCOUT, (EPHC0IN0_OUT_BIT | EPHC0IN0_OUT_BIT), ENABLE);

    InitStruct.CountDownEdgeSelForP1 = EPHC_CNT_MA1DN_NOCNTDOWN0;        
    InitStruct.CountUpEdgeSelForP1 = EPHC_CNT_MA1UP_NOCNTUP0;             
    InitStruct.InputClkSelection = EPHC_CNT_BRCK_FC;                      
    InitStruct.PhaseSelection = EPHC_CNT_PBDIR_POSITIVEPHASE;             
    InitStruct.ModeSetting = EPHC_CNT_MA12_PULSE_2;                       
    InitStruct.DirectionSetting = EPHC_CNT_MA2DIR_NEGATIVEDIR;            
    InitStruct.NoiseFilterCtrl = EPHC_CNT_NOISEFILTER_NONE;               
    InitStruct.CountClearCtrl = EPHC_COUNT_CLR;
    
    EPHC_Enable(TSB_EPHC);                            /* Enable EPHC Operation */
    EPHC_Init(TSB_EPHC, &InitStruct);                 /* Set InitStruct to EPHC */
    EPHC_DisableInterrupt(TSB_EPHC, EPHC_IE_INT_ALL);  /* Disable all interrupt */
    EPHC_EnableInterrupt(TSB_EPHC,EPHC_IE_INT_ALL);  /* Enable interrupt */
    NVIC_EnableIRQ(INTEPHC_IRQn);                 
    EPHC_ASetRunState(TSB_EPHC, EPHC_RUN);              /* Run EPHC */
}

/*******************************************************************************
* Function Name  : App_EPHCNT_Monitor
* Description    : Monitor the phase counter
* Input          : None.
* Return         : None.
*******************************************************************************/
void delay(void)
{
  uint32_t i;
  for(i = 0; i<10; i++);
}
int main(void)
{    
    uint32_t i;
    
    App_EPHCNT_Init();
    
    for(i = 0; i < 10; i++)
    {
      /* Generate Input */
      /* PR6(output) -> EPHC0IN0 */
      /* PR7(output) -> EPHC0IN1 */
      GPIO_WriteDataBit(GPIO_EPHCOUT, EPHC0IN0_OUT_BIT,GPIO_BIT_VALUE_1);
      GPIO_WriteDataBit(GPIO_EPHCOUT, EPHC0IN1_OUT_BIT,GPIO_BIT_VALUE_0);
      delay();
      count_result[count_num++] = EPHC_AGetPulseCntValue(TSB_EPHC);/* get count value */
      GPIO_WriteDataBit(GPIO_EPHCOUT, EPHC0IN0_OUT_BIT,GPIO_BIT_VALUE_1);
      GPIO_WriteDataBit(GPIO_EPHCOUT, EPHC0IN1_OUT_BIT,GPIO_BIT_VALUE_1);
      delay();
      count_result[count_num++] = EPHC_AGetPulseCntValue(TSB_EPHC);/* get count value */
      GPIO_WriteDataBit(GPIO_EPHCOUT, EPHC0IN0_OUT_BIT,GPIO_BIT_VALUE_0);
      GPIO_WriteDataBit(GPIO_EPHCOUT, EPHC0IN1_OUT_BIT,GPIO_BIT_VALUE_1);
      delay();
      count_result[count_num++] = EPHC_AGetPulseCntValue(TSB_EPHC);/* get count value */
      GPIO_WriteDataBit(GPIO_EPHCOUT, EPHC0IN0_OUT_BIT,GPIO_BIT_VALUE_0);
      GPIO_WriteDataBit(GPIO_EPHCOUT, EPHC0IN1_OUT_BIT,GPIO_BIT_VALUE_0);
      delay();
      count_result[count_num++] = EPHC_AGetPulseCntValue(TSB_EPHC);/* get count value */
    }
    for(i = 0; i < 10; i++)
    {
      GPIO_WriteDataBit(GPIO_EPHCOUT, EPHC0IN0_OUT_BIT,GPIO_BIT_VALUE_0);
      GPIO_WriteDataBit(GPIO_EPHCOUT, EPHC0IN1_OUT_BIT,GPIO_BIT_VALUE_1);
      delay();
      count_result[count_num++] = EPHC_AGetPulseCntValue(TSB_EPHC);/* get count value */
      GPIO_WriteDataBit(GPIO_EPHCOUT, EPHC0IN0_OUT_BIT,GPIO_BIT_VALUE_1);
      GPIO_WriteDataBit(GPIO_EPHCOUT, EPHC0IN1_OUT_BIT,GPIO_BIT_VALUE_1);
      delay();
      count_result[count_num++] = EPHC_AGetPulseCntValue(TSB_EPHC);/* get count value */
      GPIO_WriteDataBit(GPIO_EPHCOUT, EPHC0IN0_OUT_BIT,GPIO_BIT_VALUE_1);
      GPIO_WriteDataBit(GPIO_EPHCOUT, EPHC0IN1_OUT_BIT,GPIO_BIT_VALUE_0);
      delay();
      count_result[count_num++] = EPHC_AGetPulseCntValue(TSB_EPHC);/* get count value */
      GPIO_WriteDataBit(GPIO_EPHCOUT, EPHC0IN0_OUT_BIT,GPIO_BIT_VALUE_0);
      GPIO_WriteDataBit(GPIO_EPHCOUT, EPHC0IN1_OUT_BIT,GPIO_BIT_VALUE_0);
      delay();
      count_result[count_num++] = EPHC_AGetPulseCntValue(TSB_EPHC);/* get count value */
    }
    while(1);
}

void INTEPHC_IRQHandler(void)
{
     /* if overflow or underflow interrupt occured,clear EPHC0ADAT to 0. */
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif

