/**
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of KSCAN demo for the TOSHIBA
 *          'TMPM440' Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
 
#include "tmpm440_kscan.h"
#include "tmpm440_gpio.h"
#include "tmpm440_cg.h"
#include  "led.h"

#define  WARMTIME           0xC35U
#define GPIO_KSIN           GPIO_PAH
#define GPIO_KSOUT          GPIO_PAJ 
#define KSOUT_FUNC_REG      GPIO_FUNC_REG_1

void KSCAN_GPIOConfig(void);
int main()
{
    WorkState st = BUSY;
    LED_Init();
    /*Set clock to FS*/
    CG_SetFs(ENABLE);
    CG_SetWarmUpTime(CG_WARM_UP_SRC_OSC_EXT,WARMTIME);
    do {
        st = CG_GetWarmUpState();
    } while (st != DONE);
    /*configure the KSCAN GPIO*/
    KSCAN_GPIOConfig();
    /*Clock starts*/
    KSCAN_SetSCLK(KSCAN_KSCL_FS);
    /*Set KSEN(SC)*/
    
    /*Set KSEN<EN>*/
    KSCAN_Enable();

    /*Set KSICR, KSOCR*/
    KSCAN_SetInputCtrlMask(KSCAN_CH_0,DISABLE); /*All channels are not masked*/
  
    KSCAN_SetOutputCtrl((~KSCAN_CH_0),ENABLE);
    KSCAN_SetStrobeOutput(2U);
    /*KSTCR*/
    KSCAN_SetStrobeWidth(3U);
    KSCAN_SetChatCancelTime(60U);   /*Sets a chattering cancel time*/
    
    KSCAN_SetConsecutiveMatches(KSCAN_MATCH_2C);
    /*KSINTCR<INTEN>*/
    KSCAN_SetINTReq(ENABLE);
    /*Set CG interrupt*/
    CG_SetSTBYReleaseINTSrc(CG_INT_SRC_INTKSCAN,CG_INT_ACTIVE_STATE_RISING,ENABLE);
    NVIC_EnableIRQ(INTKSCAN_IRQn);
    KSCAN_Start();
    while(1){
    }
}

void INTKSCAN_IRQHandler()
{
  uint8_t retval;
  CG_ClearINTReq(CG_INT_SRC_INTKSCAN);
  retval = KSCAN_ReadBufChecked(1U);
  LED_On(retval);
  LED_Off(~retval);
}

void KSCAN_GPIOConfig()
{ 
  GPIO_SetInput(GPIO_KSIN,GPIO_BIT_ALL);
  GPIO_SetOutput(GPIO_KSOUT,GPIO_BIT_ALL);
  GPIO_EnableFuncReg(GPIO_KSOUT,KSOUT_FUNC_REG,GPIO_BIT_ALL);
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1U) {
        __NOP();
    }
}
#endif

