/**
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of PHC demo for the TOSHIBA
 *          'TMPM440' Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm440_gpio.h"
#include "tmpm440_phc.h"

/* Private macro -------------------------------------------------------------*/
#define     GPIO_PHC0           GPIO_PU
#define     PHC0IN0_BIT         GPIO_BIT_4
#define     PHC0IN1_BIT         GPIO_BIT_5
#define     GPIO_PHCOUT         GPIO_PU
#define     PHC0IN0_OUT_BIT     GPIO_BIT_2
#define     PHC0IN1_OUT_BIT     GPIO_BIT_3
/* Private variables ---------------------------------------------------------*/
volatile uint16_t count_result[100U] = {0U};
volatile uint32_t count_num = 0U;

/* Private function prototypes -----------------------------------------------*/
void App_PHCNT_Init(void);
void delay(void);
/*******************************************************************************
* Function Name  : App_PHCNT_Init
* Description    : Initialize phase counter
* Input          : None.
* Return         : None.
*******************************************************************************/
void App_PHCNT_Init(void)
{
    PHC_InitTypeDef InitStruct;

    /*PHC0IN0/PHC0IN1 setting */
    GPIO_SetInput(GPIO_PHC0, (PHC0IN0_BIT | PHC0IN1_BIT));
    GPIO_SetOutput(GPIO_PHCOUT, (PHC0IN0_OUT_BIT | PHC0IN1_OUT_BIT));
    GPIO_EnableFuncReg(GPIO_PHC0, GPIO_FUNC_REG_1, (PHC0IN0_BIT | PHC0IN1_BIT));
    GPIO_DisableFuncReg(GPIO_PHCOUT, GPIO_FUNC_REG_1, (PHC0IN0_OUT_BIT | PHC0IN0_OUT_BIT));

    InitStruct.Mode = PHC_CR_MODE_4TIMES;                /* Quadruple mode */
    InitStruct.NoiseFilterCtrl = PHC_CR_NOISEFILTER_ON;  /* Noise Filter on */
    InitStruct.CountClearCtrl = PHC_COUNT_CLR;           /* Clear counter */
    
    PHC_Enable(TSB_PHC0);                            /* Enable PHC Operation */
    PHC_Init(TSB_PHC0, &InitStruct);                 /* Set InitStruct to PHC */
    PHC_DisableInterrupt(TSB_PHC0, PHC_CR_INT_ALL);  /* Disable all interrupt */
    PHC_EnableInterrupt(TSB_PHC0,PHC_CR_INT_EVERY);  /* Enable INTPHEVRY0 interrupt */
    NVIC_EnableIRQ(INTPHC0EVRY_IRQn);                 
    PHC_SetRunState(TSB_PHC0, PHC_RUN);              /* Run PHC0 */
}

/*******************************************************************************
* Function Name  : delay
* Description    : time delay 
* Input          : None.
* Return         : None.
*******************************************************************************/
void delay(void)
{
  uint32_t i;
  for(i = 0U; i<10U; i++);
}
/*******************************************************************************
* Function Name  : main
* Description    : the main function
* Input          : None.
* Return         : None.
*******************************************************************************/
int main(void)
{    
    uint32_t i;
    
    App_PHCNT_Init();
    
    for(i = 0U; i < 10U; i++)
    {
      /* Generate Input */
      GPIO_WriteDataBit(GPIO_PHCOUT, PHC0IN0_OUT_BIT,GPIO_BIT_VALUE_1);
      GPIO_WriteDataBit(GPIO_PHCOUT, PHC0IN1_OUT_BIT,GPIO_BIT_VALUE_0);
      delay();
      GPIO_WriteDataBit(GPIO_PHCOUT, PHC0IN0_OUT_BIT,GPIO_BIT_VALUE_1);
      GPIO_WriteDataBit(GPIO_PHCOUT, PHC0IN1_OUT_BIT,GPIO_BIT_VALUE_1);
      delay();
      GPIO_WriteDataBit(GPIO_PHCOUT, PHC0IN0_OUT_BIT,GPIO_BIT_VALUE_0);
      GPIO_WriteDataBit(GPIO_PHCOUT, PHC0IN1_OUT_BIT,GPIO_BIT_VALUE_1);
      delay();
      GPIO_WriteDataBit(GPIO_PHCOUT, PHC0IN0_OUT_BIT,GPIO_BIT_VALUE_0);
      GPIO_WriteDataBit(GPIO_PHCOUT, PHC0IN1_OUT_BIT,GPIO_BIT_VALUE_0);
      delay();
    }
    for(i = 0U; i < 10U; i++)
    {
      GPIO_WriteDataBit(GPIO_PHCOUT, PHC0IN0_OUT_BIT,GPIO_BIT_VALUE_0);
      GPIO_WriteDataBit(GPIO_PHCOUT, PHC0IN1_OUT_BIT,GPIO_BIT_VALUE_1);
      delay();
      GPIO_WriteDataBit(GPIO_PHCOUT, PHC0IN0_OUT_BIT,GPIO_BIT_VALUE_1);
      GPIO_WriteDataBit(GPIO_PHCOUT, PHC0IN1_OUT_BIT,GPIO_BIT_VALUE_1);
      delay();
      GPIO_WriteDataBit(GPIO_PHCOUT, PHC0IN0_OUT_BIT,GPIO_BIT_VALUE_1);
      GPIO_WriteDataBit(GPIO_PHCOUT, PHC0IN1_OUT_BIT,GPIO_BIT_VALUE_0);
      delay();
      GPIO_WriteDataBit(GPIO_PHCOUT, PHC0IN0_OUT_BIT,GPIO_BIT_VALUE_0);
      GPIO_WriteDataBit(GPIO_PHCOUT, PHC0IN1_OUT_BIT,GPIO_BIT_VALUE_0);
      delay();
    }
    while(1U);
}
/*******************************************************************************
* Function Name  : INTPHC0EVRY_IRQHandler
* Description    : PHC0 interrupt
* Input          : None.
* Return         : None.
*******************************************************************************/
void INTPHC0EVRY_IRQHandler(void)
{
    if(count_num < 100U) {          /* Judge if overflow count_result[] or not */
        count_result[count_num++] = PHC_GetPulseCntValue(TSB_PHC0);/* get count value */
    } else {
        /* Do nothing */
    }
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1U) {
        __NOP();
    }
}
#endif

