/** 
 *******************************************************************************
 * @file	psc.c
 * @brief	the application functions of PSC_Repeat_Proc demo for the TOSHIBA
 *			'TMPM440' Device Series 
 * @version	V2.0.2.3
 * @date	2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/* Includes ------------------------------------------------------------------*/
#include "psc.h"
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define PSC_DATAADDRSTA      				((uint32_t)0x40014000)            /* PSC DATA TOP Address */
#define PSC_CODEADDRSTA      				((uint32_t)0x40010000)            /* PSC CODE TOP Address */
#define PSC_DATAADDREND      				((uint32_t)0x40015fff)            /* PSC DATA END Address */
#define PSC_CODEADDREND      				((uint32_t)0x40011fff)            /* PSC CODE END Address */
#define PSC_DATATRANSSIZE    				((uint32_t)(0x00002000UL / 4))    /* PSC DATA Transmission size maximum : 8KByte */
#define PSC_CODETRANSSIZE    				((uint32_t)(0x00002000UL / 4))    /* PSC CODE Transmission size maximum : 8KByte */
#define PSC_STARTADDR        				PSC_CODEADDRSTA

#if defined ( __CC_ARM   )      /* RealView Compiler */

extern uint32_t Image$$ROM_CODE$$Base;
extern uint32_t Image$$ROM_CODE$$ZI$$Length;
extern uint32_t Image$$ROM_DATA$$Base;
extern uint32_t Image$$ROM_DATA$$ZI$$Length;

#define PSC_CODEROM      (uint32_t *)&Image$$ROM_CODE$$Base
#define PSC_CODEROMSIZE  (uint32_t)&Image$$ROM_CODE$$ZI$$Length
#define PSC_DATAROM      (uint32_t *)&Image$$ROM_DATA$$Base
#define PSC_DATAROMSIZE  (uint32_t)&Image$$ROM_DATA$$ZI$$Length

#elif defined ( __ICCARM__ )    /* IAR Compiler */
#pragma section = "PSC_CodeBinary_init"
#pragma section = "PSC_DataBinary_init"

#define PSC_CODEROM      ((uint32_t*) __section_begin("PSC_CodeBinary_init"))
#define PSC_CODEROMEND   ((uint32_t*) __section_end("PSC_CodeBinary_init"))
#define PSC_CODEROMSIZE  ((uint32_t)(PSC_CODEROMEND - PSC_CODEROM))

#define PSC_DATAROM      ((uint32_t*) __section_begin("PSC_DataBinary_init"))
#define PSC_DATAROMEND   ((uint32_t*) __section_end("PSC_DataBinary_init"))
#define PSC_DATAROMSIZE  ((uint32_t)(PSC_DATAROMEND - PSC_DATAROM))
#endif

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
uint16_t    gPSC_codeChksum;                         /* PSC CODE checksum */
uint16_t    gPSC_dataChksum;                         /* PSC DATA checksum */
uint32_t    GetPSC_DestAddr;
/* external variables --------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/

/**
  * @brief  DMA PSC code/data Transmission
  * @param  None.
  * @retval None.
  */
void PSC_DMATransfer(void)
{
    DMAC_InitTypeDef DMAC_InitStruct;
    DMAC_Channel myChannel = DMAC_CHANNEL_0;
    uint8_t    *pData;

    /* DMA channel3 configuration */
    DMAC_InitStruct.TxINT = ENABLE;
    DMAC_InitStruct.SrcAddr = (uint32_t) PSC_CODEROM;
    DMAC_InitStruct.SrcIncrementState = ENABLE;
    DMAC_InitStruct.SrcBitWidth = DMAC_WORD;
    DMAC_InitStruct.SrcBurstSize = DMAC_1_BEAT;
    DMAC_InitStruct.DstAddr = (uint32_t)PSC_CODEADDRSTA;
    DMAC_InitStruct.DstIncrementState = ENABLE;
    DMAC_InitStruct.DstBitWidth = DMAC_WORD;
    DMAC_InitStruct.DstBurstSize = DMAC_1_BEAT;
    DMAC_InitStruct.TxSize = PSC_CODETRANSSIZE;
    DMAC_InitStruct.TxDirection = DMAC_MEMORY_TO_MEMORY;

    /* Enable DMA circuit */
    DMAC_Enable(DMAC_UNIT_C);
    /* PSC data/code RAMs use little-endian, so DMA should only be set to little-endian */
    DMAC_SetTransferEndian(DMAC_UNIT_C, DMAC_LITTLE_ENDIAN);
    /*Initialize DMA channel */
    DMAC_Init(DMAC_UNIT_C, myChannel, &DMAC_InitStruct);
    DMAC_SetTxINTConfig(DMAC_UNIT_C, myChannel, DMAC_INT_TX_END, ENABLE);

    pData = (uint8_t*)PSC_CODEADDRSTA;
    gPSC_codeChksum = 0x0000;                            /* clear checksum */

    DMAC_SetDMAChannel(DMAC_UNIT_C, myChannel, ENABLE);


    do {
        GetPSC_DestAddr = TSB_DMACC->C0DESTADDR + 0x03;
        if (pData < (uint8_t*)GetPSC_DestAddr) {
            gPSC_codeChksum += *pData;
            pData++;
        }
    } while ((DMAC_GetTxINTReq(DMAC_UNIT_C, myChannel) != DMAC_TX_END_REQ)
            || (pData < (uint8_t*)PSC_CODEADDREND));     /* Was calculation of the checksum completed? */

    DMAC_ClearTxINTReq(DMAC_UNIT_C, myChannel, DMAC_INT_TX_END);

    /* DMA channel3 configuration */
    DMAC_InitStruct.TxINT = ENABLE;
    DMAC_InitStruct.SrcAddr = (uint32_t) PSC_DATAROM;
    DMAC_InitStruct.SrcIncrementState = ENABLE;
    DMAC_InitStruct.SrcBitWidth = DMAC_WORD;
    DMAC_InitStruct.SrcBurstSize = DMAC_1_BEAT;
    DMAC_InitStruct.DstAddr = (uint32_t)PSC_DATAADDRSTA;
    DMAC_InitStruct.DstIncrementState = ENABLE;
    DMAC_InitStruct.DstBitWidth = DMAC_WORD;
    DMAC_InitStruct.DstBurstSize = DMAC_1_BEAT;
    DMAC_InitStruct.TxSize = PSC_DATATRANSSIZE;
    DMAC_InitStruct.TxDirection = DMAC_MEMORY_TO_MEMORY;

    /*Enable DMA circuit */
    DMAC_Enable(DMAC_UNIT_C);
    /*Initialize DMA channel */
    DMAC_Init(DMAC_UNIT_C, myChannel, &DMAC_InitStruct);
    DMAC_SetTxINTConfig(DMAC_UNIT_C, myChannel, DMAC_INT_TX_END, ENABLE);

    pData = (uint8_t*)PSC_DATAADDRSTA;
    gPSC_dataChksum = 0x0000;                            /* clear checksum */

    DMAC_SetDMAChannel(DMAC_UNIT_C, myChannel, ENABLE);

    do {
        GetPSC_DestAddr = TSB_DMACC->C0DESTADDR + 0x03;
        if (pData < (uint8_t*)GetPSC_DestAddr) {
            gPSC_dataChksum += *pData;
            pData++;
        }
    } while ((DMAC_GetTxINTReq(DMAC_UNIT_C, myChannel) != DMAC_TX_END_REQ)
            || (pData < (uint8_t*)PSC_DATAADDREND));    /* Was calculation of the checksum completed? */

    DMAC_ClearTxINTReq(DMAC_UNIT_C, myChannel, DMAC_INT_TX_END);
}

/**
  * @brief  PSC Initialization
  * @param  None.
  * @retval None.
  */
void PSC_Init(void)
{
    PSC_SetRepetProcVectPointer(PSC_STARTADDR);
    PSC_SetTriggerEventFUNC(PSC_TRIGGER_EVENT_0, ENABLE);
}
