/**
 *******************************************************************************
 * @file    main.c
 * @brief   The main functions of SBI slave demo for the
 *          TOSHIBA 'TMPM440' Device Series
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

#include "main.h"

Result gTRxResult;
SBI_InitI2CTypeDef myI2Cm, myI2Cs;
uint32_t gI2CWCnt;
uint32_t gI2CRCnt;
uint32_t gI2CTxDataLen;
char gI2CTxData[16] = { 0U };

uint32_t gI2CRxDataLen;
char gI2CRxData[16] = { 0U };

uint32_t gSBIMode;

/**
  * @brief  The main routine of SBI demo I2C
  * @param  None
  * @retval None
  */
int main(void)
{
    uint32_t gICnt;
    SBI_I2CState i2c_state;
    SW_Init();
    LED_Init();
    /*  Initialize IO for SBIs */
    SBI_IO_Configuration();
    /* Initialize variables for SBIs */
    /* Master mode */
    myI2Cm.I2CSelfAddr = SELF_ADDR;
    myI2Cm.I2CDataLen = SBI_I2C_DATA_LEN_8;
    myI2Cm.I2CACKState = ENABLE;
    myI2Cm.I2CClkDiv = SBI_I2C_CLK_DIV_328;
    /* Slave mode */
    myI2Cs.I2CSelfAddr = SLAVE_ADDR;
    myI2Cs.I2CDataLen = SBI_I2C_DATA_LEN_8;
    myI2Cs.I2CACKState = ENABLE;
    myI2Cs.I2CClkDiv = SBI_I2C_CLK_DIV_328;
    gSBIMode = MODE_SBI_I2C_IDLE;
    NVIC_EnableIRQ(INTI2C_IRQn);

    while (1U) {
    
        switch (gSBIMode) {
        
        case MODE_SBI_I2C_IDLE:
            gI2CTxDataLen = 0U;
            gI2CWCnt = 0U;
            gI2CRxDataLen = 0U;
            gI2CRCnt = 0U;
        
            if (SW_Get(SW0)) {
                gSBIMode = MODE_SBI_MASTER_INITIAL;
            } else if (SW_Get(SW2)) {
                gSBIMode = MODE_SBI_SLAVE_INITIAL;
            } else {
                /* Do Nothing */
            }
            break;
            /* Initialize TRx buffer and Tx length */
        case MODE_SBI_MASTER_INITIAL:
            /* Initialize SBI channel to Master mode */
        
            gI2CTxDataLen = 7U;
            gI2CTxData[0] = gI2CTxDataLen;
            gI2CTxData[1] = 'T';
            gI2CTxData[2] = 'O';
            gI2CTxData[3] = 'S';
            gI2CTxData[4] = 'H';
            gI2CTxData[5] = 'I';
            gI2CTxData[6] = 'B';
            gI2CTxData[7] = 'A';
        
            while (SW_Get(SW1) == 0U) {
                /* Do Nothing */
            }
        
            SBI_Enable(TSB_SBI0);
            SBI_SWReset(TSB_SBI0);
            SBI_InitI2C(TSB_SBI0, &myI2Cm);
            gSBIMode = MODE_SBI_I2C_START;
            break;
        
        case MODE_SBI_SLAVE_INITIAL:
            /* Initialize SBI channel to Slave mode */
        
            gI2CWCnt = 0U;
            for (gICnt = 0U; gICnt < 8U; gICnt++) {
                gI2CRxData[gICnt] = 0U;
            }
        
            while (SW_Get(SW1) == 0U) {
                /* Do Nothing */
            }
            SBI_Enable(TSB_SBI0);
            SBI_SWReset(TSB_SBI0);
            SBI_InitI2C(TSB_SBI0, &myI2Cs);
            gSBIMode = MODE_SBI_I2C_RX;
        
            break;
        
            /* Check I2C bus state and start Tx */
        case MODE_SBI_I2C_START:
            i2c_state = SBI_GetI2CState(TSB_SBI0);
            if (!i2c_state.Bit.BusState) {
                SBI_SetSendData(TSB_SBI0, SLAVE_ADDR | SBI_I2C_SEND);
                SBI_GenerateI2CStart(TSB_SBI0);
                gSBIMode = MODE_SBI_I2C_TX;
                LED_On(LED0); 
            } else {
                /* Do nothing */
            }
            break;
            /* Read gI2CRCnt to check Rx end */
        case MODE_SBI_I2C_RX:
            LED_On(LED4);
            if (gI2CRCnt > gI2CRxData[0]) {
                gI2CRxDataLen = gI2CRxData[0];
                gI2CRCnt = 0U;
                gSBIMode = MODE_SBI_SLAVE_FINISHED;
            } else {
                /* Do nothing */
            }
            break;
            /* Read gI2CWCnt to check Tx end */
        case MODE_SBI_I2C_TX:
            LED_On(LED1);
            if (gI2CWCnt > gI2CTxData[0]) {
                gSBIMode = MODE_SBI_MASTER_FINISHED;
            } else {
                /* Do nothing */
            }
            break;
            /* Check transmission result */
        case MODE_SBI_MASTER_FINISHED:
            LED_On(LED2);
            if (SW_Get(SW3)) {
                SBI_Disable(TSB_SBI0);
                gSBIMode = MODE_SBI_I2C_IDLE;
            } else {
                /* Do Nothing */
            }
            LED_Off(LED0);
            LED_Off(LED1);
            LED_Off(LED2);
            break;
        
        case MODE_SBI_SLAVE_FINISHED:
        
            LED_On(LED5);
            if ((gI2CTxData[7] == gI2CRxData[gI2CRxDataLen])) {
                gTRxResult = SUCCESS;
                LED_On(LED6);
            } else {
                gTRxResult = ERROR;
                LED_On(LED7);
            }
            if (SW_Get(SW3)) {
                SBI_Disable(TSB_SBI0);
                gSBIMode = MODE_SBI_I2C_IDLE;
            } else {
                /* Do Nothing */
            }
            LED_Off(LED7);
            LED_Off(LED6);
            LED_Off(LED5);
            LED_Off(LED4);
            break;
        
        default:
        
            gSBIMode = MODE_SBI_I2C_IDLE;
            break;
        }
    }
}

/**
  * @brief  Config the GPIO for SBI
  * @param  None
  * @retval None
  */
void SBI_IO_Configuration(void)
{
    TSB_PR->FR1 |= GPIO_BIT_4 | GPIO_BIT_5;
    TSB_PR->CR |= GPIO_BIT_4 | GPIO_BIT_5;
    TSB_PR->IE |= GPIO_BIT_4 | GPIO_BIT_5;
    TSB_PR->OD |= GPIO_BIT_4 | GPIO_BIT_5;
    TSB_PR->PUP |= GPIO_BIT_4 | GPIO_BIT_5;
}   


#ifdef DEBUG
/**
  * @brief  Deal with the error parameter
  * @param  file: Pointer to the file where the error parameter locates
  * @param  line: Number of the line in which the error parameter locates
  * @retval None
  */
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
