/**
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of TMRC demo for the TOSHIBA
 *          'TMPM440' Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
#include "tmpm440_tmrc.h"
#include "tmpm440_cg.h"

/* fosc = 10MHz(external high-speed oscillator)
 * fpll = 10 * fosc = 100MHz(in this demo, PLL multiplying value is set to 10 multiplying)
 * fperiph= fc = fpll = 100MHz
 * T0(prescaler clock) = fperiph(Prescaler clock selection)=100MHz
 * T2(source clock) = T0/4 = 25MHz
 */
int main(void)
{
    uint32_t regval = 0U;

    /* TMRC CG setting */
     CG_SetPhiT0Src(CG_PHIT0_SRC_FC);
     CG_SetPhiT0Level(CG_DIVIDE_1);
     CG_SetFcPeriphB(FC_B_ALL, ENABLE);
    
    /* TMRC enable */
    TMRC_Enable(TSB_TC);
    /* TMRC IDLE mode */
    TMRC_SetIdleMode(TSB_TC, ENABLE);
    /* TMRC source clock setting TBTCLK[3:0]0001: T2*/
    TMRC_SetSrcClk(TSB_TC, TMRC_CLK_DIV_4);
    /* Enable TMRC intterrupt */
    NVIC_EnableIRQ(INTTCTBT_IRQn);
    /* Start to run TMRC */
    TMRC_SetRunState(TSB_TC, TMRC_RUN);
    
    while (1) {
        /* software capture  */
      TMRC_ExecuteSWCapture(TSB_TC);
      regval = TMRC_GetSWCaptureValue(TSB_TC);
    }
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
