/**
 *******************************************************************************
 * @file	kscan.c
 * @brief	KSCAN driver for the TOSHIBA 'TMPM440' Device Series
 *			'TMPM440' Device Series 
 * @version V0.101
 * @date	2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/** include files **/
#include "tmpm440_kscan.h"
#include "tmpm440_gpio.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

/* GPIO PORT*/
#define GPIO_KSIN			GPIO_PAH
#define GPIO_KSOUT			GPIO_PAJ
#define KSOUT_FUNC_REG		GPIO_FUNC_REG_1

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* external variables --------------------------------------------------------*/
/* Exported functions ------------------------------------------------------- */
void configGPIO_KSCAN(FunctionalState newStateIPUP, FunctionalState newStateOD);
void configKSCAN(void);
void startKSCAN(FunctionalState newState);

/**
  * @brief  Set GPIO of External interrupt source
  * @param  newStateIPUP,newStateOD
  * @retval None
  */
void configGPIO_KSCAN(FunctionalState newStateIPUP, FunctionalState newStateOD)
{
    /* Enable KSIN port  input */
    GPIO_SetInput(GPIO_KSIN, GPIO_BIT_ALL);

    /* set KSIN  port  pull up */
    GPIO_SetPullUp(GPIO_KSIN, GPIO_BIT_ALL, newStateIPUP);

    /* Enabole KSOUT port output */
    GPIO_SetOutput(GPIO_KSOUT, GPIO_BIT_ALL);


    GPIO_SetOpenDrain(GPIO_KSOUT, GPIO_BIT_ALL, newStateOD);

    /* set GPIO port  function as KSOUT */
    GPIO_EnableFuncReg(GPIO_KSOUT, KSOUT_FUNC_REG, GPIO_BIT_ALL);
}

/**
  * @brief  Configure the KSCAN
  * @param  None
  * @retval None
  */
void configKSCAN(void)
{

    KSCAN_Enable();

    /* Set KSICR, KSOCR */
    KSCAN_SetStrobeOutput(0xFF);

    /* KSTCR */
    KSCAN_SetStrobeWidth(5U);
    KSCAN_SetChatCancelTime(60U);       /*Sets a chattering cancel time */
    KSCAN_SetConsecutiveMatches(KSCAN_MATCH_2C);
}

/**
  * @brief  Start KSCAN
  * @param  newState
  * @retval None
  */
void startKSCAN(FunctionalState newState)
{
    /* KSINTCR<INTEN> */
    KSCAN_SetINTReq(newState);

    TSB_KS_CR_START = newState;
    while (TSB_KS_CR_START != newState) {
    }
}
